@@ -1,6 +1,6 @@
 #![cfg(feature = "std")]
 
-use papergrid::config::{AlignmentHorizontal, AlignmentVertical, Entity, Formatting};
+use papergrid::config::{AlignmentHorizontal, AlignmentVertical, Entity, Formatting, Position};
 
 use crate::util::grid;
 use testing_table::static_table;
@@ -776,10 +776,16 @@ fn formatting_test() {
     ];
 
     let grid = grid(3, 2)
-        .change_cell((0, 0), "A long string")
-        .change_cell((0, 1), "\n\n\nA\n    string\nwith\n new\nline\n\n\n")
-        .change_cell((2, 0), "A one more\n    string\nwith\n new\nline")
-        .change_cell((2, 1), "...");
+        .change_cell(Position::new(0, 0), "A long string")
+        .change_cell(
+            Position::new(0, 1),
+            "\n\n\nA\n    string\nwith\n new\nline\n\n\n",
+        )
+        .change_cell(
+            Position::new(2, 0),
+            "A one more\n    string\nwith\n new\nline",
+        )
+        .change_cell(Position::new(2, 1), "...");
 
     for (i, test) in tests.iter().enumerate() {
         let table = grid
@@ -787,7 +793,9 @@ fn formatting_test() {
             .config(|cfg| {
                 cfg.set_alignment_horizontal(Entity::Global, test.0);
                 cfg.set_alignment_vertical(Entity::Global, test.1);
-                cfg.set_formatting(Entity::Global, test.2);
+                cfg.set_line_alignment(Entity::Global, test.2.allow_lines_alignment);
+                cfg.set_trim_horizontal(Entity::Global, test.2.horizontal_trim);
+                cfg.set_trim_vertical(Entity::Global, test.2.vertical_trim);
             })
             .clone()
             .build();
@@ -803,7 +811,11 @@ fn formatting_empty_test() {
         let formatting = Formatting::new(true, true, true);
         assert_eq!(
             grid(rows, cols)
-                .config(|cfg| cfg.set_formatting(Entity::Global, formatting))
+                .config(|cfg| {
+                    cfg.set_line_alignment(Entity::Global, formatting.allow_lines_alignment);
+                    cfg.set_trim_horizontal(Entity::Global, formatting.horizontal_trim);
+                    cfg.set_trim_vertical(Entity::Global, formatting.vertical_trim);
+                })
                 .build(),
             ""
         );
@@ -855,7 +867,7 @@ fn formatting_1x1_test() {
 
     assert_eq!(
         grid.clone()
-            .config(|cfg| cfg.set_formatting(Entity::Global, Formatting::new(false, false, true)))
+            .config(|cfg| cfg.set_line_alignment(Entity::Global, true))
             .build(),
         static_table!(
             r#"+--------------------------------------------------+"#
@@ -878,7 +890,8 @@ fn formatting_1x1_test() {
 
     assert_eq!(
         grid.clone()
-            .config(|cfg| cfg.set_formatting(Entity::Global, Formatting::new(true, false, true)))
+            .config(|cfg| cfg.set_line_alignment(Entity::Global, true))
+            .config(|cfg| cfg.set_trim_horizontal(Entity::Global, true))
             .build(),
         static_table!(
             r#"+--------------------------------------------------+"#
@@ -900,7 +913,9 @@ fn formatting_1x1_test() {
     );
 
     assert_eq!(
-        grid.config(|cfg| cfg.set_formatting(Entity::Global, Formatting::new(true, true, true)))
+        grid.config(|cfg| cfg.set_line_alignment(Entity::Global, true))
+            .config(|cfg| cfg.set_trim_horizontal(Entity::Global, true))
+            .config(|cfg| cfg.set_trim_vertical(Entity::Global, true))
             .build(),
         static_table!(
             r#"+--------------------------------------------------+"#
