@@ -73,7 +73,7 @@ fn test_filter_diff_arithmetic() -> PolarsResult<()> {
             * lit(2))
         .alias("diff")])
         .sort(["user"], Default::default())
-        .explode([col("diff")])
+        .explode(cols(["diff"]))
         .collect()?;
 
     let out = out.column("diff")?;
@@ -141,7 +141,7 @@ fn test_logical_mean_partitioned_group_by_block() -> PolarsResult<()> {
 
     let out = df
         .lazy()
-        .with_column(col("decimal").cast(DataType::Decimal(None, Some(2))))
+        .with_column(col("decimal").cast(DataType::Decimal(38, 2)))
         .with_column(col("duration").cast(DataType::Duration(TimeUnit::Microseconds)))
         .group_by([col("decimal")])
         .agg([col("duration").mean()])
