@@ -296,7 +296,11 @@ fn concatenate_view<V: ViewType + ?Sized, A: AsRef<dyn Array>>(
                 .map(|l| if l > 12 { l as usize } else { 0 })
                 .sum::<usize>();
         }
-        let mut new_buffer = Vec::with_capacity(total_buffer_len);
+
+        let mut unprocessed_buffer_len = total_buffer_len;
+        let mut new_buffers: Vec<Vec<u8>> = vec![Vec::with_capacity(
+            unprocessed_buffer_len.min(u32::MAX as usize),
+        )];
         for arr in arrays {
             let arr: &BinaryViewArrayGeneric<V> = arr.as_ref().as_any().downcast_ref().unwrap();
             let buffers = arr.data_buffers();
@@ -305,17 +309,28 @@ fn concatenate_view<V: ViewType + ?Sized, A: AsRef<dyn Array>>(
                 for mut view in arr.views().iter().copied() {
                     total_bytes_len += view.length as usize;
                     if view.length > 12 {
-                        let new_offset = new_buffer.len().try_into().unwrap();
-                        new_buffer.extend_from_slice(view.get_slice_unchecked(buffers));
+                        if new_buffers.last().unwrap_unchecked().len() + view.length as usize
+                            >= u32::MAX as usize
+                        {
+                            new_buffers.push(Vec::with_capacity(
+                                unprocessed_buffer_len.min(u32::MAX as usize),
+                            ));
+                        }
+                        let new_offset = new_buffers.last().unwrap_unchecked().len() as u32;
+                        new_buffers
+                            .last_mut()
+                            .unwrap_unchecked()
+                            .extend_from_slice(view.get_slice_unchecked(buffers));
                         view.offset = new_offset;
-                        view.buffer_idx = 0;
+                        view.buffer_idx = new_buffers.len() as u32 - 1;
+                        unprocessed_buffer_len -= view.length as usize;
                     }
                     views.push_unchecked(view);
                 }
             }
         }
 
-        Arc::new([Buffer::from(new_buffer)]) as Arc<[_]>
+        new_buffers.into_iter().map(Buffer::from).collect()
     };
 
     unsafe {
