@@ -8,13 +8,18 @@ mod schema;
 use std::collections::BTreeMap;
 use std::sync::Arc;
 
-pub use field::{DTYPE_CATEGORICAL, DTYPE_ENUM_VALUES, Field};
+pub use field::{
+    DTYPE_CATEGORICAL_LEGACY, DTYPE_CATEGORICAL_NEW, DTYPE_ENUM_VALUES_LEGACY,
+    DTYPE_ENUM_VALUES_NEW, Field, MAINTAIN_PL_TYPE, PARQUET_EMPTY_STRUCT, PL_KEY,
+};
 pub use physical_type::*;
 use polars_utils::pl_str::PlSmallStr;
 pub use schema::{ArrowSchema, ArrowSchemaRef};
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
 
+use crate::array::LIST_VALUES_NAME;
+
 /// typedef for [BTreeMap<PlSmallStr, PlSmallStr>] denoting [`Field`]'s and [`ArrowSchema`]'s metadata.
 pub type Metadata = BTreeMap<PlSmallStr, PlSmallStr>;
 /// typedef for [Option<(PlSmallStr, Option<PlSmallStr>)>] descr
@@ -55,6 +60,8 @@ pub enum ArrowDataType {
     UInt32,
     /// An [`u64`]
     UInt64,
+    /// An [`u128`]
+    UInt128,
     /// An 16-bit float
     Float16,
     /// A [`f32`]
@@ -251,6 +258,10 @@ pub enum IntervalUnit {
     DayTime,
     /// The number of elapsed months (i32), days (i32) and nanoseconds (i64).
     MonthDayNano,
+    /// `(months: i32, days: i32, milliseconds: i32)`.
+    /// Used when loading the Parquet INTERVAL type. This is expected to be
+    /// unreachable outside of Parquet reading.
+    MonthDayMillis,
 }
 
 impl ArrowDataType {
@@ -280,6 +291,7 @@ impl ArrowDataType {
             Int64 | Date64 | Timestamp(_, _) | Time64(_) | Duration(_) => {
                 PhysicalType::Primitive(PrimitiveType::Int64)
             },
+            Int128 => PhysicalType::Primitive(PrimitiveType::Int128),
             Decimal(_, _) => PhysicalType::Primitive(PrimitiveType::Int128),
             Decimal32(_, _) => PhysicalType::Primitive(PrimitiveType::Int32),
             Decimal64(_, _) => PhysicalType::Primitive(PrimitiveType::Int64),
@@ -288,14 +300,17 @@ impl ArrowDataType {
             UInt16 => PhysicalType::Primitive(PrimitiveType::UInt16),
             UInt32 => PhysicalType::Primitive(PrimitiveType::UInt32),
             UInt64 => PhysicalType::Primitive(PrimitiveType::UInt64),
+            UInt128 => PhysicalType::Primitive(PrimitiveType::UInt128),
             Float16 => PhysicalType::Primitive(PrimitiveType::Float16),
             Float32 => PhysicalType::Primitive(PrimitiveType::Float32),
             Float64 => PhysicalType::Primitive(PrimitiveType::Float64),
-            Int128 => PhysicalType::Primitive(PrimitiveType::Int128),
             Interval(IntervalUnit::DayTime) => PhysicalType::Primitive(PrimitiveType::DaysMs),
             Interval(IntervalUnit::MonthDayNano) => {
                 PhysicalType::Primitive(PrimitiveType::MonthDayNano)
             },
+            Interval(IntervalUnit::MonthDayMillis) => {
+                PhysicalType::Primitive(PrimitiveType::MonthDayMillis)
+            },
             Binary => PhysicalType::Binary,
             FixedSizeBinary(_) => PhysicalType::FixedSizeBinary,
             LargeBinary => PhysicalType::LargeBinary,
@@ -319,14 +334,17 @@ impl ArrowDataType {
     pub fn underlying_physical_type(&self) -> ArrowDataType {
         use ArrowDataType::*;
         match self {
-            Date32 | Time32(_) | Interval(IntervalUnit::YearMonth) => Int32,
-            Date64
+            Decimal32(_, _) | Date32 | Time32(_) | Interval(IntervalUnit::YearMonth) => Int32,
+            Decimal64(_, _)
+            | Date64
             | Timestamp(_, _)
             | Time64(_)
             | Duration(_)
             | Interval(IntervalUnit::DayTime) => Int64,
             Interval(IntervalUnit::MonthDayNano) => unimplemented!(),
             Binary => Binary,
+            Decimal(_, _) => Int128,
+            Decimal256(_, _) => unimplemented!(),
             List(field) => List(Box::new(Field {
                 dtype: field.dtype.underlying_physical_type(),
                 ..*field.clone()
@@ -412,6 +430,7 @@ impl ArrowDataType {
                 | D::UInt16
                 | D::UInt32
                 | D::UInt64
+                | D::UInt128
                 | D::Float32
                 | D::Float64
                 | D::Decimal(_, _)
@@ -421,13 +440,13 @@ impl ArrowDataType {
         )
     }
 
+    pub fn to_large_list(self, is_nullable: bool) -> ArrowDataType {
+        ArrowDataType::LargeList(Box::new(Field::new(LIST_VALUES_NAME, self, is_nullable)))
+    }
+
     pub fn to_fixed_size_list(self, size: usize, is_nullable: bool) -> ArrowDataType {
         ArrowDataType::FixedSizeList(
-            Box::new(Field::new(
-                PlSmallStr::from_static("item"),
-                self,
-                is_nullable,
-            )),
+            Box::new(Field::new(LIST_VALUES_NAME, self, is_nullable)),
             size,
         )
     }
@@ -442,11 +461,12 @@ impl ArrowDataType {
             | D::Int16
             | D::Int32
             | D::Int64
+            | D::Int128
             | D::UInt8
             | D::UInt16
             | D::UInt32
             | D::UInt64
-            | D::Int128
+            | D::UInt128
             | D::Float16
             | D::Float32
             | D::Float64
@@ -493,6 +513,7 @@ impl From<IntegerType> for ArrowDataType {
             IntegerType::UInt16 => ArrowDataType::UInt16,
             IntegerType::UInt32 => ArrowDataType::UInt32,
             IntegerType::UInt64 => ArrowDataType::UInt64,
+            IntegerType::UInt128 => ArrowDataType::UInt128,
         }
     }
 }
@@ -504,18 +525,19 @@ impl From<PrimitiveType> for ArrowDataType {
             PrimitiveType::Int16 => ArrowDataType::Int16,
             PrimitiveType::Int32 => ArrowDataType::Int32,
             PrimitiveType::Int64 => ArrowDataType::Int64,
+            PrimitiveType::Int128 => ArrowDataType::Int128,
             PrimitiveType::UInt8 => ArrowDataType::UInt8,
             PrimitiveType::UInt16 => ArrowDataType::UInt16,
             PrimitiveType::UInt32 => ArrowDataType::UInt32,
             PrimitiveType::UInt64 => ArrowDataType::UInt64,
-            PrimitiveType::Int128 => ArrowDataType::Int128,
+            PrimitiveType::UInt128 => ArrowDataType::UInt128,
             PrimitiveType::Int256 => ArrowDataType::Decimal256(32, 32),
             PrimitiveType::Float16 => ArrowDataType::Float16,
             PrimitiveType::Float32 => ArrowDataType::Float32,
             PrimitiveType::Float64 => ArrowDataType::Float64,
             PrimitiveType::DaysMs => ArrowDataType::Interval(IntervalUnit::DayTime),
             PrimitiveType::MonthDayNano => ArrowDataType::Interval(IntervalUnit::MonthDayNano),
-            PrimitiveType::UInt128 => unimplemented!(),
+            PrimitiveType::MonthDayMillis => ArrowDataType::Interval(IntervalUnit::MonthDayMillis),
         }
     }
 }
