@@ -94,7 +94,7 @@ impl ArrowSchema {
 
         let metadata = if let ArrowDataType::Extension(ext) = field.dtype() {
             // append extension information.
-            let mut metadata = metadata.clone();
+            let mut metadata = metadata;
 
             // metadata
             if let Some(extension_metadata) = &ext.metadata {
@@ -273,6 +273,7 @@ unsafe fn to_dtype(schema: &ArrowSchema) -> PolarsResult<ArrowDataType> {
         "l" => ArrowDataType::Int64,
         "L" => ArrowDataType::UInt64,
         "_pli128" => ArrowDataType::Int128,
+        "_plu128" => ArrowDataType::UInt128,
         "e" => ArrowDataType::Float16,
         "f" => ArrowDataType::Float32,
         "g" => ArrowDataType::Float64,
@@ -292,6 +293,7 @@ unsafe fn to_dtype(schema: &ArrowSchema) -> PolarsResult<ArrowDataType> {
         "tDn" => ArrowDataType::Duration(TimeUnit::Nanosecond),
         "tiM" => ArrowDataType::Interval(IntervalUnit::YearMonth),
         "tiD" => ArrowDataType::Interval(IntervalUnit::DayTime),
+        "tin" => ArrowDataType::Interval(IntervalUnit::MonthDayNano),
         "vu" => ArrowDataType::Utf8View,
         "vz" => ArrowDataType::BinaryView,
         "+l" => {
@@ -462,6 +464,8 @@ fn to_format(dtype: &ArrowDataType) -> String {
         ArrowDataType::UInt64 => "L".to_string(),
         // Doesn't exist in arrow, '_pl' prefixed is Polars specific
         ArrowDataType::Int128 => "_pli128".to_string(),
+        // Doesn't exist in arrow, '_pl' prefixed is Polars specific
+        ArrowDataType::UInt128 => "_plu128".to_string(),
         ArrowDataType::Float16 => "e".to_string(),
         ArrowDataType::Float32 => "f".to_string(),
         ArrowDataType::Float64 => "g".to_string(),
@@ -487,9 +491,8 @@ fn to_format(dtype: &ArrowDataType) -> String {
         ArrowDataType::Duration(TimeUnit::Nanosecond) => "tDn".to_string(),
         ArrowDataType::Interval(IntervalUnit::YearMonth) => "tiM".to_string(),
         ArrowDataType::Interval(IntervalUnit::DayTime) => "tiD".to_string(),
-        ArrowDataType::Interval(IntervalUnit::MonthDayNano) => {
-            todo!("Spec for FFI for MonthDayNano still not defined.")
-        },
+        ArrowDataType::Interval(IntervalUnit::MonthDayNano) => "tin".to_string(),
+        ArrowDataType::Interval(IntervalUnit::MonthDayMillis) => unimplemented!(),
         ArrowDataType::Timestamp(unit, tz) => {
             let unit = match unit {
                 TimeUnit::Second => "s",
@@ -609,6 +612,7 @@ unsafe fn metadata_from_bytes(data: *const ::std::os::raw::c_char) -> (Metadata,
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::array::LIST_VALUES_NAME;
 
     #[test]
     fn test_all() {
@@ -619,10 +623,12 @@ mod tests {
             ArrowDataType::UInt16,
             ArrowDataType::UInt32,
             ArrowDataType::UInt64,
+            ArrowDataType::UInt128,
             ArrowDataType::Int8,
             ArrowDataType::Int16,
             ArrowDataType::Int32,
             ArrowDataType::Int64,
+            ArrowDataType::Int128,
             ArrowDataType::Float32,
             ArrowDataType::Float64,
             ArrowDataType::Date32,
@@ -660,7 +666,7 @@ mod tests {
                 Field::new(
                     PlSmallStr::from_static("b"),
                     ArrowDataType::List(Box::new(Field::new(
-                        PlSmallStr::from_static("item"),
+                        LIST_VALUES_NAME,
                         ArrowDataType::Int32,
                         true,
                     ))),
@@ -681,7 +687,7 @@ mod tests {
                     Field::new(
                         PlSmallStr::from_static("b"),
                         ArrowDataType::List(Box::new(Field::new(
-                            PlSmallStr::from_static("item"),
+                            LIST_VALUES_NAME,
                             ArrowDataType::Int32,
                             true,
                         ))),
@@ -697,7 +703,7 @@ mod tests {
                     Field::new(
                         PlSmallStr::from_static("b"),
                         ArrowDataType::List(Box::new(Field::new(
-                            PlSmallStr::from_static("item"),
+                            LIST_VALUES_NAME,
                             ArrowDataType::Int32,
                             true,
                         ))),
