@@ -4,7 +4,7 @@ mod boolean;
 mod var;
 
 pub use boolean::*;
-use num_traits::{NumCast, ToPrimitive};
+use num_traits::ToPrimitive;
 use polars_utils::IdxSize;
 pub use var::*;
 
@@ -17,36 +17,27 @@ use crate::types::NativeType;
 #[inline]
 pub unsafe fn take_agg_no_null_primitive_iter_unchecked<
     T: NativeType + ToPrimitive,
-    TOut: NumCast + NativeType,
     I: IntoIterator<Item = usize>,
-    F: Fn(TOut, TOut) -> TOut,
 >(
     arr: &PrimitiveArray<T>,
     indices: I,
-    f: F,
-) -> Option<TOut> {
+) -> impl Iterator<Item = T> {
     debug_assert!(arr.null_count() == 0);
     let array_values = arr.values().as_slice();
 
     indices
         .into_iter()
-        .map(|idx| TOut::from(*array_values.get_unchecked(idx)).unwrap_unchecked())
-        .reduce(f)
+        .map(|idx| *array_values.get_unchecked(idx))
 }
 
 /// Take kernel for single chunk and an iterator as index.
 /// # Safety
 /// caller must ensure iterators indexes are in bounds
 #[inline]
-pub unsafe fn take_agg_primitive_iter_unchecked<
-    T: NativeType,
-    I: IntoIterator<Item = usize>,
-    F: Fn(T, T) -> T,
->(
+pub unsafe fn take_agg_primitive_iter_unchecked<T: NativeType, I: IntoIterator<Item = usize>>(
     arr: &PrimitiveArray<T>,
     indices: I,
-    f: F,
-) -> Option<T> {
+) -> impl Iterator<Item = T> {
     let array_values = arr.values().as_slice();
     let validity = arr.validity().unwrap();
 
@@ -54,23 +45,22 @@ pub unsafe fn take_agg_primitive_iter_unchecked<
         .into_iter()
         .filter(|&idx| validity.get_bit_unchecked(idx))
         .map(|idx| *array_values.get_unchecked(idx))
-        .reduce(f)
 }
 
 /// Take kernel for single chunk and an iterator as index.
 /// # Safety
-/// caller must enure iterators indexes are in bounds
+/// caller must ensure iterators indexes are in bounds
 #[inline]
 pub unsafe fn take_agg_primitive_iter_unchecked_count_nulls<
     T: NativeType + ToPrimitive,
-    TOut: NumCast + NativeType,
     I: IntoIterator<Item = usize>,
-    F: Fn(TOut, TOut) -> TOut,
+    TOut,
+    F: Fn(TOut, T) -> TOut,
 >(
     arr: &PrimitiveArray<T>,
     indices: I,
-    f: F,
     init: TOut,
+    f: F,
     len: IdxSize,
 ) -> Option<(TOut, IdxSize)> {
     let array_values = arr.values().as_slice();
@@ -79,10 +69,7 @@ pub unsafe fn take_agg_primitive_iter_unchecked_count_nulls<
     let mut null_count = 0 as IdxSize;
     let out = indices.into_iter().fold(init, |acc, idx| {
         if validity.get_bit_unchecked(idx) {
-            f(
-                acc,
-                NumCast::from(*array_values.get_unchecked(idx)).unwrap_unchecked(),
-            )
+            f(acc, *array_values.get_unchecked(idx))
         } else {
             null_count += 1;
             acc
