@@ -262,15 +262,19 @@ pub struct SkewState {
 
 impl SkewState {
     fn new(x: &[f64]) -> Self {
-        if x.is_empty() {
+        Self::from_iter(x.iter().copied(), x.len())
+    }
+
+    fn from_iter(iter: impl Iterator<Item = f64> + Clone, length: usize) -> Self {
+        if length == 0 {
             return Self::default();
         }
 
-        let weight = x.len() as f64;
-        let mean = alg_sum_f64(x.iter().copied()) / weight;
+        let weight = length as f64;
+        let mean = alg_sum_f64(iter.clone()) / weight;
         let mut m2 = 0.0;
         let mut m3 = 0.0;
-        for xi in x.iter() {
+        for xi in iter {
             let d = xi - mean;
             let d2 = d * d;
             let d3 = d * d2;
@@ -294,6 +298,24 @@ impl SkewState {
         }
     }
 
+    pub fn from_array(arr: &PrimitiveArray<f64>, start: usize, length: usize) -> Self {
+        let validity = arr.validity().cloned();
+        let validity = validity
+            .map(|v| v.sliced(start, length))
+            .filter(|v| v.unset_bits() > 0);
+
+        match validity {
+            None => Self::new(&arr.values().as_slice()[start..][..length]),
+            Some(validity) => {
+                let iter = arr.values()[start..][..length].iter().copied();
+                let iter = iter
+                    .zip(validity.iter())
+                    .filter_map(|(x, v)| v.then_some(x));
+                Self::from_iter(iter, validity.set_bits())
+            },
+        }
+    }
+
     pub fn insert_one(&mut self, x: f64) {
         // Specialization of self.combine(&SkewState { weight: 1.0, mean: x, m2: 0.0, m3: 0.0 });
         let new_weight = self.weight + 1.0;
@@ -394,17 +416,21 @@ pub struct KurtosisState {
 }
 
 impl KurtosisState {
-    fn new(x: &[f64]) -> Self {
-        if x.is_empty() {
+    pub fn new(x: &[f64]) -> Self {
+        Self::from_iter(x.iter().copied(), x.len())
+    }
+
+    pub fn from_iter(iter: impl Iterator<Item = f64> + Clone, length: usize) -> Self {
+        if length == 0 {
             return Self::default();
         }
 
-        let weight = x.len() as f64;
-        let mean = alg_sum_f64(x.iter().copied()) / weight;
+        let weight = length as f64;
+        let mean = alg_sum_f64(iter.clone()) / weight;
         let mut m2 = 0.0;
         let mut m3 = 0.0;
         let mut m4 = 0.0;
-        for xi in x.iter() {
+        for xi in iter {
             let d = xi - mean;
             let d2 = d * d;
             let d3 = d * d2;
@@ -422,6 +448,24 @@ impl KurtosisState {
         }
     }
 
+    pub fn from_array(arr: &PrimitiveArray<f64>, start: usize, length: usize) -> Self {
+        let validity = arr.validity().cloned();
+        let validity = validity
+            .map(|v| v.sliced(start, length))
+            .filter(|v| v.unset_bits() > 0);
+
+        match validity {
+            None => Self::new(&arr.values().as_slice()[start..][..length]),
+            Some(validity) => {
+                let iter = arr.values()[start..][..length].iter().copied();
+                let iter = iter
+                    .zip(validity.iter())
+                    .filter_map(|(x, v)| v.then_some(x));
+                Self::from_iter(iter, validity.set_bits())
+            },
+        }
+    }
+
     fn clear_zero_weight_nan(&mut self) {
         // Clear NaNs due to division by zero.
         if self.weight == 0.0 {
