@@ -466,177 +466,185 @@ impl ChunkCompareIneq<&StringChunked> for StringChunked {
     }
 }
 
-impl ChunkCompareEq<&BinaryChunked> for BinaryChunked {
-    type Item = BooleanChunked;
+macro_rules! binary_eq_ineq_impl {
+    ($($ca:ident),+) => {
+        $(
+        impl ChunkCompareEq<&$ca> for $ca {
+            type Item = BooleanChunked;
 
-    fn equal(&self, rhs: &BinaryChunked) -> BooleanChunked {
-        // Broadcast.
-        match (self.len(), rhs.len()) {
-            (_, 1) => {
-                if let Some(value) = rhs.get(0) {
-                    self.equal(value)
-                } else {
-                    BooleanChunked::full_null(PlSmallStr::EMPTY, self.len())
+            fn equal(&self, rhs: &$ca) -> BooleanChunked {
+                // Broadcast.
+                match (self.len(), rhs.len()) {
+                    (_, 1) => {
+                        if let Some(value) = rhs.get(0) {
+                            self.equal(value)
+                        } else {
+                            BooleanChunked::full_null(PlSmallStr::EMPTY, self.len())
+                        }
+                    },
+                    (1, _) => {
+                        if let Some(value) = self.get(0) {
+                            rhs.equal(value)
+                        } else {
+                            BooleanChunked::full_null(PlSmallStr::EMPTY, rhs.len())
+                        }
+                    },
+                    _ => arity::binary_mut_values(
+                        self,
+                        rhs,
+                        |a, b| a.tot_eq_kernel(b).into(),
+                        PlSmallStr::EMPTY,
+                    ),
                 }
-            },
-            (1, _) => {
-                if let Some(value) = self.get(0) {
-                    rhs.equal(value)
-                } else {
-                    BooleanChunked::full_null(PlSmallStr::EMPTY, rhs.len())
-                }
-            },
-            _ => arity::binary_mut_values(
-                self,
-                rhs,
-                |a, b| a.tot_eq_kernel(b).into(),
-                PlSmallStr::EMPTY,
-            ),
-        }
-    }
+            }
 
-    fn equal_missing(&self, rhs: &BinaryChunked) -> BooleanChunked {
-        // Broadcast.
-        match (self.len(), rhs.len()) {
-            (_, 1) => {
-                if let Some(value) = rhs.get(0) {
-                    self.equal_missing(value)
-                } else {
-                    self.is_null()
+            fn equal_missing(&self, rhs: &$ca) -> BooleanChunked {
+                // Broadcast.
+                match (self.len(), rhs.len()) {
+                    (_, 1) => {
+                        if let Some(value) = rhs.get(0) {
+                            self.equal_missing(value)
+                        } else {
+                            self.is_null()
+                        }
+                    },
+                    (1, _) => {
+                        if let Some(value) = self.get(0) {
+                            rhs.equal_missing(value)
+                        } else {
+                            rhs.is_null()
+                        }
+                    },
+                    _ => arity::binary_mut_with_options(
+                        self,
+                        rhs,
+                        |a, b| a.tot_eq_missing_kernel(b).into(),
+                        PlSmallStr::EMPTY,
+                    ),
                 }
-            },
-            (1, _) => {
-                if let Some(value) = self.get(0) {
-                    rhs.equal_missing(value)
-                } else {
-                    rhs.is_null()
-                }
-            },
-            _ => arity::binary_mut_with_options(
-                self,
-                rhs,
-                |a, b| a.tot_eq_missing_kernel(b).into(),
-                PlSmallStr::EMPTY,
-            ),
-        }
-    }
+            }
 
-    fn not_equal(&self, rhs: &BinaryChunked) -> BooleanChunked {
-        // Broadcast.
-        match (self.len(), rhs.len()) {
-            (_, 1) => {
-                if let Some(value) = rhs.get(0) {
-                    self.not_equal(value)
-                } else {
-                    BooleanChunked::full_null(PlSmallStr::EMPTY, self.len())
+            fn not_equal(&self, rhs: &$ca) -> BooleanChunked {
+                // Broadcast.
+                match (self.len(), rhs.len()) {
+                    (_, 1) => {
+                        if let Some(value) = rhs.get(0) {
+                            self.not_equal(value)
+                        } else {
+                            BooleanChunked::full_null(PlSmallStr::EMPTY, self.len())
+                        }
+                    },
+                    (1, _) => {
+                        if let Some(value) = self.get(0) {
+                            rhs.not_equal(value)
+                        } else {
+                            BooleanChunked::full_null(PlSmallStr::EMPTY, rhs.len())
+                        }
+                    },
+                    _ => arity::binary_mut_values(
+                        self,
+                        rhs,
+                        |a, b| a.tot_ne_kernel(b).into(),
+                        PlSmallStr::EMPTY,
+                    ),
                 }
-            },
-            (1, _) => {
-                if let Some(value) = self.get(0) {
-                    rhs.not_equal(value)
-                } else {
-                    BooleanChunked::full_null(PlSmallStr::EMPTY, rhs.len())
-                }
-            },
-            _ => arity::binary_mut_values(
-                self,
-                rhs,
-                |a, b| a.tot_ne_kernel(b).into(),
-                PlSmallStr::EMPTY,
-            ),
-        }
-    }
+            }
 
-    fn not_equal_missing(&self, rhs: &BinaryChunked) -> BooleanChunked {
-        // Broadcast.
-        match (self.len(), rhs.len()) {
-            (_, 1) => {
-                if let Some(value) = rhs.get(0) {
-                    self.not_equal_missing(value)
-                } else {
-                    self.is_not_null()
+            fn not_equal_missing(&self, rhs: &$ca) -> BooleanChunked {
+                // Broadcast.
+                match (self.len(), rhs.len()) {
+                    (_, 1) => {
+                        if let Some(value) = rhs.get(0) {
+                            self.not_equal_missing(value)
+                        } else {
+                            self.is_not_null()
+                        }
+                    },
+                    (1, _) => {
+                        if let Some(value) = self.get(0) {
+                            rhs.not_equal_missing(value)
+                        } else {
+                            rhs.is_not_null()
+                        }
+                    },
+                    _ => arity::binary_mut_with_options(
+                        self,
+                        rhs,
+                        |a, b| a.tot_ne_missing_kernel(b).into(),
+                        PlSmallStr::EMPTY,
+                    ),
                 }
-            },
-            (1, _) => {
-                if let Some(value) = self.get(0) {
-                    rhs.not_equal_missing(value)
-                } else {
-                    rhs.is_not_null()
-                }
-            },
-            _ => arity::binary_mut_with_options(
-                self,
-                rhs,
-                |a, b| a.tot_ne_missing_kernel(b).into(),
-                PlSmallStr::EMPTY,
-            ),
+            }
         }
-    }
+
+        impl ChunkCompareIneq<&$ca> for $ca {
+            type Item = BooleanChunked;
+
+            fn lt(&self, rhs: &$ca) -> BooleanChunked {
+                // Broadcast.
+                match (self.len(), rhs.len()) {
+                    (_, 1) => {
+                        if let Some(value) = rhs.get(0) {
+                            self.lt(value)
+                        } else {
+                            BooleanChunked::full_null(PlSmallStr::EMPTY, self.len())
+                        }
+                    },
+                    (1, _) => {
+                        if let Some(value) = self.get(0) {
+                            rhs.gt(value)
+                        } else {
+                            BooleanChunked::full_null(PlSmallStr::EMPTY, rhs.len())
+                        }
+                    },
+                    _ => arity::binary_mut_values(
+                        self,
+                        rhs,
+                        |a, b| a.tot_lt_kernel(b).into(),
+                        PlSmallStr::EMPTY,
+                    ),
+                }
+            }
+
+            fn lt_eq(&self, rhs: &$ca) -> BooleanChunked {
+                // Broadcast.
+                match (self.len(), rhs.len()) {
+                    (_, 1) => {
+                        if let Some(value) = rhs.get(0) {
+                            self.lt_eq(value)
+                        } else {
+                            BooleanChunked::full_null(PlSmallStr::EMPTY, self.len())
+                        }
+                    },
+                    (1, _) => {
+                        if let Some(value) = self.get(0) {
+                            rhs.gt_eq(value)
+                        } else {
+                            BooleanChunked::full_null(PlSmallStr::EMPTY, rhs.len())
+                        }
+                    },
+                    _ => arity::binary_mut_values(
+                        self,
+                        rhs,
+                        |a, b| a.tot_le_kernel(b).into(),
+                        PlSmallStr::EMPTY,
+                    ),
+                }
+            }
+
+            fn gt(&self, rhs: &Self) -> BooleanChunked {
+                rhs.lt(self)
+            }
+
+            fn gt_eq(&self, rhs: &Self) -> BooleanChunked {
+                rhs.lt_eq(self)
+            }
+        }
+        )+
+    };
 }
 
-impl ChunkCompareIneq<&BinaryChunked> for BinaryChunked {
-    type Item = BooleanChunked;
-
-    fn lt(&self, rhs: &BinaryChunked) -> BooleanChunked {
-        // Broadcast.
-        match (self.len(), rhs.len()) {
-            (_, 1) => {
-                if let Some(value) = rhs.get(0) {
-                    self.lt(value)
-                } else {
-                    BooleanChunked::full_null(PlSmallStr::EMPTY, self.len())
-                }
-            },
-            (1, _) => {
-                if let Some(value) = self.get(0) {
-                    rhs.gt(value)
-                } else {
-                    BooleanChunked::full_null(PlSmallStr::EMPTY, rhs.len())
-                }
-            },
-            _ => arity::binary_mut_values(
-                self,
-                rhs,
-                |a, b| a.tot_lt_kernel(b).into(),
-                PlSmallStr::EMPTY,
-            ),
-        }
-    }
-
-    fn lt_eq(&self, rhs: &BinaryChunked) -> BooleanChunked {
-        // Broadcast.
-        match (self.len(), rhs.len()) {
-            (_, 1) => {
-                if let Some(value) = rhs.get(0) {
-                    self.lt_eq(value)
-                } else {
-                    BooleanChunked::full_null(PlSmallStr::EMPTY, self.len())
-                }
-            },
-            (1, _) => {
-                if let Some(value) = self.get(0) {
-                    rhs.gt_eq(value)
-                } else {
-                    BooleanChunked::full_null(PlSmallStr::EMPTY, rhs.len())
-                }
-            },
-            _ => arity::binary_mut_values(
-                self,
-                rhs,
-                |a, b| a.tot_le_kernel(b).into(),
-                PlSmallStr::EMPTY,
-            ),
-        }
-    }
-
-    fn gt(&self, rhs: &Self) -> BooleanChunked {
-        rhs.lt(self)
-    }
-
-    fn gt_eq(&self, rhs: &Self) -> BooleanChunked {
-        rhs.lt_eq(self)
-    }
-}
+binary_eq_ineq_impl!(BinaryChunked, BinaryOffsetChunked);
 
 fn _list_comparison_helper<F, B>(
     lhs: &ListChunked,
@@ -843,9 +851,12 @@ where
             let mut a = a.into_owned();
             a.zip_outer_validity(&b);
             unsafe {
+                let mut new_null_count = 0;
                 for (arr, a) in out.downcast_iter_mut().zip(a.downcast_iter()) {
-                    arr.set_validity(a.validity().cloned())
+                    arr.set_validity(a.validity().cloned());
+                    new_null_count += arr.null_count();
                 }
+                out.set_null_count(new_null_count);
             }
         }
 
