@@ -46,14 +46,7 @@ impl Default for RollingOptionsFixedWindow {
 
 #[cfg(feature = "rolling_window")]
 mod inner_mod {
-    use std::ops::SubAssign;
-
-    use arrow::bitmap::MutableBitmap;
-    use arrow::bitmap::utils::set_bit_unchecked;
-    use arrow::legacy::trusted_len::TrustedLenPush;
-    use num_traits::pow::Pow;
-    use num_traits::{Float, Zero};
-    use polars_utils::float::IsFloat;
+    use num_traits::Zero;
 
     use crate::chunked_array::cast::CastOptions;
     use crate::prelude::*;
@@ -87,7 +80,7 @@ mod inner_mod {
         /// Apply a rolling custom function. This is pretty slow because of dynamic dispatch.
         fn rolling_map(
             &self,
-            f: &dyn Fn(&Series) -> Series,
+            f: &dyn Fn(&Series) -> PolarsResult<Series>,
             mut options: RollingOptionsFixedWindow,
         ) -> PolarsResult<Series> {
             check_input(options.window_size, options.min_periods)?;
@@ -144,23 +137,36 @@ mod inner_mod {
                         // ensure the length is correct
                         series_container._get_inner_mut().compute_len();
                         let s = if size == options.window_size {
-                            f(&series_container.multiply(&weights_series).unwrap())
+                            f(&series_container.multiply(&weights_series).unwrap())?
                         } else {
+                            // Determine which side to slice weights from
                             let weights_cutoff: Series = match self.dtype() {
-                                DataType::Float64 => weights_series
-                                    .f64()
-                                    .unwrap()
-                                    .into_iter()
-                                    .take(series_container.len())
-                                    .collect(),
-                                _ => weights_series // Float32 case
-                                    .f32()
-                                    .unwrap()
-                                    .into_iter()
-                                    .take(series_container.len())
-                                    .collect(),
+                                DataType::Float64 => {
+                                    let ws = weights_series.f64().unwrap();
+                                    if start == 0 {
+                                        ws.slice(
+                                            (ws.len() - series_container.len()) as i64,
+                                            series_container.len(),
+                                        )
+                                        .into_series()
+                                    } else {
+                                        ws.slice(0, series_container.len()).into_series()
+                                    }
+                                },
+                                _ => {
+                                    let ws = weights_series.f32().unwrap();
+                                    if start == 0 {
+                                        ws.slice(
+                                            (ws.len() - series_container.len()) as i64,
+                                            series_container.len(),
+                                        )
+                                        .into_series()
+                                    } else {
+                                        ws.slice(0, series_container.len()).into_series()
+                                    }
+                                },
                             };
-                            f(&series_container.multiply(&weights_cutoff).unwrap())
+                            f(&series_container.multiply(&weights_cutoff).unwrap())?
                         };
 
                         let out = self.unpack_series_matching_type(&s)?;
@@ -197,7 +203,7 @@ mod inner_mod {
                         series_container.clear_flags();
                         // ensure the length is correct
                         series_container._get_inner_mut().compute_len();
-                        let s = f(&series_container);
+                        let s = f(&series_container)?;
                         let out = self.unpack_series_matching_type(&s)?;
                         builder.append_option(out.get(0));
                     }
@@ -207,73 +213,4 @@ mod inner_mod {
             }
         }
     }
-
-    impl<T> ChunkedArray<T>
-    where
-        T: PolarsFloatType,
-        T::Native: Float + IsFloat + SubAssign + Pow<T::Native, Output = T::Native>,
-    {
-        /// Apply a rolling custom function. This is pretty slow because of dynamic dispatch.
-        pub fn rolling_map_float<F>(&self, window_size: usize, mut f: F) -> PolarsResult<Self>
-        where
-            F: FnMut(&mut ChunkedArray<T>) -> Option<T::Native>,
-        {
-            if window_size > self.len() {
-                return Ok(Self::full_null(self.name().clone(), self.len()));
-            }
-            let ca = self.rechunk();
-            let arr = ca.downcast_as_array();
-
-            // We create a temporary dummy ChunkedArray. This will be a
-            // container where we swap the window contents every iteration doing
-            // so will save a lot of heap allocations.
-            let mut heap_container =
-                ChunkedArray::<T>::from_slice(PlSmallStr::EMPTY, &[T::Native::zero()]);
-            let ptr = heap_container.chunks[0].as_mut() as *mut dyn Array
-                as *mut PrimitiveArray<T::Native>;
-
-            let mut validity = MutableBitmap::with_capacity(ca.len());
-            validity.extend_constant(window_size - 1, false);
-            validity.extend_constant(ca.len() - (window_size - 1), true);
-            let validity_slice = validity.as_mut_slice();
-
-            let mut values = Vec::with_capacity(ca.len());
-            values.extend(std::iter::repeat_n(T::Native::default(), window_size - 1));
-
-            for offset in 0..self.len() + 1 - window_size {
-                debug_assert!(offset + window_size <= arr.len());
-                let arr_window = unsafe { arr.slice_typed_unchecked(offset, window_size) };
-                // The lengths are cached, so we must update them.
-                heap_container.length = arr_window.len();
-
-                // SAFETY: ptr is not dropped as we are in scope. We are also the only
-                // owner of the contents of the Arc (we do this to reduce heap allocs).
-                unsafe {
-                    *ptr = arr_window;
-                }
-
-                let out = f(&mut heap_container);
-                match out {
-                    Some(v) => {
-                        // SAFETY: we have pre-allocated.
-                        unsafe { values.push_unchecked(v) }
-                    },
-                    None => {
-                        // SAFETY: we allocated enough for both the `values` vec
-                        // and the `validity_ptr`.
-                        unsafe {
-                            values.push_unchecked(T::Native::default());
-                            set_bit_unchecked(validity_slice, offset + window_size - 1, false);
-                        }
-                    },
-                }
-            }
-            let arr = PrimitiveArray::new(
-                T::get_static_dtype().to_arrow(CompatLevel::newest()),
-                values.into(),
-                Some(validity.into()),
-            );
-            Ok(Self::with_chunk(self.name().clone(), arr))
-        }
-    }
 }
