@@ -1,11 +1,7 @@
 use std::borrow::Cow;
 
 use arrow::compute::utils::combine_validities_and_many;
-use polars_row::{
-    RowEncodingCategoricalContext, RowEncodingContext, RowEncodingOptions, RowsEncoded,
-    convert_columns,
-};
-use polars_utils::itertools::Itertools;
+use polars_row::{RowEncodingContext, RowEncodingOptions, RowsEncoded, convert_columns};
 use rayon::prelude::*;
 
 use crate::POOL;
@@ -76,13 +72,14 @@ pub fn encode_rows_vertical_par_unordered_broadcast_nulls(
 ///
 /// This should be given the logical type in order to communicate Polars datatype information down
 /// into the row encoding / decoding.
-pub fn get_row_encoding_context(dtype: &DataType, ordered: bool) -> Option<RowEncodingContext> {
+pub fn get_row_encoding_context(dtype: &DataType) -> Option<RowEncodingContext> {
     match dtype {
         DataType::Boolean
         | DataType::UInt8
         | DataType::UInt16
         | DataType::UInt32
         | DataType::UInt64
+        | DataType::UInt128
         | DataType::Int8
         | DataType::Int16
         | DataType::Int32
@@ -99,97 +96,35 @@ pub fn get_row_encoding_context(dtype: &DataType, ordered: bool) -> Option<RowEn
         | DataType::Datetime(_, _)
         | DataType::Duration(_) => None,
 
+        #[cfg(feature = "dtype-categorical")]
+        DataType::Categorical(_, mapping) | DataType::Enum(_, mapping) => {
+            use polars_row::RowEncodingCategoricalContext;
+
+            Some(RowEncodingContext::Categorical(
+                RowEncodingCategoricalContext {
+                    is_enum: matches!(dtype, DataType::Enum(_, _)),
+                    mapping: mapping.clone(),
+                },
+            ))
+        },
+
         DataType::Unknown(_) => panic!("Unsupported in row encoding"),
 
         #[cfg(feature = "object")]
         DataType::Object(_) => panic!("Unsupported in row encoding"),
 
         #[cfg(feature = "dtype-decimal")]
-        DataType::Decimal(precision, _) => {
-            Some(RowEncodingContext::Decimal(precision.unwrap_or(38)))
-        },
+        DataType::Decimal(precision, _) => Some(RowEncodingContext::Decimal(*precision)),
 
         #[cfg(feature = "dtype-array")]
-        DataType::Array(dtype, _) => get_row_encoding_context(dtype, ordered),
-        DataType::List(dtype) => get_row_encoding_context(dtype, ordered),
-        #[cfg(feature = "dtype-categorical")]
-        DataType::Categorical(revmap, ordering) | DataType::Enum(revmap, ordering) => {
-            let is_enum = dtype.is_enum();
-            let ctx = match revmap {
-                Some(revmap) => {
-                    let (num_known_categories, lexical_sort_idxs) = match revmap.as_ref() {
-                        RevMapping::Global(map, _, _) => {
-                            let num_known_categories =
-                                map.keys().max().copied().map_or(0, |m| m + 1);
-
-                            // @TODO: This should probably be cached.
-                            let lexical_sort_idxs = (ordered
-                                && matches!(ordering, CategoricalOrdering::Lexical))
-                            .then(|| {
-                                let read_map = crate::STRING_CACHE.read_map();
-                                let payloads = read_map.get_current_payloads();
-                                assert!(payloads.len() >= num_known_categories as usize);
-
-                                let mut idxs = (0..num_known_categories).collect::<Vec<u32>>();
-                                idxs.sort_by_key(|&k| payloads[k as usize].as_str());
-                                let mut sort_idxs = vec![0; num_known_categories as usize];
-                                for (i, idx) in idxs.into_iter().enumerate_u32() {
-                                    sort_idxs[idx as usize] = i;
-                                }
-                                sort_idxs
-                            });
-
-                            (num_known_categories, lexical_sort_idxs)
-                        },
-                        RevMapping::Local(values, _) => {
-                            // @TODO: This should probably be cached.
-                            let lexical_sort_idxs = (ordered
-                                && matches!(ordering, CategoricalOrdering::Lexical))
-                            .then(|| {
-                                assert_eq!(values.null_count(), 0);
-                                let values: Vec<&str> = values.values_iter().collect();
-
-                                let mut idxs = (0..values.len() as u32).collect::<Vec<u32>>();
-                                idxs.sort_by_key(|&k| values[k as usize]);
-                                let mut sort_idxs = vec![0; values.len()];
-                                for (i, idx) in idxs.into_iter().enumerate_u32() {
-                                    sort_idxs[idx as usize] = i;
-                                }
-                                sort_idxs
-                            });
-
-                            (values.len() as u32, lexical_sort_idxs)
-                        },
-                    };
-
-                    RowEncodingCategoricalContext {
-                        num_known_categories,
-                        is_enum,
-                        lexical_sort_idxs,
-                    }
-                },
-                None => {
-                    let num_known_categories = u32::MAX;
-
-                    if matches!(ordering, CategoricalOrdering::Lexical) && ordered {
-                        panic!("lexical ordering not yet supported if rev-map not given");
-                    }
-                    RowEncodingCategoricalContext {
-                        num_known_categories,
-                        is_enum,
-                        lexical_sort_idxs: None,
-                    }
-                },
-            };
-
-            Some(RowEncodingContext::Categorical(ctx))
-        },
+        DataType::Array(dtype, _) => get_row_encoding_context(dtype),
+        DataType::List(dtype) => get_row_encoding_context(dtype),
         #[cfg(feature = "dtype-struct")]
         DataType::Struct(fs) => {
             let mut ctxts = Vec::new();
 
             for (i, f) in fs.iter().enumerate() {
-                if let Some(ctxt) = get_row_encoding_context(f.dtype(), ordered) {
+                if let Some(ctxt) = get_row_encoding_context(f.dtype()) {
                     ctxts.reserve(fs.len());
                     ctxts.extend(std::iter::repeat_n(None, i));
                     ctxts.push(Some(ctxt));
@@ -204,7 +139,7 @@ pub fn get_row_encoding_context(dtype: &DataType, ordered: bool) -> Option<RowEn
             ctxts.extend(
                 fs[ctxts.len()..]
                     .iter()
-                    .map(|f| get_row_encoding_context(f.dtype(), ordered)),
+                    .map(|f| get_row_encoding_context(f.dtype())),
             );
 
             Some(RowEncodingContext::Struct(ctxts))
@@ -239,7 +174,7 @@ pub fn _get_rows_encoded_unordered(by: &[Column]) -> PolarsResult<RowsEncoded> {
         let by = by.as_materialized_series();
         let arr = by.to_physical_repr().rechunk().chunks()[0].to_boxed();
         let opt = RowEncodingOptions::new_unsorted();
-        let ctxt = get_row_encoding_context(by.dtype(), false);
+        let ctxt = get_row_encoding_context(by.dtype());
 
         cols.push(arr);
         opts.push(opt);
@@ -274,7 +209,7 @@ pub fn _get_rows_encoded(
         let by = by.as_materialized_series();
         let arr = by.to_physical_repr().rechunk().chunks()[0].to_boxed();
         let opt = RowEncodingOptions::new_sorted(*desc, *null_last);
-        let ctxt = get_row_encoding_context(by.dtype(), true);
+        let ctxt = get_row_encoding_context(by.dtype());
 
         cols.push(arr);
         opts.push(opt);
@@ -308,3 +243,54 @@ pub fn _get_rows_encoded_ca_unordered(
     _get_rows_encoded_unordered(by)
         .map(|rows| BinaryOffsetChunked::with_chunk(name, rows.into_array()))
 }
+
+#[cfg(feature = "dtype-struct")]
+pub fn row_encoding_decode(
+    ca: &BinaryOffsetChunked,
+    fields: &[Field],
+    opts: &[RowEncodingOptions],
+) -> PolarsResult<StructChunked> {
+    let (ctxts, dtypes) = fields
+        .iter()
+        .map(|f| {
+            (
+                get_row_encoding_context(f.dtype()),
+                f.dtype().to_physical().to_arrow(CompatLevel::newest()),
+            )
+        })
+        .collect::<(Vec<_>, Vec<_>)>();
+
+    let struct_arrow_dtype = ArrowDataType::Struct(
+        fields
+            .iter()
+            .map(|v| v.to_physical().to_arrow(CompatLevel::newest()))
+            .collect(),
+    );
+
+    let mut rows = Vec::new();
+    let chunks = ca
+        .downcast_iter()
+        .map(|array| {
+            let decoded_arrays = unsafe {
+                polars_row::decode::decode_rows_from_binary(array, opts, &ctxts, &dtypes, &mut rows)
+            };
+            assert_eq!(decoded_arrays.len(), fields.len());
+
+            StructArray::new(
+                struct_arrow_dtype.clone(),
+                array.len(),
+                decoded_arrays,
+                None,
+            )
+            .to_boxed()
+        })
+        .collect::<Vec<_>>();
+
+    Ok(unsafe {
+        StructChunked::from_chunks_and_dtype(
+            ca.name().clone(),
+            chunks,
+            DataType::Struct(fields.to_vec()),
+        )
+    })
+}
