@@ -1,4 +1,7 @@
-use arrow::datatypes::{DTYPE_ENUM_VALUES, Metadata};
+use arrow::datatypes::{IntervalUnit, Metadata};
+use polars_dtype::categorical::CategoricalPhysical;
+use polars_error::feature_gated;
+use polars_utils::check_allow_importing_interval_as_struct;
 use polars_utils::pl_str::PlSmallStr;
 
 use super::*;
@@ -120,6 +123,13 @@ impl Field {
     pub fn to_arrow(&self, compat_level: CompatLevel) -> ArrowField {
         self.dtype.to_arrow_field(self.name.clone(), compat_level)
     }
+
+    pub fn to_physical(&self) -> Field {
+        Self {
+            name: self.name.clone(),
+            dtype: self.dtype().to_physical(),
+        }
+    }
 }
 
 impl AsRef<DataType> for Field {
@@ -154,6 +164,8 @@ impl DataType {
             ArrowDataType::UInt16 => DataType::UInt16,
             ArrowDataType::UInt32 => DataType::UInt32,
             ArrowDataType::UInt64 => DataType::UInt64,
+            #[cfg(feature = "dtype-u128")]
+            ArrowDataType::UInt128 => DataType::UInt128,
             ArrowDataType::Int8 => DataType::Int8,
             ArrowDataType::Int16 => DataType::Int16,
             ArrowDataType::Int32 => DataType::Int32,
@@ -178,42 +190,58 @@ impl DataType {
             ArrowDataType::Duration(tu) => DataType::Duration(tu.into()),
             ArrowDataType::Date64 => DataType::Datetime(TimeUnit::Milliseconds, None),
             ArrowDataType::Time64(_) | ArrowDataType::Time32(_) => DataType::Time,
+
             #[cfg(feature = "dtype-categorical")]
             ArrowDataType::Dictionary(_, value_type, _) => {
-                if md.map(|md| md.is_enum()).unwrap_or(false) {
-                    let md = md.unwrap();
-                    let encoded = md.get(DTYPE_ENUM_VALUES).unwrap();
-                    let mut encoded = encoded.as_str();
-                    let mut cats = MutableBinaryViewArray::<str>::new();
+                // The metadata encoding here must match DataType::to_arrow_field.
+                if let Some(mut enum_md) = md.and_then(|md| md.pl_enum_metadata()) {
+                    let cats = move || {
+                        if enum_md.is_empty() {
+                            return None;
+                        }
 
-                    // Data is encoded as <len in ascii><sep ';'><payload>
-                    // We know thus that len is only [0-9] and the first ';' doesn't belong to the
-                    // payload.
-                    while let Some(pos) = encoded.find(';') {
-                        let (len, remainder) = encoded.split_at(pos);
-                        // Split off ';'
-                        encoded = &remainder[1..];
+                        let len;
+                        (len, enum_md) = enum_md.split_once(';').unwrap();
                         let len = len.parse::<usize>().unwrap();
+                        let cat;
+                        (cat, enum_md) = enum_md.split_at(len);
+                        Some(cat)
+                    };
 
-                        let (value, remainder) = encoded.split_at(len);
-                        cats.push_value(value);
-                        encoded = remainder;
-                    }
-                    DataType::Enum(
-                        Some(Arc::new(RevMapping::build_local(cats.into()))),
-                        Default::default(),
-                    )
-                } else if let Some(ordering) = md.and_then(|md| md.categorical()) {
-                    DataType::Categorical(None, ordering)
+                    let fcats = FrozenCategories::new(std::iter::from_fn(cats)).unwrap();
+                    DataType::from_frozen_categories(fcats)
+                } else if let Some(mut cat_md) = md.and_then(|md| md.pl_categorical_metadata()) {
+                    let name_len;
+                    (name_len, cat_md) = cat_md.split_once(';').unwrap();
+                    let name_len = name_len.parse::<usize>().unwrap();
+                    let name;
+                    (name, cat_md) = cat_md.split_at(name_len);
+
+                    let namespace_len;
+                    (namespace_len, cat_md) = cat_md.split_once(';').unwrap();
+                    let namespace_len = namespace_len.parse::<usize>().unwrap();
+                    let namespace;
+                    (namespace, cat_md) = cat_md.split_at(namespace_len);
+
+                    let (physical, _rest) = cat_md.split_once(';').unwrap();
+
+                    let physical: CategoricalPhysical = physical.parse().ok().unwrap();
+                    let cats = Categories::new(
+                        PlSmallStr::from_str(name),
+                        PlSmallStr::from_str(namespace),
+                        physical,
+                    );
+                    DataType::from_categories(cats)
                 } else if matches!(
                     value_type.as_ref(),
                     ArrowDataType::Utf8 | ArrowDataType::LargeUtf8 | ArrowDataType::Utf8View
                 ) {
-                    DataType::Categorical(None, Default::default())
+                    DataType::from_categories(Categories::global())
                 } else {
                     Self::from_arrow(value_type, None)
                 }
             },
+
             #[cfg(feature = "dtype-struct")]
             ArrowDataType::Struct(fields) => {
                 DataType::Struct(fields.iter().map(|fld| fld.into()).collect())
@@ -233,9 +261,7 @@ impl DataType {
                 }
             },
             #[cfg(feature = "dtype-decimal")]
-            ArrowDataType::Decimal(precision, scale) => {
-                DataType::Decimal(Some(*precision), Some(*scale))
-            },
+            ArrowDataType::Decimal(precision, scale) => DataType::Decimal(*precision, *scale),
             ArrowDataType::Utf8View | ArrowDataType::LargeUtf8 | ArrowDataType::Utf8 => {
                 DataType::String
             },
@@ -253,6 +279,14 @@ impl DataType {
             ArrowDataType::Map(inner, _is_sorted) => {
                 DataType::List(Self::from_arrow_field(inner).boxed())
             },
+            ArrowDataType::Interval(IntervalUnit::MonthDayNano) => {
+                check_allow_importing_interval_as_struct("month_day_nano_interval").unwrap();
+                feature_gated!("dtype-struct", DataType::_month_days_ns_struct_type())
+            },
+            ArrowDataType::Interval(IntervalUnit::MonthDayMillis) => {
+                check_allow_importing_interval_as_struct("month_day_millisecond_interval").unwrap();
+                feature_gated!("dtype-struct", DataType::_month_days_ns_struct_type())
+            },
             dt => panic!(
                 "Arrow datatype {dt:?} not supported by Polars. \
                 You probably need to activate that data-type feature."
