@@ -16,7 +16,6 @@ pub mod aggregations;
 pub mod expr;
 pub(crate) mod hashing;
 mod into_groups;
-mod perfect;
 mod position;
 
 pub use into_groups::*;
@@ -32,7 +31,7 @@ impl DataFrame {
         mut by: Vec<Column>,
         multithreaded: bool,
         sorted: bool,
-    ) -> PolarsResult<GroupBy> {
+    ) -> PolarsResult<GroupBy<'_>> {
         polars_ensure!(
             !by.is_empty(),
             ComputeError: "at least one key is required in a group_by operation"
@@ -89,7 +88,7 @@ impl DataFrame {
                 };
                 Ok(GroupsType::Slice {
                     groups,
-                    rolling: false,
+                    overlapping: false,
                 })
             } else {
                 let rows = if multithreaded {
@@ -116,7 +115,7 @@ impl DataFrame {
     ///     .sum()
     /// }
     /// ```
-    pub fn group_by<I, S>(&self, by: I) -> PolarsResult<GroupBy>
+    pub fn group_by<I, S>(&self, by: I) -> PolarsResult<GroupBy<'_>>
     where
         I: IntoIterator<Item = S>,
         S: Into<PlSmallStr>,
@@ -127,7 +126,7 @@ impl DataFrame {
 
     /// Group DataFrame using a Series column.
     /// The groups are ordered by their smallest row index.
-    pub fn group_by_stable<I, S>(&self, by: I) -> PolarsResult<GroupBy>
+    pub fn group_by_stable<I, S>(&self, by: I) -> PolarsResult<GroupBy<'_>>
     where
         I: IntoIterator<Item = S>,
         S: Into<PlSmallStr>,
@@ -275,8 +274,11 @@ impl<'a> GroupBy<'a> {
                             };
                             out
                         },
-                        GroupsType::Slice { groups, rolling } => {
-                            if *rolling && !groups.is_empty() {
+                        GroupsType::Slice {
+                            groups,
+                            overlapping,
+                        } => {
+                            if *overlapping && !groups.is_empty() {
                                 // Groups can be sliced.
                                 let offset = groups[0][0];
                                 let [upper_offset, upper_len] = groups[groups.len() - 1];
@@ -848,7 +850,7 @@ impl<'a> GroupBy<'a> {
         match slice {
             None => self,
             Some((offset, length)) => {
-                self.groups = (self.groups.slice(offset, length)).clone();
+                self.groups = self.groups.slice(offset, length);
                 self.selected_keys = self.keys_sliced(slice);
                 self
             },
@@ -873,6 +875,7 @@ pub enum GroupByMethod {
     Mean,
     First,
     Last,
+    Item { allow_empty: bool },
     Sum,
     Groups,
     NUnique,
@@ -895,6 +898,7 @@ impl Display for GroupByMethod {
             Mean => "mean",
             First => "first",
             Last => "last",
+            Item { .. } => "item",
             Sum => "sum",
             Groups => "groups",
             NUnique => "n_unique",
@@ -920,6 +924,7 @@ pub fn fmt_group_by_column(name: &str, method: GroupByMethod) -> PlSmallStr {
         Mean => format_pl_smallstr!("{name}_mean"),
         First => format_pl_smallstr!("{name}_first"),
         Last => format_pl_smallstr!("{name}_last"),
+        Item { .. } => format_pl_smallstr!("{name}_item"),
         Sum => format_pl_smallstr!("{name}_sum"),
         Groups => PlSmallStr::from_static("groups"),
         NUnique => format_pl_smallstr!("{name}_n_unique"),
@@ -1118,7 +1123,7 @@ mod test {
         .unwrap();
 
         df.apply("foo", |s| {
-            s.cast(&DataType::Categorical(None, Default::default()))
+            s.cast(&DataType::from_categories(Categories::global()))
                 .unwrap()
         })
         .unwrap();
@@ -1199,7 +1204,7 @@ mod test {
         ]?;
 
         df.try_apply("g", |s| {
-            s.cast(&DataType::Categorical(None, Default::default()))
+            s.cast(&DataType::from_categories(Categories::global()))
         })?;
 
         // Use of deprecated `sum()` for testing purposes
