@@ -1,8 +1,9 @@
-use std::borrow::Cow;
 use std::fmt::Write;
 
 use arrow::bitmap::MutableBitmap;
 
+#[cfg(feature = "dtype-categorical")]
+use crate::chunked_array::builder::CategoricalChunkedBuilder;
 use crate::chunked_array::builder::{AnonymousOwnedListBuilder, get_list_builder};
 use crate::prelude::*;
 use crate::utils::any_values_to_supertype;
@@ -22,48 +23,6 @@ impl<'a, T: AsRef<[AnyValue<'a>]>> NamedFrom<T, [AnyValue<'a>]> for Series {
     }
 }
 
-fn initialize_empty_categorical_revmap_rec(dtype: &DataType) -> Cow<DataType> {
-    use DataType as T;
-    match dtype {
-        #[cfg(feature = "dtype-categorical")]
-        T::Categorical(None, o) => {
-            Cow::Owned(T::Categorical(Some(Arc::new(RevMapping::default())), *o))
-        },
-        T::List(inner_dtype) => match initialize_empty_categorical_revmap_rec(inner_dtype) {
-            Cow::Owned(inner_dtype) => Cow::Owned(T::List(Box::new(inner_dtype))),
-            _ => Cow::Borrowed(dtype),
-        },
-        #[cfg(feature = "dtype-array")]
-        T::Array(inner_dtype, width) => {
-            match initialize_empty_categorical_revmap_rec(inner_dtype) {
-                Cow::Owned(inner_dtype) => Cow::Owned(T::Array(Box::new(inner_dtype), *width)),
-                _ => Cow::Borrowed(dtype),
-            }
-        },
-        #[cfg(feature = "dtype-struct")]
-        T::Struct(fields) => {
-            for (i, field) in fields.iter().enumerate() {
-                if let Cow::Owned(field_dtype) =
-                    initialize_empty_categorical_revmap_rec(field.dtype())
-                {
-                    let mut new_fields = Vec::with_capacity(fields.len());
-                    new_fields.extend(fields[..i].iter().cloned());
-                    new_fields.push(Field::new(field.name().clone(), field_dtype));
-                    new_fields.extend(fields[i + 1..].iter().map(|field| {
-                        let field_dtype =
-                            initialize_empty_categorical_revmap_rec(field.dtype()).into_owned();
-                        Field::new(field.name().clone(), field_dtype)
-                    }));
-                    return Cow::Owned(T::Struct(new_fields));
-                }
-            }
-
-            Cow::Borrowed(dtype)
-        },
-        _ => Cow::Borrowed(dtype),
-    }
-}
-
 impl Series {
     /// Construct a new [`Series`] from a slice of AnyValues.
     ///
@@ -71,7 +30,8 @@ impl Series {
     /// and the `strict` parameter:
     /// - If `strict` is `true`, the data type is equal to the data type of the
     ///   first non-null value. If any other non-null values do not match this
-    ///   data type, an error is raised.
+    ///   data type, an error is raised. If the first non-null value is a
+    ///   decimal the slice is scanned for the maximum precision and scale possible.
     /// - If `strict` is `false`, the data type is the supertype of the `values`.
     ///   An error is returned if no supertype can be determined.
     ///   **WARNING**: A full pass over the values is required to determine the supertype.
@@ -104,19 +64,23 @@ impl Series {
             }
         }
         let dtype = if strict {
-            get_first_non_null_dtype(values)
+            match get_first_non_null_dtype(values) {
+                #[cfg(feature = "dtype-decimal")]
+                DataType::Decimal(mut prec, mut scale) => {
+                    for v in values {
+                        if let DataType::Decimal(p, s) = v.dtype() {
+                            prec = prec.max(p);
+                            scale = scale.max(s);
+                        }
+                    }
+                    DataType::Decimal(prec, scale)
+                },
+                dt => dt,
+            }
         } else {
-            // Currently does not work correctly for Decimal because equality is not implemented.
             any_values_to_supertype(values)?
         };
 
-        // TODO: Remove this when Decimal data type equality is implemented.
-        #[cfg(feature = "dtype-decimal")]
-        if dtype.is_decimal() {
-            let dtype = DataType::Decimal(None, None);
-            return Self::from_any_values_and_dtype(name, values, &dtype, strict);
-        }
-
         Self::from_any_values_and_dtype(name, values, &dtype, strict)
     }
 
@@ -132,12 +96,7 @@ impl Series {
         strict: bool,
     ) -> PolarsResult<Self> {
         if values.is_empty() {
-            return Ok(Self::new_empty(
-                name,
-                // This is given categoricals with empty revmaps, but we need to always return
-                // categoricals with non-empty revmaps.
-                initialize_empty_categorical_revmap_rec(dtype).as_ref(),
-            ));
+            return Ok(Self::new_empty(name, dtype));
         }
 
         let mut s = match dtype {
@@ -155,6 +114,10 @@ impl Series {
             DataType::UInt16 => any_values_to_integer::<UInt16Type>(values, strict)?.into_series(),
             DataType::UInt32 => any_values_to_integer::<UInt32Type>(values, strict)?.into_series(),
             DataType::UInt64 => any_values_to_integer::<UInt64Type>(values, strict)?.into_series(),
+            #[cfg(feature = "dtype-u128")]
+            DataType::UInt128 => {
+                any_values_to_integer::<UInt128Type>(values, strict)?.into_series()
+            },
             DataType::Float32 => any_values_to_f32(values, strict)?.into_series(),
             DataType::Float64 => any_values_to_f64(values, strict)?.into_series(),
             DataType::Boolean => any_values_to_bool(values, strict)?.into_series(),
@@ -172,9 +135,9 @@ impl Series {
             #[cfg(feature = "dtype-duration")]
             DataType::Duration(tu) => any_values_to_duration(values, *tu, strict)?.into_series(),
             #[cfg(feature = "dtype-categorical")]
-            dt @ DataType::Categorical(_, _) => any_values_to_categorical(values, dt, strict)?,
-            #[cfg(feature = "dtype-categorical")]
-            dt @ DataType::Enum(_, _) => any_values_to_enum(values, dt, strict)?,
+            dt @ (DataType::Categorical(_, _) | DataType::Enum(_, _)) => {
+                any_values_to_categorical(values, dt, strict)?
+            },
             #[cfg(feature = "dtype-decimal")]
             DataType::Decimal(precision, scale) => {
                 any_values_to_decimal(values, *precision, *scale, strict)?.into_series()
@@ -498,139 +461,73 @@ fn any_values_to_categorical(
     dtype: &DataType,
     strict: bool,
 ) -> PolarsResult<Series> {
-    let ordering = match dtype {
-        DataType::Categorical(_, ordering) => ordering,
-        _ => panic!("any_values_to_categorical with dtype={dtype:?}"),
-    };
+    with_match_categorical_physical_type!(dtype.cat_physical().unwrap(), |$C| {
+        let mut builder = CategoricalChunkedBuilder::<$C>::new(PlSmallStr::EMPTY, dtype.clone());
 
-    let mut builder = CategoricalChunkedBuilder::new(PlSmallStr::EMPTY, values.len(), *ordering);
+        let mut owned = String::new(); // Amortize allocations.
+        for av in values {
+            let ret = match av {
+                AnyValue::String(s) => builder.append_str(s),
+                AnyValue::StringOwned(s) => builder.append_str(s),
 
-    let mut owned = String::new(); // Amortize allocations.
-    for av in values {
-        match av {
-            AnyValue::String(s) => builder.append_value(s),
-            AnyValue::StringOwned(s) => builder.append_value(s),
+                &AnyValue::Enum(cat, &ref map) |
+                &AnyValue::EnumOwned(cat, ref map) |
+                &AnyValue::Categorical(cat, &ref map) |
+                &AnyValue::CategoricalOwned(cat, ref map) => builder.append_cat(cat, map),
 
-            AnyValue::Enum(s, rev, _) => builder.append_value(rev.get(*s)),
-            AnyValue::EnumOwned(s, rev, _) => builder.append_value(rev.get(*s)),
-
-            AnyValue::Categorical(s, rev, _) => builder.append_value(rev.get(*s)),
-            AnyValue::CategoricalOwned(s, rev, _) => builder.append_value(rev.get(*s)),
-
-            AnyValue::Binary(_) | AnyValue::BinaryOwned(_) if !strict => builder.append_null(),
-            AnyValue::Null => builder.append_null(),
-
-            av => {
-                if strict {
-                    return Err(invalid_value_error(&DataType::String, av));
+                AnyValue::Binary(_) | AnyValue::BinaryOwned(_) if !strict => {
+                    builder.append_null();
+                    Ok(())
+                },
+                AnyValue::Null => {
+                    builder.append_null();
+                    Ok(())
                 }
 
-                owned.clear();
-                write!(owned, "{av}").unwrap();
-                builder.append_value(&owned);
-            },
+                av => {
+                    if strict {
+                        return Err(invalid_value_error(&DataType::String, av));
+                    }
+
+                    owned.clear();
+                    write!(owned, "{av}").unwrap();
+                    builder.append_str(&owned)
+                },
+            };
+
+            if let Err(e) = ret {
+                if strict {
+                    return Err(e);
+                } else {
+                    builder.append_null();
+                }
+            }
         }
-    }
 
-    let ca = builder.finish();
-
-    Ok(ca.into_series())
-}
-
-#[cfg(feature = "dtype-categorical")]
-fn any_values_to_enum(values: &[AnyValue], dtype: &DataType, strict: bool) -> PolarsResult<Series> {
-    use self::enum_::EnumChunkedBuilder;
-
-    let (rev, ordering) = match dtype {
-        DataType::Enum(rev, ordering) => (rev.clone(), ordering),
-        _ => panic!("any_values_to_categorical with dtype={dtype:?}"),
-    };
-
-    let Some(rev) = rev else {
-        polars_bail!(nyi = "Not yet possible to create enum series without a rev-map");
-    };
-
-    let mut builder =
-        EnumChunkedBuilder::new(PlSmallStr::EMPTY, values.len(), rev, *ordering, strict);
-
-    let mut owned = String::new(); // Amortize allocations.
-    for av in values {
-        match av {
-            AnyValue::String(s) => builder.append_str(s)?,
-            AnyValue::StringOwned(s) => builder.append_str(s)?,
-
-            AnyValue::Enum(s, rev, _) => builder.append_enum(*s, rev)?,
-            AnyValue::EnumOwned(s, rev, _) => builder.append_enum(*s, rev)?,
-
-            AnyValue::Categorical(s, rev, _) => builder.append_str(rev.get(*s))?,
-            AnyValue::CategoricalOwned(s, rev, _) => builder.append_str(rev.get(*s))?,
-
-            AnyValue::Binary(_) | AnyValue::BinaryOwned(_) if !strict => builder.append_null(),
-            AnyValue::Null => builder.append_null(),
-
-            av => {
-                if strict {
-                    return Err(invalid_value_error(&DataType::String, av));
-                }
-
-                owned.clear();
-                write!(owned, "{av}").unwrap();
-                builder.append_str(&owned)?
-            },
-        };
-    }
-
-    let ca = builder.finish();
-
-    Ok(ca.into_series())
+        let ca = builder.finish();
+        Ok(ca.into_series())
+    })
 }
 
 #[cfg(feature = "dtype-decimal")]
 fn any_values_to_decimal(
     values: &[AnyValue],
-    precision: Option<usize>,
-    scale: Option<usize>, // If None, we're inferring the scale.
+    precision: usize,
+    scale: usize,
     strict: bool,
 ) -> PolarsResult<DecimalChunked> {
-    /// Get the maximum scale among AnyValues
-    fn infer_scale(
-        values: &[AnyValue],
-        precision: Option<usize>,
-        strict: bool,
-    ) -> PolarsResult<usize> {
-        let mut max_scale = 0;
-        for av in values {
-            let av_scale = match av {
-                AnyValue::Decimal(_, scale) => *scale,
-                AnyValue::Null => continue,
-                av => {
-                    if strict {
-                        let target_dtype = DataType::Decimal(precision, None);
-                        return Err(invalid_value_error(&target_dtype, av));
-                    }
-                    continue;
-                },
-            };
-            max_scale = max_scale.max(av_scale);
-        }
-        Ok(max_scale)
-    }
-    let scale = match scale {
-        Some(s) => s,
-        None => infer_scale(values, precision, strict)?,
-    };
-    let target_dtype = DataType::Decimal(precision, Some(scale));
+    let target_dtype = DataType::Decimal(precision, scale);
 
     let mut builder = PrimitiveChunkedBuilder::<Int128Type>::new(PlSmallStr::EMPTY, values.len());
     for av in values {
         match av {
             // Allow equal or less scale. We do want to support different scales even in 'strict' mode.
-            AnyValue::Decimal(v, s) if *s <= scale => {
-                if *s == scale {
+            AnyValue::Decimal(v, p, s) if *s <= scale => {
+                if *p <= precision && *s == scale {
                     builder.append_value(*v)
                 } else {
                     match av.strict_cast(&target_dtype) {
-                        Some(AnyValue::Decimal(i, _)) => builder.append_value(i),
+                        Some(AnyValue::Decimal(i, _, _)) => builder.append_value(i),
                         _ => builder.append_null(),
                     }
                 }
@@ -640,9 +537,8 @@ fn any_values_to_decimal(
                 if strict {
                     return Err(invalid_value_error(&target_dtype, av));
                 }
-                // TODO: Precision check, else set to null
                 match av.strict_cast(&target_dtype) {
-                    Some(AnyValue::Decimal(i, _)) => builder.append_value(i),
+                    Some(AnyValue::Decimal(i, _, _)) => builder.append_value(i),
                     _ => builder.append_null(),
                 }
             },
@@ -701,20 +597,8 @@ fn any_values_to_list(
         DataType::Object(_) => polars_bail!(nyi = "Nested object types"),
 
         _ => {
-            let list_inner_type = match inner_type {
-                // Categoricals may not have a revmap yet. We just give them an empty one here and
-                // the list builder takes care of the rest.
-                #[cfg(feature = "dtype-categorical")]
-                DataType::Categorical(None, ordering) => {
-                    DataType::Categorical(Some(Arc::new(RevMapping::default())), *ordering)
-                },
-
-                _ => inner_type.clone(),
-            };
-
             let mut builder =
-                get_list_builder(&list_inner_type, capacity * 5, capacity, PlSmallStr::EMPTY);
-
+                get_list_builder(inner_type, capacity * 5, capacity, PlSmallStr::EMPTY);
             for av in avs {
                 match av {
                     AnyValue::List(b) => match b.cast(inner_type) {
@@ -819,7 +703,7 @@ fn any_values_to_array(
     #[cfg(feature = "dtype-struct")]
     if !matches!(inner_type, DataType::Null) && out.inner_dtype().is_nested() {
         unsafe {
-            out.set_dtype(target_dtype.clone());
+            out.set_dtype(target_dtype);
         };
     }
 
