@@ -37,7 +37,7 @@ impl<T: PolarsCategoricalType> SeriesWrap<CategoricalChunked<T>> {
 }
 
 macro_rules! impl_cat_series {
-    ($ca: ident, $pdt:ty) => {
+    ($ca: ident, $pdt:ty, $ca_fn:ident) => {
         impl private::PrivateSeries for SeriesWrap<$ca> {
             fn compute_len(&mut self) {
                 self.0.physical_mut().compute_len()
@@ -99,6 +99,26 @@ macro_rules! impl_cat_series {
                 Ok(())
             }
 
+            #[cfg(feature = "algorithm_group_by")]
+            unsafe fn agg_min(&self, groups: &GroupsType) -> Series {
+                if self.0.uses_lexical_ordering() {
+                    unimplemented!()
+                } else {
+                    self.apply_on_phys(|phys| phys.agg_min(groups).$ca_fn().unwrap().clone())
+                        .into_series()
+                }
+            }
+
+            #[cfg(feature = "algorithm_group_by")]
+            unsafe fn agg_max(&self, groups: &GroupsType) -> Series {
+                if self.0.uses_lexical_ordering() {
+                    unimplemented!()
+                } else {
+                    self.apply_on_phys(|phys| phys.agg_max(groups).$ca_fn().unwrap().clone())
+                        .into_series()
+                }
+            }
+
             #[cfg(feature = "algorithm_group_by")]
             unsafe fn agg_list(&self, groups: &GroupsType) -> Series {
                 // we cannot cast and dispatch as the inner type of the list would be incorrect
@@ -202,6 +222,11 @@ macro_rules! impl_cat_series {
                 unsafe { self.apply_on_phys(|cats| cats.take_unchecked(indices)).into_series() }
             }
 
+            fn deposit(&self, validity: &Bitmap) -> Series {
+                unsafe { self.apply_on_phys(|cats| cats.deposit(validity)) }
+                    .into_series()
+            }
+
             fn len(&self) -> usize {
                 self.0.len()
             }
@@ -249,6 +274,11 @@ macro_rules! impl_cat_series {
                 self.0.physical().n_unique()
             }
 
+            #[cfg(feature = "approx_unique")]
+            fn approx_n_unique(&self) -> PolarsResult<IdxSize> {
+                Ok(self.0.physical().approx_n_unique())
+            }
+
             #[cfg(feature = "algorithm_group_by")]
             fn arg_unique(&self) -> PolarsResult<IdxCa> {
                 self.0.physical().arg_unique()
@@ -315,6 +345,6 @@ macro_rules! impl_cat_series {
     }
 }
 
-impl_cat_series!(Categorical8Chunked, Categorical8Type);
-impl_cat_series!(Categorical16Chunked, Categorical16Type);
-impl_cat_series!(Categorical32Chunked, Categorical32Type);
+impl_cat_series!(Categorical8Chunked, Categorical8Type, u8);
+impl_cat_series!(Categorical16Chunked, Categorical16Type, u16);
+impl_cat_series!(Categorical32Chunked, Categorical32Type, u32);
