@@ -1,17 +1,14 @@
-use std::sync::OnceLock;
-
 use polars_core::chunked_array::cast::CastOptions;
 use polars_core::prelude::*;
 
 use super::*;
-use crate::expressions::{AggState, AggregationContext, PartitionedAggregation, PhysicalExpr};
+use crate::expressions::{AggState, AggregationContext, PhysicalExpr};
 
 pub struct CastExpr {
     pub(crate) input: Arc<dyn PhysicalExpr>,
     pub(crate) dtype: DataType,
     pub(crate) expr: Expr,
     pub(crate) options: CastOptions,
-    pub(crate) inlined_eval: OnceLock<Option<Column>>,
 }
 
 impl CastExpr {
@@ -30,18 +27,6 @@ impl PhysicalExpr for CastExpr {
         self.finish(&column)
     }
 
-    fn evaluate_inline_impl(&self, depth_limit: u8) -> Option<Column> {
-        self.inlined_eval
-            .get_or_init(|| {
-                let depth_limit = depth_limit.checked_sub(1)?;
-                self.input
-                    .evaluate_inline_impl(depth_limit)
-                    .filter(|x| x.len() == 1)
-                    .and_then(|x| self.finish(&x).ok())
-            })
-            .clone()
-    }
-
     #[allow(clippy::ptr_arg)]
     fn evaluate_on_groups<'a>(
         &self,
@@ -69,19 +54,35 @@ impl PhysicalExpr for CastExpr {
                     ac.with_values(s, true, None)?;
                 }
             },
-            _ => {
-                // before we flatten, make sure that groups are updated
-                ac.groups();
+            AggState::NotAggregated(_) => {
+                if match self.options {
+                    CastOptions::NonStrict | CastOptions::Overflowing => true,
+                    CastOptions::Strict => ac.original_len,
+                } {
+                    // before we flatten, make sure that groups are updated
+                    ac.groups();
 
-                let s = ac.flat_naive();
-                let s = self.finish(&s.as_ref().clone().into_column())?;
+                    let s = ac.flat_naive();
+                    let s = self.finish(&s.as_ref().clone().into_column())?;
 
-                if ac.is_literal() {
-                    ac.with_literal(s);
-                } else {
                     ac.with_values(s, false, None)?;
+                } else {
+                    // We need to perform aggregation only for strict mode, since if this is not done,
+                    // filtered-out values may incorrectly cause a cast error.
+                    let s = ac.aggregated();
+                    let ca = s.list().unwrap();
+                    let casted = ca.apply_to_inner(&|s| {
+                        self.finish(&s.into_column())
+                            .map(|c| c.take_materialized_series())
+                    })?;
+                    ac.with_values(casted.into_column(), true, None)?;
                 }
             },
+
+            AggState::LiteralScalar(s) => {
+                let s = self.finish(s)?;
+                ac.with_literal(s);
+            },
         }
 
         Ok(ac)
@@ -97,30 +98,4 @@ impl PhysicalExpr for CastExpr {
     fn is_scalar(&self) -> bool {
         self.input.is_scalar()
     }
-
-    fn as_partitioned_aggregator(&self) -> Option<&dyn PartitionedAggregation> {
-        Some(self)
-    }
-}
-
-impl PartitionedAggregation for CastExpr {
-    fn evaluate_partitioned(
-        &self,
-        df: &DataFrame,
-        groups: &GroupPositions,
-        state: &ExecutionState,
-    ) -> PolarsResult<Column> {
-        let e = self.input.as_partitioned_aggregator().unwrap();
-        self.finish(&e.evaluate_partitioned(df, groups, state)?)
-    }
-
-    fn finalize(
-        &self,
-        partitioned: Column,
-        groups: &GroupPositions,
-        state: &ExecutionState,
-    ) -> PolarsResult<Column> {
-        let agg = self.input.as_partitioned_aggregator().unwrap();
-        agg.finalize(partitioned, groups, state)
-    }
 }
