@@ -1,9 +1,10 @@
 use polars_core::prelude::*;
-use polars_plan::constants::MAP_LIST_NAME;
+use polars_plan::constants::PL_ELEMENT_NAME;
 use polars_plan::prelude::expr_ir::ExprIR;
 use polars_plan::prelude::*;
 use recursive::recursive;
 
+use crate::dispatch::{function_expr_to_groups_udf, function_expr_to_udf};
 use crate::expressions as phys_expr;
 use crate::expressions::*;
 
@@ -167,28 +168,24 @@ fn create_physical_expr_inner(
 
     match expr_arena.get(expression) {
         Len => Ok(Arc::new(phys_expr::CountExpr::new())),
-        Window {
+        aexpr @ Window {
             function,
             partition_by,
             order_by,
             options,
         } => {
+            let output_field = aexpr.to_field(&ToFieldContext::new(expr_arena, schema))?;
             let function = *function;
             state.set_window();
-            let phys_function = create_physical_expr_inner(
-                function,
-                Context::Aggregation,
-                expr_arena,
-                schema,
-                state,
-            )?;
+            let phys_function =
+                create_physical_expr_inner(function, Context::Default, expr_arena, schema, state)?;
 
             let order_by = order_by
                 .map(|(node, options)| {
                     PolarsResult::Ok((
                         create_physical_expr_inner(
                             node,
-                            Context::Aggregation,
+                            Context::Default,
                             expr_arena,
                             schema,
                             state,
@@ -198,7 +195,6 @@ fn create_physical_expr_inner(
                 })
                 .transpose()?;
 
-            let function_expr = node_to_expr(function, expr_arena);
             let expr = node_to_expr(expression, expr_arena);
 
             // set again as the state can be reset
@@ -208,7 +204,7 @@ fn create_physical_expr_inner(
                     // TODO! Order by
                     let group_by = create_physical_expressions_from_nodes(
                         partition_by,
-                        Context::Aggregation,
+                        Context::Default,
                         expr_arena,
                         schema,
                         state,
@@ -262,19 +258,19 @@ fn create_physical_expr_inner(
                         group_by,
                         order_by,
                         apply_columns,
-                        function: function_expr,
                         phys_function,
                         mapping: *mapping,
                         expr,
                         has_different_group_sources,
+                        output_field,
                     }))
                 },
                 #[cfg(feature = "dynamic_group_by")]
                 WindowType::Rolling(options) => Ok(Arc::new(RollingExpr {
-                    function: function_expr,
                     phys_function,
                     options: options.clone(),
                     expr,
+                    output_field,
                 })),
             }
         },
@@ -286,10 +282,9 @@ fn create_physical_expr_inner(
             )))
         },
         BinaryExpr { left, op, right } => {
-            let output_field =
-                expr_arena
-                    .get(expression)
-                    .to_field(schema, Context::Default, expr_arena)?;
+            let output_field = expr_arena
+                .get(expression)
+                .to_field(&ToFieldContext::new(expr_arena, schema))?;
             let is_scalar = is_scalar_ae(expression, expr_arena);
             let lhs = create_physical_expr_inner(*left, ctxt, expr_arena, schema, state)?;
             let rhs = create_physical_expr_inner(*right, ctxt, expr_arena, schema, state)?;
@@ -309,6 +304,11 @@ fn create_physical_expr_inner(
             node_to_expr(expression, expr_arena),
             schema.clone(),
         ))),
+        Element => Ok(Arc::new(ColumnExpr::new(
+            PL_ELEMENT_NAME.clone(),
+            node_to_expr(expression, expr_arena),
+            schema.clone(),
+        ))),
         Sort { expr, options } => {
             let phys_expr = create_physical_expr_inner(*expr, ctxt, expr_arena, schema, state)?;
             Ok(Arc::new(SortExpr::new(
@@ -366,6 +366,9 @@ fn create_physical_expr_inner(
                 Context::Default if !matches!(agg, IRAggExpr::Quantile { .. }) => {
                     use {GroupByMethod as GBM, IRAggExpr as I};
 
+                    let output_field = expr_arena
+                        .get(expression)
+                        .to_field(&ToFieldContext::new(expr_arena, schema))?;
                     let groupby = match agg {
                         I::Min { propagate_nans, .. } if *propagate_nans => GBM::NanMin,
                         I::Min { .. } => GBM::Min,
@@ -375,11 +378,17 @@ fn create_physical_expr_inner(
                         I::NUnique(_) => GBM::NUnique,
                         I::First(_) => GBM::First,
                         I::Last(_) => GBM::Last,
+                        I::Item { allow_empty, .. } => GBM::Item {
+                            allow_empty: *allow_empty,
+                        },
                         I::Mean(_) => GBM::Mean,
                         I::Implode(_) => GBM::Implode,
                         I::Quantile { .. } => unreachable!(),
                         I::Sum(_) => GBM::Sum,
-                        I::Count(_, include_nulls) => GBM::Count {
+                        I::Count {
+                            input: _,
+                            include_nulls,
+                        } => GBM::Count {
                             include_nulls: *include_nulls,
                         },
                         I::Std(_, ddof) => GBM::Std(*ddof),
@@ -394,7 +403,11 @@ fn create_physical_expr_inner(
                         allow_threading,
                     };
 
-                    Ok(Arc::new(AggregationExpr::new(input, agg_type, None)))
+                    Ok(Arc::new(AggregationExpr::new(
+                        input,
+                        agg_type,
+                        output_field,
+                    )))
                 },
                 _ => {
                     if let IRAggExpr::Quantile {
@@ -408,18 +421,25 @@ fn create_physical_expr_inner(
                         return Ok(Arc::new(AggQuantileExpr::new(input, quantile, *interpol)));
                     }
 
-                    let field = expr_arena.get(expression).to_field(
-                        schema,
-                        Context::Aggregation,
-                        expr_arena,
-                    )?;
+                    let mut output_field = expr_arena
+                        .get(expression)
+                        .to_field(&ToFieldContext::new(expr_arena, schema))?;
+
+                    if matches!(ctxt, Context::Aggregation) && !is_scalar_ae(expression, expr_arena)
+                    {
+                        output_field.coerce(output_field.dtype.clone().implode());
+                    }
 
                     let groupby = GroupByMethod::from(agg.clone());
                     let agg_type = AggregationType {
                         groupby,
                         allow_threading: false,
                     };
-                    Ok(Arc::new(AggregationExpr::new(input, agg_type, Some(field))))
+                    Ok(Arc::new(AggregationExpr::new(
+                        input,
+                        agg_type,
+                        output_field,
+                    )))
                 },
             }
         },
@@ -434,7 +454,6 @@ fn create_physical_expr_inner(
                 dtype: dtype.clone(),
                 expr: node_to_expr(expression, expr_arena),
                 options: *options,
-                inlined_eval: Default::default(),
             }))
         },
         Ternary {
@@ -466,27 +485,31 @@ fn create_physical_expr_inner(
         AnonymousFunction {
             input,
             function,
-            output_type: _,
             options,
             fmt_str: _,
         } => {
             let is_scalar = is_scalar_ae(expression, expr_arena);
             let output_field = expr_arena
                 .get(expression)
-                .to_field(schema, ctxt, expr_arena)?;
+                .to_field(&ToFieldContext::new(expr_arena, schema))?;
 
             let input =
                 create_physical_expressions_from_irs(input, ctxt, expr_arena, schema, state)?;
 
+            let function = function.clone().materialize()?;
+            let function = function.into_inner().as_column_udf();
+
             Ok(Arc::new(ApplyExpr::new(
                 input,
-                function.clone().materialize()?,
+                SpecialEq::new(function),
+                None,
                 node_to_expr(expression, expr_arena),
                 *options,
                 state.allow_threading,
                 schema.clone(),
                 output_field,
                 is_scalar,
+                true,
             )))
         },
         Eval {
@@ -494,26 +517,39 @@ fn create_physical_expr_inner(
             evaluation,
             variant,
         } => {
-            let is_user_apply = expr_arena.iter(*expr).any(|(_, e)| matches!(e, AExpr::AnonymousFunction { fmt_str, .. } if fmt_str.as_ref().as_str() == MAP_LIST_NAME));
             let is_scalar = is_scalar_ae(expression, expr_arena);
             let evaluation_is_scalar = is_scalar_ae(*evaluation, expr_arena);
+            let evaluation_is_elementwise = is_elementwise_rec(*evaluation, expr_arena);
+            // @NOTE: This is actually also something the downstream apply code should care about.
             let mut pd_group = ExprPushdownGroup::Pushable;
             pd_group.update_with_expr_rec(expr_arena.get(*evaluation), expr_arena, None);
+            let evaluation_is_fallible = matches!(pd_group, ExprPushdownGroup::Fallible);
+
             let output_field = expr_arena
                 .get(expression)
-                .to_field(schema, ctxt, expr_arena)?;
-            let non_aggregated_output_field =
-                expr_arena
-                    .get(expression)
-                    .to_field(schema, Context::Default, expr_arena)?;
-            let input_field = expr_arena.get(*expr).to_field(schema, ctxt, expr_arena)?;
-            let expr = create_physical_expr_inner(*expr, ctxt, expr_arena, schema, state)?;
+                .to_field(&ToFieldContext::new(expr_arena, schema))?;
+            let input_field = expr_arena
+                .get(*expr)
+                .to_field(&ToFieldContext::new(expr_arena, schema))?;
+            let expr =
+                create_physical_expr_inner(*expr, Context::Default, expr_arena, schema, state)?;
 
             let element_dtype = variant.element_dtype(&input_field.dtype)?;
-            let eval_schema = Schema::from_iter([(PlSmallStr::EMPTY, element_dtype.clone())]);
+            let mut eval_schema = schema.as_ref().clone();
+            eval_schema.insert(PL_ELEMENT_NAME.clone(), element_dtype.clone());
             let evaluation = create_physical_expr_inner(
                 *evaluation,
-                Context::Default,
+                // @Hack. Since EvalVariant::Array uses `evaluate_on_groups` to determine the
+                // output and that expects to be outputting a list, we need to pretend like we are
+                // aggregating here.
+                //
+                // EvalVariant::List also has this problem but that has a List datatype, so that
+                // goes wrong by pure change and some black magic.
+                if matches!(variant, EvalVariant::Array { .. }) && !evaluation_is_elementwise {
+                    Context::Aggregation
+                } else {
+                    Context::Default
+                },
                 expr_arena,
                 &Arc::new(eval_schema),
                 state,
@@ -524,13 +560,11 @@ fn create_physical_expr_inner(
                 evaluation,
                 *variant,
                 node_to_expr(expression, expr_arena),
-                state.allow_threading,
                 output_field,
-                non_aggregated_output_field.dtype,
                 is_scalar,
-                pd_group,
                 evaluation_is_scalar,
-                is_user_apply,
+                evaluation_is_elementwise,
+                evaluation_is_fallible,
             )))
         },
         Function {
@@ -541,19 +575,22 @@ fn create_physical_expr_inner(
             let is_scalar = is_scalar_ae(expression, expr_arena);
             let output_field = expr_arena
                 .get(expression)
-                .to_field(schema, ctxt, expr_arena)?;
+                .to_field(&ToFieldContext::new(expr_arena, schema))?;
             let input =
                 create_physical_expressions_from_irs(input, ctxt, expr_arena, schema, state)?;
+            let is_fallible = expr_arena.get(expression).is_fallible_top_level(expr_arena);
 
             Ok(Arc::new(ApplyExpr::new(
                 input,
-                function.clone().into(),
+                function_expr_to_udf(function.clone()),
+                function_expr_to_groups_udf(function),
                 node_to_expr(expression, expr_arena),
                 *options,
                 state.allow_threading,
                 schema.clone(),
                 output_field,
                 is_scalar,
+                is_fallible,
             )))
         },
         Slice {
@@ -564,7 +601,8 @@ fn create_physical_expr_inner(
             let input = create_physical_expr_inner(*input, ctxt, expr_arena, schema, state)?;
             let offset = create_physical_expr_inner(*offset, ctxt, expr_arena, schema, state)?;
             let length = create_physical_expr_inner(*length, ctxt, expr_arena, schema, state)?;
-            polars_ensure!(!(state.has_implode() && matches!(ctxt, Context::Aggregation)), InvalidOperation: "'implode' followed by a slice during aggregation is not allowed");
+            polars_ensure!(!(state.has_implode() && matches!(ctxt, Context::Aggregation)),
+                InvalidOperation: "'implode' followed by a slice during aggregation is not allowed");
             Ok(Arc::new(SliceExpr {
                 input,
                 offset,
@@ -576,25 +614,28 @@ fn create_physical_expr_inner(
             let input = create_physical_expr_inner(*expr, ctxt, expr_arena, schema, state)?;
             let skip_empty = *skip_empty;
             let function = SpecialEq::new(Arc::new(
-                move |c: &mut [polars_core::frame::column::Column]| {
-                    c[0].explode(skip_empty).map(Some)
-                },
+                move |c: &mut [polars_core::frame::column::Column]| c[0].explode(skip_empty),
             ) as Arc<dyn ColumnsUdf>);
 
-            let field = expr_arena
+            let output_field = expr_arena
                 .get(expression)
-                .to_field(schema, ctxt, expr_arena)?;
+                .to_field(&ToFieldContext::new(expr_arena, schema))?;
 
             Ok(Arc::new(ApplyExpr::new(
                 vec![input],
                 function,
+                None,
                 node_to_expr(expression, expr_arena),
                 FunctionOptions::groupwise(),
                 state.allow_threading,
                 schema.clone(),
-                field,
+                output_field,
+                false,
                 false,
             )))
         },
+        AnonymousStreamingAgg { .. } => {
+            polars_bail!(ComputeError: "anonymous agg not supported in in-memory engine")
+        },
     }
 }
