@@ -9,6 +9,7 @@ use polars_core::schema::{Schema, SchemaExt};
 use polars_error::{PolarsResult, polars_bail, polars_err, to_compute_err};
 use polars_utils::mmap::MMapSemaphore;
 use polars_utils::pl_str::PlSmallStr;
+use polars_utils::plpath::PlPathRef;
 
 use crate::RowIndex;
 use crate::cloud::{
@@ -65,11 +66,12 @@ impl IpcReadOptions {
 
 impl IpcReaderAsync {
     pub async fn from_uri(
-        uri: &str,
+        uri: PlPathRef<'_>,
         cloud_options: Option<&CloudOptions>,
     ) -> PolarsResult<IpcReaderAsync> {
         let cache_entry =
-            init_entries_from_uri_list([Arc::from(uri)].into_iter(), cloud_options)?[0].clone();
+            init_entries_from_uri_list([Arc::from(uri.to_str())].into_iter(), cloud_options)?[0]
+                .clone();
         let (CloudLocation { prefix, .. }, store) =
             build_object_store(uri, cloud_options, false).await?;
 
