@@ -181,6 +181,7 @@ fn to_column_write_options_rec(
     let mut column_options = ColumnWriteOptions {
         field_id: None,
         metadata: Vec::new(),
+        required: None,
 
         // Dummy value.
         children: ChildWriteOptions::Leaf(FieldWriteOptions {
@@ -191,6 +192,7 @@ fn to_column_write_options_rec(
     if let Some(overwrites) = overwrites {
         column_options.field_id = overwrites.field_id;
         column_options.metadata = convert_metadata(&overwrites.metadata);
+        column_options.required = overwrites.required;
     }
 
     use arrow::datatypes::PhysicalType::*;
@@ -202,8 +204,9 @@ fn to_column_write_options_rec(
             });
         },
         List | FixedSizeList | LargeList => {
-            let child_overwrites = overwrites.map(|o| match &o.children {
-                ChildFieldOverwrites::ListLike(child_overwrites) => child_overwrites.as_ref(),
+            let child_overwrites = overwrites.and_then(|o| match &o.children {
+                ChildFieldOverwrites::None => None,
+                ChildFieldOverwrites::ListLike(child_overwrites) => Some(child_overwrites.as_ref()),
                 _ => unreachable!(),
             });
 
@@ -223,27 +226,37 @@ fn to_column_write_options_rec(
         },
         Struct => {
             if let ArrowDataType::Struct(fields) = field.dtype().to_logical_type() {
-                let children_overwrites = overwrites.map(|o| match &o.children {
-                    ChildFieldOverwrites::Struct(child_overwrites) => PlHashMap::from_iter(
-                        child_overwrites
-                            .iter()
-                            .map(|f| (f.name.as_ref().unwrap(), f)),
-                    ),
-                    _ => unreachable!(),
-                });
+                if fields.is_empty() {
+                    // Allow empty structs by mapping to boolean array.
+                    column_options.children = ChildWriteOptions::Leaf(FieldWriteOptions {
+                        encoding: Encoding::Rle,
+                    });
+                } else {
+                    let children_overwrites = overwrites.and_then(|o| match &o.children {
+                        ChildFieldOverwrites::None => None,
+                        ChildFieldOverwrites::Struct(child_overwrites) => {
+                            Some(PlHashMap::from_iter(
+                                child_overwrites
+                                    .iter()
+                                    .map(|f| (f.name.as_ref().unwrap(), f)),
+                            ))
+                        },
+                        _ => unreachable!(),
+                    });
 
-                let children = fields
-                    .iter()
-                    .map(|f| {
-                        let overwrites = children_overwrites
-                            .as_ref()
-                            .and_then(|o| o.get(&f.name).copied());
-                        to_column_write_options_rec(f, overwrites)
-                    })
-                    .collect();
+                    let children = fields
+                        .iter()
+                        .map(|f| {
+                            let overwrites = children_overwrites
+                                .as_ref()
+                                .and_then(|o| o.get(&f.name).copied());
+                            to_column_write_options_rec(f, overwrites)
+                        })
+                        .collect();
 
-                column_options.children =
-                    ChildWriteOptions::Struct(Box::new(StructFieldWriteOptions { children }));
+                    column_options.children =
+                        ChildWriteOptions::Struct(Box::new(StructFieldWriteOptions { children }));
+                }
             } else {
                 unreachable!()
             }
