@@ -419,14 +419,11 @@ fn test_ipc_globbing() -> PolarsResult<()> {
     let glob = "../../examples/datasets/foods*.ipc";
     let df = LazyFrame::scan_ipc(
         PlPath::new(glob),
-        ScanArgsIpc {
-            n_rows: None,
+        Default::default(),
+        UnifiedScanArgs {
             cache: true,
-            rechunk: false,
-            row_index: None,
-            cloud_options: None,
-            hive_options: Default::default(),
-            include_file_paths: None,
+            glob: true,
+            ..Default::default()
         },
     )?
     .collect()?;
@@ -519,7 +516,7 @@ fn test_union_and_agg_projections() -> PolarsResult<()> {
     // a union vstacks columns and aggscan optimization determines columns to aggregate in a
     // hashmap, if that doesn't set them sorted the vstack will panic.
     let lf1: LazyFrame = DslBuilder::scan_parquet(
-        ScanSources::Paths([PlPath::new(GLOB_PARQUET)].into()),
+        ScanSources::Paths(FromIterator::from_iter([PlPath::new(GLOB_PARQUET)])),
         Default::default(),
         UnifiedScanArgs {
             extra_columns_policy: ExtraColumnsPolicy::Ignore,
@@ -531,7 +528,7 @@ fn test_union_and_agg_projections() -> PolarsResult<()> {
     .into();
 
     let lf2: LazyFrame = DslBuilder::scan_ipc(
-        ScanSources::Paths([PlPath::new(GLOB_IPC)].into()),
+        ScanSources::Paths(FromIterator::from_iter([PlPath::new(GLOB_IPC)])),
         Default::default(),
         UnifiedScanArgs {
             extra_columns_policy: ExtraColumnsPolicy::Ignore,
@@ -543,7 +540,7 @@ fn test_union_and_agg_projections() -> PolarsResult<()> {
     .into();
 
     let lf3: LazyFrame = DslBuilder::scan_csv(
-        ScanSources::Paths([PlPath::new(GLOB_CSV)].into()),
+        ScanSources::Paths(FromIterator::from_iter([PlPath::new(GLOB_CSV)])),
         Default::default(),
         UnifiedScanArgs {
             extra_columns_policy: ExtraColumnsPolicy::Ignore,
@@ -658,8 +655,12 @@ fn test_row_index_on_files() -> PolarsResult<()> {
             (offset..27 + offset).collect::<Vec<_>>()
         );
 
-        let lf = LazyFrame::scan_ipc(PlPath::new(FOODS_IPC), Default::default())?
-            .with_row_index("index", Some(offset));
+        let lf = LazyFrame::scan_ipc(
+            PlPath::new(FOODS_IPC),
+            Default::default(),
+            Default::default(),
+        )?
+        .with_row_index("index", Some(offset));
 
         assert!(row_index_at_scan(lf.clone()));
         let df = lf.clone().collect()?;
