@@ -51,9 +51,10 @@ fn test_lazy_unpivot() {
     let df = get_df();
 
     let args = UnpivotArgsDSL {
-        on: vec!["sepal_length".into(), "sepal_width".into()],
-        index: vec!["petal_width".into(), "petal_length".into()],
-        ..Default::default()
+        on: by_name(["sepal_length", "sepal_width"], true),
+        index: by_name(["petal_width", "petal_length"], true),
+        variable_name: None,
+        value_name: None,
     };
 
     let out = df
@@ -88,7 +89,7 @@ fn test_lazy_udf() {
     let df = get_df();
     let new = df
         .lazy()
-        .select([col("sepal_width").map(|s| Ok(Some(s * 200.0)), GetOutput::same_type())])
+        .select([col("sepal_width").map(|s| Ok(s * 200.0), |_, f| Ok(f.clone()))])
         .collect()
         .unwrap();
     assert_eq!(
@@ -247,7 +248,7 @@ fn test_lazy_query_2() {
     let df = load_df();
     let ldf = df
         .lazy()
-        .with_column(col("a").map(|s| Ok(Some(s * 2)), GetOutput::same_type()))
+        .with_column(col("a").map(|s| Ok(s * 2), |_, f| Ok(f.clone())))
         .filter(col("a").lt(lit(2)))
         .select([col("b"), col("a")]);
 
@@ -283,13 +284,10 @@ fn test_lazy_query_4() -> PolarsResult<()> {
         .agg([
             col("day").alias("day"),
             col("cumcases")
-                .apply(
-                    |s: Column| (&s - &(s.shift(1))).map(Some),
-                    GetOutput::same_type(),
-                )
+                .apply(|s: Column| &s - &(s.shift(1)), |_, f| Ok(f.clone()))
                 .alias("diff_cases"),
         ])
-        .explode([col("day"), col("diff_cases")])
+        .explode(by_name(["day", "diff_cases"], true))
         .join(
             base_df,
             [col("uid"), col("day")],
@@ -585,10 +583,11 @@ fn test_simplify_expr() {
         &mut OptFlags::SIMPLIFY_EXPR,
     )
     .unwrap();
-    let plan = node_to_lp(lp_top, &expr_arena, &mut lp_arena);
-    assert!(
-        matches!(plan, DslPlan::Select{ expr, ..} if matches!(&expr[0], Expr::BinaryExpr{left, ..} if **left == Expr::Literal(LiteralValue::Dyn(DynLiteralValue::Float(2.0)))))
-    );
+
+    assert!(matches!(
+        lp_arena.get(lp_top),
+        IR::Select { expr, .. }  if matches!(expr_arena.get(expr[0].node()), AExpr::BinaryExpr{ left, ..} if matches!(expr_arena.get(*left), &AExpr::Literal(LiteralValue::Dyn(DynLiteralValue::Float(2.0)))))
+    ));
 }
 
 #[test]
@@ -665,13 +664,12 @@ fn test_type_coercion() {
     let mut expr_arena = Arena::new();
     let mut lp_arena = Arena::new();
     let lp_top = to_alp(lp, &mut expr_arena, &mut lp_arena, &mut OptFlags::default()).unwrap();
-    let lp = node_to_lp(lp_top, &expr_arena, &mut lp_arena);
 
-    if let DslPlan::Select { expr, .. } = lp {
-        if let Expr::BinaryExpr { left, right, .. } = &expr[0] {
-            assert!(matches!(&**left, Expr::Cast { .. }));
+    if let IR::Select { expr, .. } = lp_arena.get(lp_top) {
+        if let AExpr::BinaryExpr { left, right, .. } = expr_arena.get(expr[0].node()) {
+            assert!(matches!(expr_arena.get(*left), AExpr::Cast { .. }));
             // bar is already float, does not have to be coerced
-            assert!(matches!(&**right, Expr::Column { .. }));
+            assert!(matches!(expr_arena.get(*right), AExpr::Column { .. }));
         } else {
             panic!()
         }
@@ -730,8 +728,8 @@ fn test_lazy_group_by_apply() {
     df.lazy()
         .group_by([col("fruits")])
         .agg([col("cars").apply(
-            |s: Column| Ok(Some(Column::new("".into(), &[s.len() as u32]))),
-            GetOutput::from_type(DataType::UInt32),
+            |s: Column| Ok(Column::new("".into(), &[s.len() as u32])),
+            |_, f| Ok(Field::new(f.name().clone(), DataType::UInt32)),
         )])
         .collect()
         .unwrap();
@@ -1102,7 +1100,7 @@ fn test_multiple_explode() -> PolarsResult<()> {
         .lazy()
         .group_by([col("a")])
         .agg([col("b").alias("b_list"), col("c").alias("c_list")])
-        .explode([col("c_list"), col("b_list")])
+        .explode(by_name(["c_list", "b_list"], true))
         .collect()?;
     assert_eq!(out.shape(), (5, 3));
 
@@ -1125,8 +1123,6 @@ fn test_filter_and_alias() -> PolarsResult<()> {
         "a" => [2, 2],
         "a_squared" => [4.0, 4.0]
     ]?;
-    println!("{out:?}");
-    println!("{expected:?}");
     assert!(out.equals(&expected));
     Ok(())
 }
@@ -1253,7 +1249,10 @@ fn test_exclude() -> PolarsResult<()> {
     "c" => [1, 2, 3]
     ]?;
 
-    let out = df.lazy().select([col("*").exclude(["b"])]).collect()?;
+    let out = df
+        .lazy()
+        .select([all().exclude_cols(["b"]).as_expr()])
+        .collect()?;
 
     assert_eq!(out.get_column_names(), &["a", "c"]);
     Ok(())
@@ -1403,8 +1402,8 @@ fn test_categorical_addition() -> PolarsResult<()> {
     let out = df
         .lazy()
         .select([
-            col("fruits").cast(DataType::Categorical(None, Default::default())),
-            col("cars").cast(DataType::Categorical(None, Default::default())),
+            col("fruits").cast(DataType::from_categories(Categories::global())),
+            col("cars").cast(DataType::from_categories(Categories::global())),
         ])
         .select([(col("fruits") + lit(" ") + col("cars")).alias("foo")])
         .collect()?;
@@ -1580,7 +1579,7 @@ fn test_exclude_regex() -> PolarsResult<()> {
     let df = fruits_cars();
     let out = df
         .lazy()
-        .select([col("*").exclude(["^(fruits|cars)$"])])
+        .select([(all() - Selector::Matches("^(fruits|cars)$".into())).as_expr()])
         .collect()?;
 
     assert_eq!(out.get_column_names(), &["A", "B"]);
@@ -1621,7 +1620,9 @@ pub fn test_select_by_dtypes() -> PolarsResult<()> {
     ]?;
     let out = df
         .lazy()
-        .select([dtype_cols([DataType::Float32, DataType::String])])
+        .select([dtype_cols([DataType::Float32, DataType::String])
+            .as_selector()
+            .as_expr()])
         .collect()?;
     assert_eq!(out.dtypes(), &[DataType::String, DataType::Float32]);
 
@@ -1660,15 +1661,7 @@ fn test_single_group_result() -> PolarsResult<()> {
 
     let out = df
         .lazy()
-        .select([col("a")
-            .arg_sort(SortOptions {
-                descending: false,
-                nulls_last: false,
-                multithreaded: true,
-                maintain_order: false,
-                limit: None,
-            })
-            .over([col("a")])])
+        .select([col("a").arg_sort(false, false).over([col("a")])])
         .collect()?;
 
     let a = out.column("a")?.idx()?;
@@ -1947,7 +1940,6 @@ fn test_sort_maintain_order_true() -> PolarsResult<()> {
         )
         .slice(0, 3)
         .collect()?;
-    println!("{res:?}");
     assert!(res.equals(&df![
         "A" => [1, 1, 1],
         "B" => ["A", "B", "C"],
@@ -1978,3 +1970,43 @@ fn test_over_with_options_empty_join() -> PolarsResult<()> {
 
     Ok(())
 }
+
+#[test]
+#[cfg(feature = "serde")]
+fn test_named_udfs() -> PolarsResult<()> {
+    use polars_plan::dsl::named_serde::{ExprRegistry, set_named_serde_registry};
+
+    let lf = DataFrame::new(vec![Column::new("a".into(), vec![1, 2, 3, 4])])?.lazy();
+
+    struct X;
+    impl ExprRegistry for X {
+        fn get_function(&self, name: &str, payload: &[u8]) -> Option<Arc<dyn AnonymousColumnsUdf>> {
+            assert_eq!(name, "test-function");
+            assert_eq!(payload, b"check");
+            Some(Arc::new(BaseColumnUdf::new(
+                |c: &mut [Column]| Ok(std::mem::take(&mut c[0]) * 2),
+                |_: &Schema, f: &[Field]| Ok(f[0].clone()),
+            )))
+        }
+    }
+
+    set_named_serde_registry(Arc::new(X) as _);
+
+    let expr = Expr::AnonymousFunction {
+        input: vec![Expr::Column("a".into())],
+        function: LazySerde::Named {
+            name: "test-function".into(),
+            payload: Some(bytes::Bytes::from("check")),
+            value: None,
+        },
+        options: FunctionOptions::default(),
+        fmt_str: Box::new("test".into()),
+    };
+
+    assert_eq!(
+        lf.select(&[expr]).collect()?,
+        DataFrame::new(vec![Column::new("a".into(), vec![2, 4, 6, 8])])?,
+    );
+
+    Ok(())
+}
