@@ -1,3 +1,5 @@
+use arrow::temporal_conversions::MICROSECONDS_IN_DAY as US_IN_DAY;
+
 use super::*;
 
 pub(super) fn median_with_nulls(ca: &ListChunked) -> Series {
@@ -10,13 +12,12 @@ pub(super) fn median_with_nulls(ca: &ListChunked) -> Series {
         },
         #[cfg(feature = "dtype-datetime")]
         DataType::Date => {
-            const MS_IN_DAY: i64 = 86_400_000;
             let out: Int64Chunked = ca
                 .apply_amortized_generic(|s| {
-                    s.and_then(|s| s.as_ref().median().map(|v| (v * (MS_IN_DAY as f64)) as i64))
+                    s.and_then(|s| s.as_ref().median().map(|v| (v * (US_IN_DAY as f64)) as i64))
                 })
                 .with_name(ca.name().clone());
-            out.into_datetime(TimeUnit::Milliseconds, None)
+            out.into_datetime(TimeUnit::Microseconds, None)
                 .into_series()
         },
         dt if dt.is_temporal() => {
@@ -58,39 +59,22 @@ pub(super) fn std_with_nulls(ca: &ListChunked, ddof: u8) -> Series {
     }
 }
 
-pub(super) fn var_with_nulls(ca: &ListChunked, ddof: u8) -> Series {
+pub(super) fn var_with_nulls(ca: &ListChunked, ddof: u8) -> PolarsResult<Series> {
     match ca.inner_dtype() {
         DataType::Float32 => {
             let out: Float32Chunked = ca
                 .apply_amortized_generic(|s| s.and_then(|s| s.as_ref().var(ddof).map(|v| v as f32)))
                 .with_name(ca.name().clone());
-            out.into_series()
+            Ok(out.into_series())
         },
-        #[cfg(feature = "dtype-duration")]
-        DataType::Duration(TimeUnit::Milliseconds) => {
-            let out: Int64Chunked = ca
-                .apply_amortized_generic(|s| s.and_then(|s| s.as_ref().var(ddof).map(|v| v as i64)))
-                .with_name(ca.name().clone());
-            out.into_duration(TimeUnit::Milliseconds).into_series()
-        },
-        #[cfg(feature = "dtype-duration")]
-        DataType::Duration(TimeUnit::Microseconds | TimeUnit::Nanoseconds) => {
-            let out: Int64Chunked = ca
-                .cast(&DataType::List(Box::new(DataType::Duration(
-                    TimeUnit::Milliseconds,
-                ))))
-                .unwrap()
-                .list()
-                .unwrap()
-                .apply_amortized_generic(|s| s.and_then(|s| s.as_ref().var(ddof).map(|v| v as i64)))
-                .with_name(ca.name().clone());
-            out.into_duration(TimeUnit::Milliseconds).into_series()
+        dt if dt.is_temporal() => {
+            polars_bail!(InvalidOperation: "variance of type {dt} is not supported")
         },
         _ => {
             let out: Float64Chunked = ca
                 .apply_amortized_generic(|s| s.and_then(|s| s.as_ref().var(ddof)))
                 .with_name(ca.name().clone());
-            out.into_series()
+            Ok(out.into_series())
         },
     }
 }
