@@ -276,16 +276,32 @@ where
 
                 use BitRepr as B;
                 match (left_by, right_by) {
-                    (B::Small(left_by), B::Small(right_by)) => {
+                    (B::U8(left_by), B::U8(right_by)) => {
+                        asof_join_by_numeric::<T, UInt8Type, A, F>(
+                            &left_by, &right_by, left_asof, right_asof, filter, allow_eq,
+                        )?
+                    },
+                    (B::U16(left_by), B::U16(right_by)) => {
+                        asof_join_by_numeric::<T, UInt16Type, A, F>(
+                            &left_by, &right_by, left_asof, right_asof, filter, allow_eq,
+                        )?
+                    },
+                    (B::U32(left_by), B::U32(right_by)) => {
                         asof_join_by_numeric::<T, UInt32Type, A, F>(
                             &left_by, &right_by, left_asof, right_asof, filter, allow_eq,
                         )?
                     },
-                    (B::Large(left_by), B::Large(right_by)) => {
+                    (B::U64(left_by), B::U64(right_by)) => {
                         asof_join_by_numeric::<T, UInt64Type, A, F>(
                             &left_by, &right_by, left_asof, right_asof, filter, allow_eq,
                         )?
                     },
+                    #[cfg(feature = "dtype-u128")]
+                    (B::U128(left_by), B::U128(right_by)) => {
+                        asof_join_by_numeric::<T, UInt128Type, A, F>(
+                            &left_by, &right_by, left_asof, right_asof, filter, allow_eq,
+                        )?
+                    },
                     // We have already asserted that the datatypes are the same.
                     _ => unreachable!(),
                 }
@@ -296,8 +312,6 @@ where
             polars_ensure!(lhs.dtype() == rhs.dtype(),
                 ComputeError: "mismatching dtypes in 'by' parameter of asof-join: `{}` and `{}`", lhs.dtype(), rhs.dtype()
             );
-            #[cfg(feature = "dtype-categorical")]
-            _check_categorical_src(lhs.dtype(), rhs.dtype())?;
         }
 
         // TODO: @scalar-opt.
@@ -416,6 +430,20 @@ fn dispatch_join_type(
                 ca, right_asof, left_by, right_by, strategy, tolerance, allow_eq,
             )
         },
+        #[cfg(feature = "dtype-i128")]
+        DataType::Int128 => {
+            let ca = left_asof.i128().unwrap();
+            dispatch_join_strategy_numeric(
+                ca, right_asof, left_by, right_by, strategy, tolerance, allow_eq,
+            )
+        },
+        #[cfg(feature = "dtype-u128")]
+        DataType::UInt128 => {
+            let ca = left_asof.u128().unwrap();
+            dispatch_join_strategy_numeric(
+                ca, right_asof, left_by, right_by, strategy, tolerance, allow_eq,
+            )
+        },
         DataType::Float32 => {
             let ca = left_asof.f32().unwrap();
             dispatch_join_strategy_numeric(
@@ -452,7 +480,7 @@ fn dispatch_join_type(
                 allow_eq,
             )
         },
-        _ => {
+        DataType::Int8 | DataType::UInt8 | DataType::Int16 | DataType::UInt16 => {
             let left_asof = left_asof.cast(&DataType::Int32).unwrap();
             let right_asof = right_asof.cast(&DataType::Int32).unwrap();
             let ca = left_asof.i32().unwrap();
@@ -466,6 +494,7 @@ fn dispatch_join_type(
                 allow_eq,
             )
         },
+        dt => polars_bail!(opq = asof_join, dt),
     }
 }
 
@@ -487,17 +516,15 @@ pub trait AsofJoinBy: IntoDf {
         allow_eq: bool,
         check_sortedness: bool,
     ) -> PolarsResult<DataFrame> {
-        let (self_sliced_slot, other_sliced_slot, left_slice_s, right_slice_s); // Keeps temporaries alive.
+        let (self_sliced_slot, left_slice_s); // Keeps temporaries alive.
         let (self_df, other_df, left_key, right_key);
         if let Some((offset, len)) = slice {
             self_sliced_slot = self.to_df().slice(offset, len);
-            other_sliced_slot = other.slice(offset, len);
             left_slice_s = left_on.slice(offset, len);
-            right_slice_s = right_on.slice(offset, len);
             left_key = &left_slice_s;
-            right_key = &right_slice_s;
+            right_key = right_on;
             self_df = &self_sliced_slot;
-            other_df = &other_sliced_slot;
+            other_df = other;
         } else {
             self_df = self.to_df();
             other_df = other;
@@ -526,8 +553,6 @@ pub trait AsofJoinBy: IntoDf {
                 .iter_mut()
                 .zip(right_by.get_columns_mut().iter_mut())
             {
-                #[cfg(feature = "dtype-categorical")]
-                _check_categorical_src(l.dtype(), r.dtype())?;
                 *l = l.to_physical_repr();
                 *r = r.to_physical_repr();
             }
