@@ -5,15 +5,16 @@ use arrow::bitmap::{Bitmap, BitmapBuilder};
 use arrow::datatypes::ArrowDataType;
 use polars_compute::filter::filter_boolean_kernel;
 
+use super::Filter;
 use super::dictionary_encoded::{append_validity, constrain_page_validity};
 use super::utils::{
     self, Decoded, Decoder, decode_hybrid_rle_into_bitmap, filter_from_range, freeze_validity,
 };
-use super::{Filter, PredicateFilter};
 use crate::parquet::encoding::Encoding;
 use crate::parquet::encoding::hybrid_rle::{HybridRleChunk, HybridRleDecoder};
 use crate::parquet::error::ParquetResult;
 use crate::parquet::page::{DataPage, DictPage, split_buffer};
+use crate::read::expr::{ParquetScalar, SpecializedParquetColumnExpr};
 
 #[allow(clippy::large_enum_variant)]
 #[derive(Debug)]
@@ -286,10 +287,15 @@ impl Decoded for (BitmapBuilder, BitmapBuilder) {
     fn len(&self) -> usize {
         self.0.len()
     }
+
     fn extend_nulls(&mut self, n: usize) {
         self.0.extend_constant(n, false);
         self.1.extend_constant(n, false);
     }
+
+    fn remaining_capacity(&self) -> usize {
+        self.0.capacity().min(self.1.capacity())
+    }
 }
 
 pub(crate) struct BooleanDecoder;
@@ -321,10 +327,12 @@ impl Decoder for BooleanDecoder {
         Ok(BooleanArray::new(dtype, values.freeze(), validity))
     }
 
-    fn has_predicate_specialization(
-        &self,
+    fn evaluate_predicate(
+        &mut self,
         _state: &utils::State<'_, Self>,
-        _predicate: &PredicateFilter,
+        _predicate: Option<&SpecializedParquetColumnExpr>,
+        _pred_true_mask: &mut BitmapBuilder,
+        _dict_mask: Option<&Bitmap>,
     ) -> ParquetResult<bool> {
         // @TODO: This can be enabled for the fast paths
         Ok(false)
@@ -351,8 +359,8 @@ impl Decoder for BooleanDecoder {
         &mut self,
         state: utils::State<'_, Self>,
         (target, validity): &mut Self::DecodedState,
-        _pred_true_mask: &mut BitmapBuilder,
         filter: Option<super::Filter>,
+        _chunks: &mut Vec<Self::Output>,
     ) -> ParquetResult<()> {
         match state.translation {
             StateTranslation::Plain(values) => {
@@ -438,13 +446,13 @@ impl Decoder for BooleanDecoder {
                         decode_masked_optional_rle(values, target, &page_validity, &filter)
                     },
                     (Some(Filter::Range(rng)), None) => {
-                        decode_masked_required_rle(values, target, &filter_from_range(rng.clone()))
+                        decode_masked_required_rle(values, target, &filter_from_range(rng))
                     },
                     (Some(Filter::Range(rng)), Some(page_validity)) => decode_masked_optional_rle(
                         values,
                         target,
                         &page_validity,
-                        &filter_from_range(rng.clone()),
+                        &filter_from_range(rng),
                     ),
                     (Some(Filter::Predicate(_)), _) => todo!(),
                 }?;
@@ -453,4 +461,23 @@ impl Decoder for BooleanDecoder {
             },
         }
     }
+
+    fn extend_constant(
+        &mut self,
+        decoded: &mut Self::DecodedState,
+        length: usize,
+        value: &ParquetScalar,
+    ) -> ParquetResult<()> {
+        if value.is_null() {
+            decoded.extend_nulls(length);
+            return Ok(());
+        }
+
+        let value = value.as_bool().unwrap();
+
+        decoded.0.extend_constant(length, value);
+        decoded.1.extend_constant(length, true);
+
+        Ok(())
+    }
 }
