@@ -2,6 +2,7 @@ use std::hash::Hash;
 use std::sync::Mutex;
 
 use deletion::DeletionFilesList;
+use polars_core::schema::iceberg::IcebergSchemaRef;
 use polars_core::utils::get_numeric_upcast_supertype_lossless;
 use polars_io::cloud::CloudOptions;
 #[cfg(feature = "csv")]
@@ -19,6 +20,8 @@ use serde::{Deserialize, Serialize};
 use strum_macros::IntoStaticStr;
 
 use super::*;
+use crate::dsl::default_values::DefaultFieldValues;
+pub mod default_values;
 pub mod deletion;
 
 #[cfg(feature = "python")]
@@ -37,7 +40,40 @@ bitflags::bitflags! {
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 // TODO: Arc<> some of the options and the cloud options.
-pub enum FileScan {
+pub enum FileScanDsl {
+    #[cfg(feature = "csv")]
+    Csv { options: CsvReadOptions },
+
+    #[cfg(feature = "json")]
+    NDJson { options: NDJsonReadOptions },
+
+    #[cfg(feature = "parquet")]
+    Parquet { options: ParquetOptions },
+
+    #[cfg(feature = "ipc")]
+    Ipc { options: IpcScanOptions },
+
+    #[cfg(feature = "python")]
+    PythonDataset {
+        dataset_object: Arc<python_dataset::PythonDatasetProvider>,
+    },
+
+    #[cfg(feature = "scan_lines")]
+    Lines { name: PlSmallStr },
+
+    #[cfg_attr(any(feature = "serde", feature = "dsl-schema"), serde(skip))]
+    Anonymous {
+        options: Arc<AnonymousScanOptions>,
+        function: Arc<dyn AnonymousScan>,
+        file_info: FileInfo,
+    },
+}
+
+#[derive(Clone, Debug, IntoStaticStr)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
+// TODO: Arc<> some of the options and the cloud options.
+pub enum FileScanIR {
     #[cfg(feature = "csv")]
     Csv { options: CsvReadOptions },
 
@@ -61,11 +97,12 @@ pub enum FileScan {
     #[cfg(feature = "python")]
     PythonDataset {
         dataset_object: Arc<python_dataset::PythonDatasetProvider>,
-
-        #[cfg_attr(any(feature = "serde", feature = "dsl-schema"), serde(skip, default))]
         cached_ir: Arc<Mutex<Option<ExpandedDataset>>>,
     },
 
+    #[cfg(feature = "scan_lines")]
+    Lines { name: PlSmallStr },
+
     #[cfg_attr(any(feature = "serde", feature = "dsl-schema"), serde(skip))]
     Anonymous {
         options: Arc<AnonymousScanOptions>,
@@ -73,7 +110,7 @@ pub enum FileScan {
     },
 }
 
-impl FileScan {
+impl FileScanIR {
     pub fn flags(&self) -> ScanFlags {
         match self {
             #[cfg(feature = "csv")]
@@ -150,6 +187,12 @@ pub struct CastColumnsPolicy {
     /// Allow casting to change time units.
     pub datetime_convert_timezone: bool,
 
+    /// DataType::Null to any
+    pub null_upcast: bool,
+
+    /// DataType::Categorical to string
+    pub categorical_to_string: bool,
+
     pub missing_struct_fields: MissingColumnsPolicy,
     pub extra_struct_fields: ExtraColumnsPolicy,
 }
@@ -163,6 +206,8 @@ impl CastColumnsPolicy {
         datetime_nanoseconds_downcast: false,
         datetime_microseconds_downcast: false,
         datetime_convert_timezone: false,
+        null_upcast: true,
+        categorical_to_string: false,
         missing_struct_fields: MissingColumnsPolicy::Raise,
         extra_struct_fields: ExtraColumnsPolicy::Raise,
     };
@@ -184,6 +229,40 @@ pub enum ExtraColumnsPolicy {
     Ignore,
 }
 
+#[derive(Debug, Clone, Eq, Hash, PartialEq, strum_macros::IntoStaticStr)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
+pub enum ColumnMapping {
+    Iceberg(IcebergSchemaRef),
+}
+
+#[derive(Debug, Clone)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
+pub struct TableStatistics(pub Arc<DataFrame>);
+
+impl PartialEq for TableStatistics {
+    fn eq(&self, other: &Self) -> bool {
+        Arc::ptr_eq(&self.0, &other.0)
+    }
+}
+
+impl Eq for TableStatistics {}
+
+impl Hash for TableStatistics {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        state.write_usize(Arc::as_ptr(&self.0) as *const () as usize);
+    }
+}
+
+impl std::ops::Deref for TableStatistics {
+    type Target = Arc<DataFrame>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 /// Scan arguments shared across different scan types.
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
@@ -198,8 +277,13 @@ pub struct UnifiedScanArgs {
     pub rechunk: bool,
     pub cache: bool,
     pub glob: bool,
+    /// Files with these prefixes will not be read.
+    pub hidden_file_prefix: Option<Arc<[PlSmallStr]>>,
 
     pub projection: Option<Arc<[PlSmallStr]>>,
+    pub column_mapping: Option<ColumnMapping>,
+    /// Default values for missing columns.
+    pub default_values: Option<DefaultFieldValues>,
     pub row_index: Option<RowIndex>,
     /// Slice applied before predicates
     pub pre_slice: Option<Slice>,
@@ -210,8 +294,21 @@ pub struct UnifiedScanArgs {
     pub include_file_paths: Option<PlSmallStr>,
 
     pub deletion_files: Option<DeletionFilesList>,
+    pub table_statistics: Option<TableStatistics>,
+    /// Stores (physical, deleted) row counts of the table if known upfront (e.g. for Iceberg).
+    /// This allows for row-count queries to succeed without scanning all files.
+    ///
+    /// Note, intentionally store u64 instead of IdxSize to avoid erroring if it's unused.
+    pub row_count: Option<(u64, u64)>,
 }
 
+impl UnifiedScanArgs {
+    pub fn has_row_index_or_slice(&self) -> bool {
+        self.row_index.is_some() || self.pre_slice.is_some()
+    }
+}
+
+// Manual default, we have `glob: true` by default.
 impl Default for UnifiedScanArgs {
     fn default() -> Self {
         Self {
@@ -221,7 +318,10 @@ impl Default for UnifiedScanArgs {
             rechunk: false,
             cache: false,
             glob: true,
+            hidden_file_prefix: None,
             projection: None,
+            column_mapping: None,
+            default_values: None,
             row_index: None,
             pre_slice: None,
             cast_columns_policy: CastColumnsPolicy::default(),
@@ -229,6 +329,8 @@ impl Default for UnifiedScanArgs {
             extra_columns_policy: ExtraColumnsPolicy::default(),
             include_file_paths: None,
             deletion_files: None,
+            table_statistics: None,
+            row_count: None,
         }
     }
 }
@@ -239,19 +341,22 @@ mod _file_scan_eq_hash {
     use std::hash::{Hash, Hasher};
     use std::sync::Arc;
 
-    use super::FileScan;
+    #[cfg(feature = "scan_lines")]
+    use polars_utils::pl_str::PlSmallStr;
 
-    impl PartialEq for FileScan {
-        fn eq(&self, other: &Self) -> bool {
-            FileScanEqHashWrap::from(self) == FileScanEqHashWrap::from(other)
+    use super::FileScanIR;
+
+    impl Eq for FileScanIR {}
+
+    impl Hash for FileScanIR {
+        fn hash<H: Hasher>(&self, state: &mut H) {
+            FileScanEqHashWrap::from(self).hash(state)
         }
     }
 
-    impl Eq for FileScan {}
-
-    impl Hash for FileScan {
-        fn hash<H: Hasher>(&self, state: &mut H) {
-            FileScanEqHashWrap::from(self).hash(state)
+    impl PartialEq for FileScanIR {
+        fn eq(&self, other: &Self) -> bool {
+            FileScanEqHashWrap::from(self) == FileScanEqHashWrap::from(other)
         }
     }
 
@@ -288,6 +393,9 @@ mod _file_scan_eq_hash {
             cached_ir: usize,
         },
 
+        #[cfg(feature = "scan_lines")]
+        Lines { name: &'a PlSmallStr },
+
         Anonymous {
             options: &'a crate::dsl::AnonymousScanOptions,
             function: usize,
@@ -298,29 +406,29 @@ mod _file_scan_eq_hash {
         Phantom(&'a ()),
     }
 
-    impl<'a> From<&'a FileScan> for FileScanEqHashWrap<'a> {
-        fn from(value: &'a FileScan) -> Self {
+    impl<'a> From<&'a FileScanIR> for FileScanEqHashWrap<'a> {
+        fn from(value: &'a FileScanIR) -> Self {
             match value {
                 #[cfg(feature = "csv")]
-                FileScan::Csv { options } => FileScanEqHashWrap::Csv { options },
+                FileScanIR::Csv { options } => FileScanEqHashWrap::Csv { options },
 
                 #[cfg(feature = "json")]
-                FileScan::NDJson { options } => FileScanEqHashWrap::NDJson { options },
+                FileScanIR::NDJson { options } => FileScanEqHashWrap::NDJson { options },
 
                 #[cfg(feature = "parquet")]
-                FileScan::Parquet { options, metadata } => FileScanEqHashWrap::Parquet {
+                FileScanIR::Parquet { options, metadata } => FileScanEqHashWrap::Parquet {
                     options,
                     metadata: metadata.as_ref().map(arc_as_ptr),
                 },
 
                 #[cfg(feature = "ipc")]
-                FileScan::Ipc { options, metadata } => FileScanEqHashWrap::Ipc {
+                FileScanIR::Ipc { options, metadata } => FileScanEqHashWrap::Ipc {
                     options,
                     metadata: metadata.as_ref().map(arc_as_ptr),
                 },
 
                 #[cfg(feature = "python")]
-                FileScan::PythonDataset {
+                FileScanIR::PythonDataset {
                     dataset_object,
                     cached_ir,
                 } => FileScanEqHashWrap::PythonDataset {
@@ -328,7 +436,10 @@ mod _file_scan_eq_hash {
                     cached_ir: arc_as_ptr(cached_ir),
                 },
 
-                FileScan::Anonymous { options, function } => FileScanEqHashWrap::Anonymous {
+                #[cfg(feature = "scan_lines")]
+                FileScanIR::Lines { name } => FileScanEqHashWrap::Lines { name },
+
+                FileScanIR::Anonymous { options, function } => FileScanEqHashWrap::Anonymous {
                     options,
                     function: arc_as_ptr(function),
                 },
@@ -368,6 +479,14 @@ impl CastColumnsPolicy {
             )
         };
 
+        if incoming_dtype.is_null() && !target_dtype.is_null() {
+            return if self.null_upcast {
+                Ok(true)
+            } else {
+                mismatch_err("unimplemented: 'null-upcast' in scan cast options")
+            };
+        }
+
         // We intercept the nested types first to prevent an expensive recursive eq - recursion
         // is instead done manually through this function.
 
@@ -478,6 +597,13 @@ impl CastColumnsPolicy {
 
         debug_assert!(!target_dtype.is_nested());
 
+        // If we were to drop cast on an `Unknown` incoming_dtype, it could eventually
+        // lead to dtype errors. The reason is that the logic used by type coercion differs
+        // from the casting logic used by `materialize_unknown`.
+        if incoming_dtype.contains_unknown() {
+            return Ok(true);
+        }
+
         // Note: Only call this with non-nested types for performance
         let materialize_unknown = |dtype: &DataType| -> std::borrow::Cow<DataType> {
             dtype
@@ -487,7 +613,7 @@ impl CastColumnsPolicy {
                 .unwrap_or(std::borrow::Cow::Borrowed(incoming_dtype))
         };
 
-        let incoming_dtype = materialize_unknown(incoming_dtype);
+        let incoming_dtype = std::borrow::Cow::Borrowed(incoming_dtype);
         let target_dtype = materialize_unknown(target_dtype);
 
         if target_dtype == incoming_dtype {
