@@ -37,6 +37,8 @@ pub mod python_dsl;
 mod random;
 mod scan_sources;
 mod selector;
+#[cfg(feature = "serde")]
+mod serializable_plan;
 mod statistics;
 #[cfg(feature = "strings")]
 pub mod string;
@@ -69,8 +71,9 @@ use polars_core::prelude::*;
 use polars_core::series::IsSorted;
 #[cfg(feature = "diff")]
 use polars_core::series::ops::NullBehavior;
-use polars_core::utils::try_get_supertype;
-pub use selector::Selector;
+#[cfg(feature = "is_close")]
+use polars_utils::total_ord::TotalOrdWrap;
+pub use selector::{DataTypeSelector, Selector, TimeUnitSet, TimeZoneSet};
 #[cfg(feature = "dtype-struct")]
 pub use struct_::*;
 pub use udf::UserDefinedFunction;
@@ -173,6 +176,15 @@ impl Expr {
         AggExpr::Last(Arc::new(self)).into()
     }
 
+    /// Get the single value in the group. If there are multiple values, an error is returned.
+    pub fn item(self, allow_empty: bool) -> Self {
+        AggExpr::Item {
+            input: Arc::new(self),
+            allow_empty,
+        }
+        .into()
+    }
+
     /// GroupBy the group to a Series.
     pub fn implode(self) -> Self {
         AggExpr::Implode(Arc::new(self)).into()
@@ -218,26 +230,12 @@ impl Expr {
 
     /// Append expressions. This is done by adding the chunks of `other` to this [`Series`].
     pub fn append<E: Into<Expr>>(self, other: E, upcast: bool) -> Self {
-        let output_type = if upcast {
-            GetOutput::super_type()
-        } else {
-            GetOutput::same_type()
-        };
+        self.map_binary(FunctionExpr::Append { upcast }, other.into())
+    }
 
-        apply_binary(
-            self,
-            other.into(),
-            move |mut a, mut b| {
-                if upcast {
-                    let dtype = try_get_supertype(a.dtype(), b.dtype())?;
-                    a = a.cast(&dtype)?;
-                    b = b.cast(&dtype)?;
-                }
-                a.append(&b)?;
-                Ok(Some(a))
-            },
-            output_type,
-        )
+    /// Collect all chunks into a single chunk before continuing.
+    pub fn rechunk(self) -> Self {
+        self.map_unary(FunctionExpr::Rechunk)
     }
 
     /// Get the first `n` elements of the Expr result.
@@ -269,47 +267,20 @@ impl Expr {
 
     /// Get the index value that has the minimum value.
     pub fn arg_min(self) -> Self {
-        self.agg_with_fmt_str(
-            move |c: Column| {
-                Ok(Some(Column::new(
-                    c.name().clone(),
-                    &[c.as_materialized_series().arg_min().map(|idx| idx as u32)],
-                )))
-            },
-            GetOutput::from_type(IDX_DTYPE),
-            "arg_min",
-        )
+        self.map_unary(FunctionExpr::ArgMin)
     }
 
     /// Get the index value that has the maximum value.
     pub fn arg_max(self) -> Self {
-        self.agg_with_fmt_str(
-            move |c: Column| {
-                Ok(Some(Column::new(
-                    c.name().clone(),
-                    &[c.as_materialized_series()
-                        .arg_max()
-                        .map(|idx| idx as IdxSize)],
-                )))
-            },
-            GetOutput::from_type(IDX_DTYPE),
-            "arg_max",
-        )
+        self.map_unary(FunctionExpr::ArgMax)
     }
 
     /// Get the index values that would sort this expression.
-    pub fn arg_sort(self, sort_options: SortOptions) -> Self {
-        self.apply_with_fmt_str(
-            move |c: Column| {
-                Ok(Some(
-                    c.as_materialized_series()
-                        .arg_sort(sort_options)
-                        .into_column(),
-                ))
-            },
-            GetOutput::from_type(IDX_DTYPE),
-            "arg_sort",
-        )
+    pub fn arg_sort(self, descending: bool, nulls_last: bool) -> Self {
+        self.map_unary(FunctionExpr::ArgSort {
+            descending,
+            nulls_last,
+        })
     }
 
     #[cfg(feature = "index_of")]
@@ -486,23 +457,28 @@ impl Expr {
     ///
     /// It is the responsibility of the caller that the schema is correct by giving
     /// the correct output_type. If None given the output type of the input expr is used.
-    pub fn map<F>(self, function: F, output_type: GetOutput) -> Self
+    pub fn map<F, DT>(self, function: F, output_type: DT) -> Self
     where
-        F: Fn(Column) -> PolarsResult<Option<Column>> + 'static + Send + Sync,
+        F: Fn(Column) -> PolarsResult<Column> + 'static + Send + Sync,
+        DT: Fn(&Schema, &Field) -> PolarsResult<Field> + 'static + Send + Sync,
     {
         self.map_with_fmt_str(function, output_type, "map")
     }
 
-    pub fn map_with_fmt_str<F>(
+    pub fn map_with_fmt_str<F, DT>(
         self,
         function: F,
-        output_type: GetOutput,
+        output_type: DT,
         fmt_str: impl Into<PlSmallStr>,
     ) -> Self
     where
-        F: Fn(Column) -> PolarsResult<Option<Column>> + 'static + Send + Sync,
+        F: Fn(Column) -> PolarsResult<Column> + 'static + Send + Sync,
+        DT: Fn(&Schema, &Field) -> PolarsResult<Field> + 'static + Send + Sync,
     {
-        let f = move |c: &mut [Column]| function(std::mem::take(&mut c[0]));
+        let f = BaseColumnUdf::new(
+            move |c: &mut [Column]| function(std::mem::take(&mut c[0])),
+            move |schema: &Schema, fields: &[Field]| output_type(schema, &fields[0]),
+        );
 
         let options =
             FunctionOptions::elementwise().with_flags(|f| f | FunctionFlags::OPTIONAL_RE_ENTRANT);
@@ -510,51 +486,56 @@ impl Expr {
         Expr::AnonymousFunction {
             input: vec![self],
             function: new_column_udf(f),
-            output_type,
             options,
             fmt_str,
         }
     }
 
-    pub fn agg_with_fmt_str<F>(
+    pub fn agg_with_fmt_str<F, DT>(
         self,
         function: F,
-        output_type: GetOutput,
+        output_type: DT,
         fmt_str: impl Into<PlSmallStr>,
     ) -> Self
     where
-        F: Fn(Column) -> PolarsResult<Option<Column>> + 'static + Send + Sync,
+        F: Fn(Column) -> PolarsResult<Column> + 'static + Send + Sync,
+        DT: Fn(&Schema, &Field) -> PolarsResult<Field> + 'static + Send + Sync,
     {
-        let f = move |c: &mut [Column]| function(std::mem::take(&mut c[0]));
+        let f = BaseColumnUdf::new(
+            move |c: &mut [Column]| function(std::mem::take(&mut c[0])),
+            move |schema: &Schema, fields: &[Field]| output_type(schema, &fields[0]),
+        );
 
         let options = FunctionOptions::aggregation();
         let fmt_str = Box::new(fmt_str.into());
         Expr::AnonymousFunction {
             input: vec![self],
             function: new_column_udf(f),
-            output_type,
             options,
             fmt_str,
         }
     }
 
-    pub fn apply_with_fmt_str<F>(
+    pub fn apply_with_fmt_str<F, DT>(
         self,
         function: F,
-        output_type: GetOutput,
+        output_type: DT,
         fmt_str: impl Into<PlSmallStr>,
     ) -> Self
     where
-        F: Fn(Column) -> PolarsResult<Option<Column>> + 'static + Send + Sync,
+        F: Fn(Column) -> PolarsResult<Column> + 'static + Send + Sync,
+        DT: Fn(&Schema, &Field) -> PolarsResult<Field> + 'static + Send + Sync,
     {
-        let f = move |c: &mut [Column]| function(std::mem::take(&mut c[0]));
+        let f = BaseColumnUdf::new(
+            move |c: &mut [Column]| function(std::mem::take(&mut c[0])),
+            move |schema: &Schema, fields: &[Field]| output_type(schema, &fields[0]),
+        );
 
         let options = FunctionOptions::groupwise();
         let fmt_str = Box::new(fmt_str.into());
         Expr::AnonymousFunction {
             input: vec![self],
             function: new_column_udf(f),
-            output_type,
             options,
             fmt_str,
         }
@@ -563,18 +544,20 @@ impl Expr {
     /// Apply a function/closure once the logical plan get executed with many arguments.
     ///
     /// See the [`Expr::map`] function for the differences between [`map`](Expr::map) and [`apply`](Expr::apply).
-    pub fn map_many<F>(self, function: F, arguments: &[Expr], output_type: GetOutput) -> Self
+    pub fn map_many<F, DT>(self, function: F, arguments: &[Expr], output_type: DT) -> Self
     where
-        F: Fn(&mut [Column]) -> PolarsResult<Option<Column>> + 'static + Send + Sync,
+        F: Fn(&mut [Column]) -> PolarsResult<Column> + 'static + Send + Sync,
+        DT: Fn(&Schema, &[Field]) -> PolarsResult<Field> + 'static + Send + Sync,
     {
         let mut input = vec![self];
         input.extend_from_slice(arguments);
 
+        let function = BaseColumnUdf::new(function, output_type);
+
         let options = FunctionOptions::elementwise();
         Expr::AnonymousFunction {
             input,
             function: new_column_udf(function),
-            output_type,
             options,
             fmt_str: Box::new(PlSmallStr::EMPTY),
         }
@@ -589,37 +572,31 @@ impl Expr {
     ///
     /// * `map` should be used for operations that are independent of groups, e.g. `multiply * 2`, or `raise to the power`
     /// * `apply` should be used for operations that work on a group of data. e.g. `sum`, `count`, etc.
-    pub fn apply<F>(self, function: F, output_type: GetOutput) -> Self
+    pub fn apply<F, DT>(self, function: F, output_type: DT) -> Self
     where
-        F: Fn(Column) -> PolarsResult<Option<Column>> + 'static + Send + Sync,
+        F: Fn(Column) -> PolarsResult<Column> + 'static + Send + Sync,
+        DT: Fn(&Schema, &Field) -> PolarsResult<Field> + 'static + Send + Sync,
     {
-        let f = move |c: &mut [Column]| function(std::mem::take(&mut c[0]));
-
-        let options = FunctionOptions::groupwise();
-        Expr::AnonymousFunction {
-            input: vec![self],
-            function: new_column_udf(f),
-            output_type,
-            options,
-            fmt_str: Box::new(PlSmallStr::EMPTY),
-        }
+        self.apply_with_fmt_str(function, output_type, PlSmallStr::EMPTY)
     }
 
     /// Apply a function/closure over the groups with many arguments. This should only be used in a group_by aggregation.
     ///
     /// See the [`Expr::apply`] function for the differences between [`map`](Expr::map) and [`apply`](Expr::apply).
-    pub fn apply_many<F>(self, function: F, arguments: &[Expr], output_type: GetOutput) -> Self
+    pub fn apply_many<F, DT>(self, function: F, arguments: &[Expr], output_type: DT) -> Self
     where
-        F: Fn(&mut [Column]) -> PolarsResult<Option<Column>> + 'static + Send + Sync,
+        F: Fn(&mut [Column]) -> PolarsResult<Column> + 'static + Send + Sync,
+        DT: Fn(&Schema, &[Field]) -> PolarsResult<Field> + 'static + Send + Sync,
     {
         let mut input = vec![self];
         input.extend_from_slice(arguments);
 
+        let function = BaseColumnUdf::new(function, output_type);
+
         let options = FunctionOptions::groupwise();
         Expr::AnonymousFunction {
             input,
             function: new_column_udf(function),
-            output_type,
             options,
             fmt_str: Box::new(PlSmallStr::EMPTY),
         }
@@ -699,27 +676,7 @@ impl Expr {
 
     /// Get the product aggregation of an expression.
     pub fn product(self) -> Self {
-        self.agg_with_fmt_str(
-            move |c: Column| {
-                Some(
-                    c.product()
-                        .map(|sc| sc.into_series(c.name().clone()).into_column()),
-                )
-                .transpose()
-            },
-            GetOutput::map_dtype(|dt| {
-                use DataType as T;
-                Ok(match dt {
-                    T::Float32 => T::Float32,
-                    T::Float64 => T::Float64,
-                    T::UInt64 => T::UInt64,
-                    #[cfg(feature = "dtype-i128")]
-                    T::Int128 => T::Int128,
-                    _ => T::Int64,
-                })
-            }),
-            "product",
-        )
+        self.map_unary(FunctionExpr::Product)
     }
 
     /// Round underlying floating point array to given decimal numbers.
@@ -927,11 +884,19 @@ impl Expr {
     /// or
     /// Get counts of the group by operation.
     pub fn count(self) -> Self {
-        AggExpr::Count(Arc::new(self), false).into()
+        AggExpr::Count {
+            input: Arc::new(self),
+            include_nulls: false,
+        }
+        .into()
     }
 
     pub fn len(self) -> Self {
-        AggExpr::Count(Arc::new(self), true).into()
+        AggExpr::Count {
+            input: Arc::new(self),
+            include_nulls: true,
+        }
+        .into()
     }
 
     /// Get a mask of duplicated values.
@@ -958,6 +923,26 @@ impl Expr {
         self.map_unary(BooleanFunction::IsUnique)
     }
 
+    /// Check whether floating point values are close to each other.
+    #[allow(clippy::wrong_self_convention)]
+    #[cfg(feature = "is_close")]
+    pub fn is_close<E: Into<Expr>>(
+        self,
+        expr: E,
+        abs_tol: f64,
+        rel_tol: f64,
+        nans_equal: bool,
+    ) -> Self {
+        self.map_binary(
+            BooleanFunction::IsClose {
+                abs_tol: TotalOrdWrap(abs_tol),
+                rel_tol: TotalOrdWrap(rel_tol),
+                nans_equal,
+            },
+            expr.into(),
+        )
+    }
+
     /// Get the approximate count of unique values.
     #[cfg(feature = "approx_unique")]
     pub fn approx_n_unique(self) -> Self {
@@ -994,9 +979,6 @@ impl Expr {
     /// Should be used in aggregation context. If you want to filter on a
     /// DataFrame level, use `LazyFrame::filter`.
     pub fn filter<E: Into<Expr>>(self, predicate: E) -> Self {
-        if has_expr(&self, |e| matches!(e, Expr::Wildcard)) {
-            panic!("filter '*' not allowed, use LazyFrame::filter")
-        };
         Expr::Filter {
             input: Arc::new(self),
             by: Arc::new(predicate.into()),
@@ -1091,23 +1073,6 @@ impl Expr {
         self.map_unary(FunctionExpr::Mode)
     }
 
-    /// Exclude a column from a wildcard/regex selection.
-    ///
-    /// You may also use regexes in the exclude as long as they start with `^` and end with `$`.
-    pub fn exclude(self, columns: impl IntoVec<PlSmallStr>) -> Expr {
-        let v = columns.into_vec().into_iter().map(Excluded::Name).collect();
-        Expr::Exclude(Arc::new(self), v)
-    }
-
-    pub fn exclude_dtype<D: AsRef<[DataType]>>(self, dtypes: D) -> Expr {
-        let v = dtypes
-            .as_ref()
-            .iter()
-            .map(|dt| Excluded::Dtype(dt.clone()))
-            .collect();
-        Expr::Exclude(Arc::new(self), v)
-    }
-
     #[cfg(feature = "interpolate")]
     /// Interpolate intermediate values.
     /// Nulls at the beginning and end of the series remain null.
@@ -1121,10 +1086,13 @@ impl Expr {
         self,
         by: Expr,
         options: RollingOptionsDynamicWindow,
-        rolling_function_by: fn(RollingOptionsDynamicWindow) -> RollingFunctionBy,
+        rolling_function_by: RollingFunctionBy,
     ) -> Expr {
         self.map_binary(
-            FunctionExpr::RollingExprBy(rolling_function_by(options)),
+            FunctionExpr::RollingExprBy {
+                function_by: rolling_function_by,
+                options,
+            },
             by,
         )
     }
@@ -1142,9 +1110,12 @@ impl Expr {
     fn finish_rolling(
         self,
         options: RollingOptionsFixedWindow,
-        rolling_function: fn(RollingOptionsFixedWindow) -> RollingFunction,
+        rolling_function: RollingFunction,
     ) -> Expr {
-        self.map_unary(FunctionExpr::RollingExpr(rolling_function(options)))
+        self.map_unary(FunctionExpr::RollingExpr {
+            function: rolling_function,
+            options,
+        })
     }
 
     /// Apply a rolling minimum based on another column.
@@ -1207,6 +1178,11 @@ impl Expr {
         self.rolling_quantile_by(by, QuantileMethod::Linear, 0.5, options)
     }
 
+    #[cfg(feature = "rolling_window_by")]
+    pub fn rolling_rank_by(self, by: Expr, options: RollingOptionsDynamicWindow) -> Expr {
+        self.finish_rolling_by(by, options, RollingFunctionBy::RankBy)
+    }
+
     /// Apply a rolling minimum.
     ///
     /// See: [`RollingAgg::rolling_min`]
@@ -1279,6 +1255,11 @@ impl Expr {
         self.finish_rolling(options, RollingFunction::Std)
     }
 
+    #[cfg(feature = "rolling_window")]
+    pub fn rolling_rank(self, options: RollingOptionsFixedWindow) -> Expr {
+        self.finish_rolling(options, RollingFunction::Rank)
+    }
+
     /// Apply a rolling skew.
     #[cfg(feature = "rolling_window")]
     #[cfg(feature = "moment")]
@@ -1298,60 +1279,10 @@ impl Expr {
     /// This has quite some dynamic dispatch, so prefer rolling_min, max, mean, sum over this.
     pub fn rolling_map(
         self,
-        f: Arc<dyn Fn(&Series) -> Series + Send + Sync>,
-        output_type: GetOutput,
+        f: PlanCallback<Series, Series>,
         options: RollingOptionsFixedWindow,
     ) -> Expr {
-        self.apply_with_fmt_str(
-            move |c: Column| {
-                c.as_materialized_series()
-                    .rolling_map(f.as_ref(), options.clone())
-                    .map(Column::from)
-                    .map(Some)
-            },
-            output_type,
-            "rolling_map",
-        )
-    }
-
-    #[cfg(feature = "rolling_window")]
-    /// Apply a custom function over a rolling/ moving window of the array.
-    /// Prefer this over rolling_apply in case of floating point numbers as this is faster.
-    /// This has quite some dynamic dispatch, so prefer rolling_min, max, mean, sum over this.
-    pub fn rolling_map_float<F>(self, window_size: usize, f: F) -> Expr
-    where
-        F: 'static + FnMut(&mut Float64Chunked) -> Option<f64> + Send + Sync + Copy,
-    {
-        self.apply_with_fmt_str(
-            move |c: Column| {
-                let out = match c.dtype() {
-                    DataType::Float64 => c
-                        .f64()
-                        .unwrap()
-                        .rolling_map_float(window_size, f)
-                        .map(|ca| ca.into_column()),
-                    _ => c
-                        .cast(&DataType::Float64)?
-                        .f64()
-                        .unwrap()
-                        .rolling_map_float(window_size, f)
-                        .map(|ca| ca.into_column()),
-                }?;
-                if let DataType::Float32 = c.dtype() {
-                    out.cast(&DataType::Float32).map(Some)
-                } else {
-                    Ok(Some(out))
-                }
-            },
-            GetOutput::map_field(|field| {
-                Ok(match field.dtype() {
-                    DataType::Float64 => field.clone(),
-                    DataType::Float32 => Field::new(field.name().clone(), DataType::Float32),
-                    _ => Field::new(field.name().clone(), DataType::Float64),
-                })
-            }),
-            "rolling_map_float",
-        )
+        self.finish_rolling(options, RollingFunction::Map(f))
     }
 
     #[cfg(feature = "peaks")]
@@ -1570,13 +1501,6 @@ impl Expr {
         self.map_unary(BooleanFunction::All { ignore_nulls })
     }
 
-    /// Shrink numeric columns to the minimal required datatype
-    /// needed to fit the extrema of this [`Series`].
-    /// This can be used to reduce memory pressure.
-    pub fn shrink_dtype(self) -> Self {
-        self.map_unary(FunctionExpr::ShrinkType)
-    }
-
     #[cfg(feature = "dtype-struct")]
     /// Count all unique values and create a struct mapping value to count.
     /// (Note that it is better to turn parallel off in the aggregation context).
@@ -1600,8 +1524,8 @@ impl Expr {
 
     #[cfg(feature = "log")]
     /// Compute the logarithm to a given base.
-    pub fn log(self, base: f64) -> Self {
-        self.map_unary(FunctionExpr::Log { base })
+    pub fn log(self, base: Expr) -> Self {
+        self.map_binary(FunctionExpr::Log, base)
     }
 
     #[cfg(feature = "log")]
@@ -1617,7 +1541,7 @@ impl Expr {
     }
 
     #[cfg(feature = "log")]
-    /// Compute the entropy as `-sum(pk * log(pk)`.
+    /// Compute the entropy as `-sum(pk * log(pk))`.
     /// where `pk` are discrete probabilities.
     pub fn entropy(self, base: f64, normalize: bool) -> Self {
         self.map_unary(FunctionExpr::Entropy { base, normalize })
@@ -1721,18 +1645,20 @@ impl Expr {
 ///
 /// It is the responsibility of the caller that the schema is correct by giving
 /// the correct output_type. If None given the output type of the input expr is used.
-pub fn map_multiple<F, E>(function: F, expr: E, output_type: GetOutput) -> Expr
+pub fn map_multiple<F, DT, E>(function: F, expr: E, output_type: DT) -> Expr
 where
-    F: Fn(&mut [Column]) -> PolarsResult<Option<Column>> + 'static + Send + Sync,
+    F: Fn(&mut [Column]) -> PolarsResult<Column> + 'static + Send + Sync,
+    DT: Fn(&Schema, &[Field]) -> PolarsResult<Field> + 'static + Send + Sync,
     E: AsRef<[Expr]>,
 {
     let input = expr.as_ref().to_vec();
 
+    let function = BaseColumnUdf::new(function, output_type);
+
     let options = FunctionOptions::elementwise();
     Expr::AnonymousFunction {
         input,
         function: new_column_udf(function),
-        output_type,
         options,
         fmt_str: Box::new(PlSmallStr::EMPTY),
     }
@@ -1747,14 +1673,10 @@ where
 ///
 /// * [`map_multiple`] should be used for operations that are independent of groups, e.g. `multiply * 2`, or `raise to the power`
 /// * [`apply_multiple`] should be used for operations that work on a group of data. e.g. `sum`, `count`, etc.
-pub fn apply_multiple<F, E>(
-    function: F,
-    expr: E,
-    output_type: GetOutput,
-    returns_scalar: bool,
-) -> Expr
+pub fn apply_multiple<F, DT, E>(function: F, expr: E, output_type: DT, returns_scalar: bool) -> Expr
 where
-    F: Fn(&mut [Column]) -> PolarsResult<Option<Column>> + 'static + Send + Sync,
+    F: Fn(&mut [Column]) -> PolarsResult<Column> + 'static + Send + Sync,
+    DT: Fn(&Schema, &[Field]) -> PolarsResult<Field> + 'static + Send + Sync,
     E: AsRef<[Expr]>,
 {
     let input = expr.as_ref().to_vec();
@@ -1762,10 +1684,10 @@ where
         f.set(FunctionFlags::RETURNS_SCALAR, returns_scalar);
         f
     });
+    let function = BaseColumnUdf::new(function, output_type);
     Expr::AnonymousFunction {
         input,
         function: new_column_udf(function),
-        output_type,
         options,
         fmt_str: Box::new(PlSmallStr::EMPTY),
     }
@@ -1777,16 +1699,19 @@ pub fn len() -> Expr {
 }
 
 /// First column in a DataFrame.
-pub fn first() -> Expr {
-    Expr::Nth(0)
+pub fn first() -> Selector {
+    nth(0)
 }
 
 /// Last column in a DataFrame.
-pub fn last() -> Expr {
-    Expr::Nth(-1)
+pub fn last() -> Selector {
+    nth(-1)
 }
 
 /// Nth column in a DataFrame.
-pub fn nth(n: i64) -> Expr {
-    Expr::Nth(n)
+pub fn nth(n: i64) -> Selector {
+    Selector::ByIndex {
+        indices: [n].into(),
+        strict: true,
+    }
 }
