@@ -1,5 +1,6 @@
 use std::fmt;
 use std::hash::{Hash, Hasher};
+use std::num::NonZeroUsize;
 use std::path::PathBuf;
 use std::sync::Arc;
 
@@ -17,6 +18,7 @@ use polars_utils::plpath::PlPath;
 
 use super::{ExprIR, FileType};
 use crate::dsl::{AExpr, Expr, SpecialEq};
+use crate::prelude::PlanCallback;
 
 /// Options that apply to all sinks.
 #[derive(Clone, PartialEq, Eq, Debug, Hash)]
@@ -178,10 +180,20 @@ pub struct FileSinkType {
     pub cloud_options: Option<polars_io::cloud::CloudOptions>,
 }
 
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
+#[derive(Clone, Debug, PartialEq, Hash)]
+pub struct CallbackSinkType {
+    pub function: PlanCallback<DataFrame, bool>,
+    pub maintain_order: bool,
+    pub chunk_size: Option<NonZeroUsize>,
+}
+
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[derive(Clone, Debug, PartialEq)]
 pub enum SinkTypeIR {
     Memory,
+    Callback(CallbackSinkType),
     File(FileSinkType),
     #[cfg_attr(all(feature = "serde", not(feature = "ir_serde")), serde(skip))]
     Partition(PartitionSinkTypeIR),
@@ -253,10 +265,10 @@ impl PartitionTargetContextKey {
         Ok(value.to_string())
     }
     #[getter]
-    pub fn raw_value(&self) -> pyo3::PyObject {
+    pub fn raw_value(&self) -> pyo3::Py<pyo3::PyAny> {
         let converter = polars_core::chunked_array::object::registry::get_pyobject_converter();
         *(converter.as_ref())(self.raw_value.as_any_value())
-            .downcast::<pyo3::PyObject>()
+            .downcast::<pyo3::Py<pyo3::PyAny>>()
             .unwrap()
     }
 }
@@ -306,7 +318,7 @@ impl SinkFinishCallback {
         match self {
             Self::Rust(f) => f(df),
             #[cfg(feature = "python")]
-            Self::Python(f) => pyo3::Python::with_gil(|py| {
+            Self::Python(f) => pyo3::Python::attach(|py| {
                 let converter =
                     polars_utils::python_convert_registry::get_python_convert_registry();
                 let df = (converter.to_py.df)(Box::new(df) as Box<dyn std::any::Any>)?;
@@ -315,6 +327,26 @@ impl SinkFinishCallback {
             }),
         }
     }
+
+    pub fn display_str(&self) -> PlSmallStr {
+        match self {
+            Self::Rust(_) => PlSmallStr::from_static("Rust(<dyn Fn>)"),
+            #[cfg(feature = "python")]
+            Self::Python(f) => pyo3::Python::attach(|py| {
+                use polars_utils::format_pl_smallstr;
+                use pyo3::intern;
+                use pyo3::pybacked::PyBackedStr;
+
+                let class_name: PyBackedStr = f
+                    .getattr(py, intern!(py, "__class__"))
+                    .unwrap()
+                    .extract(py)
+                    .unwrap();
+
+                format_pl_smallstr!("Python({class_name})")
+            }),
+        }
+    }
 }
 
 #[derive(Clone)]
@@ -328,7 +360,7 @@ impl PartitionTargetCallback {
         match self {
             Self::Rust(f) => f(ctx),
             #[cfg(feature = "python")]
-            Self::Python(f) => pyo3::Python::with_gil(|py| {
+            Self::Python(f) => pyo3::Python::attach(|py| {
                 let partition_target = f.call1(py, (ctx,))?;
                 let converter =
                     polars_utils::python_convert_registry::get_python_convert_registry();
@@ -342,6 +374,26 @@ impl PartitionTargetCallback {
             }),
         }
     }
+
+    pub fn display_str(&self) -> PlSmallStr {
+        match self {
+            Self::Rust(_) => PlSmallStr::from_static("Rust(<dyn Fn>)"),
+            #[cfg(feature = "python")]
+            Self::Python(f) => pyo3::Python::attach(|py| {
+                use polars_utils::format_pl_smallstr;
+                use pyo3::intern;
+                use pyo3::pybacked::PyBackedStr;
+
+                let class_name: PyBackedStr = f
+                    .getattr(py, intern!(py, "__class__"))
+                    .unwrap()
+                    .extract(py)
+                    .unwrap();
+
+                format_pl_smallstr!("Python({class_name})")
+            }),
+        }
+    }
 }
 
 #[cfg(feature = "serde")]
@@ -501,6 +553,7 @@ pub struct PartitionSinkTypeIR {
 #[derive(Clone, Debug, PartialEq)]
 pub enum SinkType {
     Memory,
+    Callback(CallbackSinkType),
     File(FileSinkType),
     Partition(PartitionSinkType),
 }
@@ -521,7 +574,7 @@ pub enum PartitionVariant {
 }
 
 #[cfg_attr(feature = "ir_serde", derive(serde::Serialize, serde::Deserialize))]
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Clone, Debug, PartialEq, Eq, strum_macros::IntoStaticStr)]
 pub enum PartitionVariantIR {
     MaxSize(IdxSize),
     Parted {
@@ -540,6 +593,7 @@ impl SinkTypeIR {
         std::mem::discriminant(self).hash(state);
         match self {
             Self::Memory => {},
+            Self::Callback(f) => f.hash(state),
             Self::File(f) => f.hash(state),
             Self::Partition(f) => f.traverse_and_hash(expr_arena, state),
         }
@@ -552,6 +606,7 @@ impl SinkTypeIR {
             SinkTypeIR::Memory => true,
             SinkTypeIR::File(s) => s.sink_options.maintain_order,
             SinkTypeIR::Partition(s) => s.sink_options.maintain_order,
+            SinkTypeIR::Callback(s) => s.maintain_order,
         }
     }
 }
