@@ -1,79 +1,41 @@
-#[cfg(feature = "abs")]
-mod abs;
-#[cfg(feature = "arg_where")]
-mod arg_where;
 #[cfg(feature = "dtype-array")]
 mod array;
 mod binary;
 #[cfg(feature = "bitwise")]
 mod bitwise;
 mod boolean;
-mod bounds;
 #[cfg(feature = "business")]
 mod business;
 #[cfg(feature = "dtype-categorical")]
 mod cat;
-#[cfg(feature = "round_series")]
-mod clip;
-#[cfg(feature = "dtype-struct")]
-mod coerce;
-mod concat;
 #[cfg(feature = "cov")]
 mod correlation;
 #[cfg(feature = "cum_agg")]
 mod cum;
-#[cfg(feature = "cutqcut")]
-mod cut;
 #[cfg(feature = "temporal")]
 mod datetime;
-mod dispatch;
-#[cfg(feature = "ewma")]
-mod ewm;
-#[cfg(feature = "ewma_by")]
-mod ewm_by;
-mod fill_null;
 #[cfg(feature = "fused")]
 mod fused;
-#[cfg(feature = "index_of")]
-mod index_of;
 mod list;
-#[cfg(feature = "log")]
-mod log;
-mod nan;
-#[cfg(feature = "peaks")]
-mod peaks;
 #[cfg(feature = "ffi_plugin")]
-mod plugin;
+pub mod plugin;
 mod pow;
 #[cfg(feature = "random")]
 mod random;
 #[cfg(feature = "range")]
 mod range;
-mod repeat;
 #[cfg(feature = "rolling_window")]
 mod rolling;
 #[cfg(feature = "rolling_window_by")]
 mod rolling_by;
-#[cfg(feature = "round_series")]
-mod round;
-#[cfg(feature = "row_hash")]
-mod row_hash;
+mod row_encode;
 pub(super) mod schema;
-#[cfg(feature = "search_sorted")]
-mod search_sorted;
-mod shift_and_fill;
-mod shrink_type;
-#[cfg(feature = "sign")]
-mod sign;
 #[cfg(feature = "strings")]
 mod strings;
 #[cfg(feature = "dtype-struct")]
 mod struct_;
-#[cfg(feature = "temporal")]
-mod temporal;
 #[cfg(feature = "trigonometry")]
 mod trigonometry;
-mod unique;
 
 use std::fmt::{Display, Formatter};
 use std::hash::{Hash, Hasher};
@@ -111,8 +73,9 @@ pub use self::range::IRRangeFunction;
 pub use self::rolling::IRRollingFunction;
 #[cfg(feature = "rolling_window_by")]
 pub use self::rolling_by::IRRollingFunctionBy;
+pub use self::row_encode::RowEncodingVariant;
 #[cfg(feature = "strings")]
-pub use self::strings::IRStringFunction;
+pub use self::strings::{IRStringFunction, TZ_AWARE_RE};
 #[cfg(feature = "dtype-struct")]
 pub use self::struct_::IRStructFunction;
 #[cfg(feature = "trigonometry")]
@@ -175,9 +138,19 @@ pub enum IRFunctionExpr {
     FillNull,
     FillNullWithStrategy(FillNullStrategy),
     #[cfg(feature = "rolling_window")]
-    RollingExpr(IRRollingFunction),
+    RollingExpr {
+        function: IRRollingFunction,
+        options: RollingOptionsFixedWindow,
+    },
     #[cfg(feature = "rolling_window_by")]
-    RollingExprBy(IRRollingFunctionBy),
+    RollingExprBy {
+        function_by: IRRollingFunctionBy,
+        options: RollingOptionsDynamicWindow,
+    },
+    Rechunk,
+    Append {
+        upcast: bool,
+    },
     ShiftAndFill,
     Shift,
     DropNans,
@@ -193,6 +166,13 @@ pub enum IRFunctionExpr {
     #[cfg(feature = "repeat_by")]
     RepeatBy,
     ArgUnique,
+    ArgMin,
+    ArgMax,
+    ArgSort {
+        descending: bool,
+        nulls_last: bool,
+    },
+    Product,
     #[cfg(feature = "rank")]
     Rank {
         options: RankOptions,
@@ -247,7 +227,6 @@ pub enum IRFunctionExpr {
     #[cfg(feature = "approx_unique")]
     ApproxNUnique,
     Coalesce,
-    ShrinkType,
     #[cfg(feature = "diff")]
     Diff(NullBehavior),
     #[cfg(feature = "pct_change")]
@@ -262,9 +241,7 @@ pub enum IRFunctionExpr {
         normalize: bool,
     },
     #[cfg(feature = "log")]
-    Log {
-        base: f64,
-    },
+    Log,
     #[cfg(feature = "log")]
     Log1p,
     #[cfg(feature = "log")]
@@ -283,8 +260,6 @@ pub enum IRFunctionExpr {
     Floor,
     #[cfg(feature = "round_series")]
     Ceil,
-    UpperBound,
-    LowerBound,
     #[cfg(feature = "fused")]
     Fused(fused::FusedOperator),
     ConcatExpr(bool),
@@ -334,6 +309,31 @@ pub enum IRFunctionExpr {
         /// Pickle serialized keyword arguments.
         kwargs: Arc<[u8]>,
     },
+
+    FoldHorizontal {
+        callback: PlanCallback<(Series, Series), Series>,
+        returns_scalar: bool,
+        return_dtype: Option<DataType>,
+    },
+    ReduceHorizontal {
+        callback: PlanCallback<(Series, Series), Series>,
+        returns_scalar: bool,
+        return_dtype: Option<DataType>,
+    },
+    #[cfg(feature = "dtype-struct")]
+    CumReduceHorizontal {
+        callback: PlanCallback<(Series, Series), Series>,
+        returns_scalar: bool,
+        return_dtype: Option<DataType>,
+    },
+    #[cfg(feature = "dtype-struct")]
+    CumFoldHorizontal {
+        callback: PlanCallback<(Series, Series), Series>,
+        returns_scalar: bool,
+        return_dtype: Option<DataType>,
+        include_init: bool,
+    },
+
     MaxHorizontal,
     MinHorizontal,
     SumHorizontal {
@@ -371,6 +371,10 @@ pub enum IRFunctionExpr {
     #[cfg(feature = "reinterpret")]
     Reinterpret(bool),
     ExtendConstant,
+
+    RowEncode(Vec<DataType>, RowEncodingVariant),
+    #[cfg(feature = "dtype-struct")]
+    RowDecode(Vec<Field>, RowEncodingVariant),
 }
 
 impl Hash for IRFunctionExpr {
@@ -433,16 +437,59 @@ impl Hash for IRFunctionExpr {
                 lib.hash(state);
                 symbol.hash(state);
             },
-            MaxHorizontal
-            | MinHorizontal
-            | SumHorizontal { .. }
-            | MeanHorizontal { .. }
-            | DropNans
-            | DropNulls
-            | Reverse
-            | ArgUnique
-            | Shift
-            | ShiftAndFill => {},
+
+            FoldHorizontal {
+                callback,
+                returns_scalar,
+                return_dtype,
+            }
+            | ReduceHorizontal {
+                callback,
+                returns_scalar,
+                return_dtype,
+            } => {
+                callback.hash(state);
+                returns_scalar.hash(state);
+                return_dtype.hash(state);
+            },
+            #[cfg(feature = "dtype-struct")]
+            CumReduceHorizontal {
+                callback,
+                returns_scalar,
+                return_dtype,
+            } => {
+                callback.hash(state);
+                returns_scalar.hash(state);
+                return_dtype.hash(state);
+            },
+            #[cfg(feature = "dtype-struct")]
+            CumFoldHorizontal {
+                callback,
+                returns_scalar,
+                return_dtype,
+                include_init,
+            } => {
+                callback.hash(state);
+                returns_scalar.hash(state);
+                return_dtype.hash(state);
+                include_init.hash(state);
+            },
+
+            SumHorizontal { ignore_nulls } | MeanHorizontal { ignore_nulls } => {
+                ignore_nulls.hash(state)
+            },
+            MaxHorizontal | MinHorizontal | DropNans | DropNulls | Reverse | ArgUnique | ArgMin
+            | ArgMax | Product | Shift | ShiftAndFill | Rechunk => {},
+            Append { upcast } => {
+                upcast.hash(state);
+            },
+            ArgSort {
+                descending,
+                nulls_last,
+            } => {
+                descending.hash(state);
+                nulls_last.hash(state);
+            },
             #[cfg(feature = "mode")]
             Mode => {},
             #[cfg(feature = "abs")]
@@ -461,12 +508,17 @@ impl Hash for IRFunctionExpr {
             Hash(a, b, c, d) => (a, b, c, d).hash(state),
             FillNull => {},
             #[cfg(feature = "rolling_window")]
-            RollingExpr(f) => {
-                f.hash(state);
+            RollingExpr { function, options } => {
+                function.hash(state);
+                options.hash(state);
             },
             #[cfg(feature = "rolling_window_by")]
-            RollingExprBy(f) => {
-                f.hash(state);
+            RollingExprBy {
+                function_by,
+                options,
+            } => {
+                function_by.hash(state);
+                options.hash(state);
             },
             #[cfg(feature = "moment")]
             Skew(a) => a.hash(state),
@@ -515,7 +567,6 @@ impl Hash for IRFunctionExpr {
             #[cfg(feature = "approx_unique")]
             ApproxNUnique => {},
             Coalesce => {},
-            ShrinkType => {},
             #[cfg(feature = "pct_change")]
             PctChange => {},
             #[cfg(feature = "log")]
@@ -524,7 +575,7 @@ impl Hash for IRFunctionExpr {
                 normalize.hash(state);
             },
             #[cfg(feature = "log")]
-            Log { base } => base.to_bits().hash(state),
+            Log => {},
             #[cfg(feature = "log")]
             Log1p => {},
             #[cfg(feature = "log")]
@@ -541,8 +592,6 @@ impl Hash for IRFunctionExpr {
             IRFunctionExpr::Floor => {},
             #[cfg(feature = "round_series")]
             Ceil => {},
-            UpperBound => {},
-            LowerBound => {},
             ConcatExpr(a) => a.hash(state),
             #[cfg(feature = "peaks")]
             PeakMin => {},
@@ -615,6 +664,16 @@ impl Hash for IRFunctionExpr {
             ExtendConstant => {},
             #[cfg(feature = "top_k")]
             TopKBy { descending } => descending.hash(state),
+
+            RowEncode(dts, variants) => {
+                dts.hash(state);
+                variants.hash(state);
+            },
+            #[cfg(feature = "dtype-struct")]
+            RowDecode(fs, variants) => {
+                fs.hash(state);
+                variants.hash(state);
+            },
         }
     }
 }
@@ -666,9 +725,11 @@ impl Display for IRFunctionExpr {
             Sign => "sign",
             FillNull => "fill_null",
             #[cfg(feature = "rolling_window")]
-            RollingExpr(func, ..) => return write!(f, "{func}"),
+            RollingExpr { function, .. } => return write!(f, "{function}"),
             #[cfg(feature = "rolling_window_by")]
-            RollingExprBy(func, ..) => return write!(f, "{func}"),
+            RollingExprBy { function_by, .. } => return write!(f, "{function_by}"),
+            Rechunk => "rechunk",
+            Append { .. } => "append",
             ShiftAndFill => "shift_and_fill",
             DropNans => "drop_nans",
             DropNulls => "drop_nulls",
@@ -679,6 +740,10 @@ impl Display for IRFunctionExpr {
             #[cfg(feature = "moment")]
             Kurtosis(..) => "kurtosis",
             ArgUnique => "arg_unique",
+            ArgMin => "arg_min",
+            ArgMax => "arg_max",
+            ArgSort { .. } => "arg_sort",
+            Product => "product",
             Repeat => "repeat",
             #[cfg(feature = "rank")]
             Rank { .. } => "rank",
@@ -720,7 +785,6 @@ impl Display for IRFunctionExpr {
             #[cfg(feature = "approx_unique")]
             ApproxNUnique => "approx_n_unique",
             Coalesce => "coalesce",
-            ShrinkType => "shrink_dtype",
             #[cfg(feature = "diff")]
             Diff(_) => "diff",
             #[cfg(feature = "pct_change")]
@@ -732,7 +796,7 @@ impl Display for IRFunctionExpr {
             #[cfg(feature = "log")]
             Entropy { .. } => "entropy",
             #[cfg(feature = "log")]
-            Log { .. } => "log",
+            Log => "log",
             #[cfg(feature = "log")]
             Log1p => "log1p",
             #[cfg(feature = "log")]
@@ -752,8 +816,6 @@ impl Display for IRFunctionExpr {
             Floor => "floor",
             #[cfg(feature = "round_series")]
             Ceil => "ceil",
-            UpperBound => "upper_bound",
-            LowerBound => "lower_bound",
             #[cfg(feature = "fused")]
             Fused(fused) => return Display::fmt(fused, f),
             ConcatExpr(_) => "concat_expr",
@@ -781,6 +843,14 @@ impl Display for IRFunctionExpr {
             SetSortedFlag(_) => "set_sorted",
             #[cfg(feature = "ffi_plugin")]
             FfiPlugin { lib, symbol, .. } => return write!(f, "{lib}:{symbol}"),
+
+            FoldHorizontal { .. } => "fold",
+            ReduceHorizontal { .. } => "reduce",
+            #[cfg(feature = "dtype-struct")]
+            CumReduceHorizontal { .. } => "cum_reduce",
+            #[cfg(feature = "dtype-struct")]
+            CumFoldHorizontal { .. } => "cum_fold",
+
             MaxHorizontal => "max_horizontal",
             MinHorizontal => "min_horizontal",
             SumHorizontal { .. } => "sum_horizontal",
@@ -804,6 +874,10 @@ impl Display for IRFunctionExpr {
             #[cfg(feature = "reinterpret")]
             Reinterpret(_) => "reinterpret",
             ExtendConstant => "extend_constant",
+
+            RowEncode(..) => "row_encode",
+            #[cfg(feature = "dtype-struct")]
+            RowDecode(..) => "row_decode",
         };
         write!(f, "{s}")
     }
@@ -831,7 +905,7 @@ macro_rules! wrap {
 macro_rules! map_as_slice {
     ($func:path) => {{
         let f = move |s: &mut [Column]| {
-            $func(s).map(Some)
+            $func(s)
         };
 
         SpecialEq::new(Arc::new(f))
@@ -839,7 +913,7 @@ macro_rules! map_as_slice {
 
     ($func:path, $($args:expr),*) => {{
         let f = move |s: &mut [Column]| {
-            $func(s, $($args),*).map(Some)
+            $func(s, $($args),*)
         };
 
         SpecialEq::new(Arc::new(f))
@@ -853,7 +927,7 @@ macro_rules! map_owned {
     ($func:path) => {{
         let f = move |c: &mut [Column]| {
             let c = std::mem::take(&mut c[0]);
-            $func(c).map(Some)
+            $func(c)
         };
 
         SpecialEq::new(Arc::new(f))
@@ -862,7 +936,7 @@ macro_rules! map_owned {
     ($func:path, $($args:expr),*) => {{
         let f = move |c: &mut [Column]| {
             let c = std::mem::take(&mut c[0]);
-            $func(c, $($args),*).map(Some)
+            $func(c, $($args),*)
         };
 
         SpecialEq::new(Arc::new(f))
@@ -875,7 +949,7 @@ macro_rules! map {
     ($func:path) => {{
         let f = move |c: &mut [Column]| {
             let c = &c[0];
-            $func(c).map(Some)
+            $func(c)
         };
 
         SpecialEq::new(Arc::new(f))
@@ -884,348 +958,13 @@ macro_rules! map {
     ($func:path, $($args:expr),*) => {{
         let f = move |c: &mut [Column]| {
             let c = &c[0];
-            $func(c, $($args),*).map(Some)
+            $func(c, $($args),*)
         };
 
         SpecialEq::new(Arc::new(f))
     }};
 }
 
-impl From<IRFunctionExpr> for SpecialEq<Arc<dyn ColumnsUdf>> {
-    fn from(func: IRFunctionExpr) -> Self {
-        use IRFunctionExpr::*;
-        match func {
-            // Namespaces
-            #[cfg(feature = "dtype-array")]
-            ArrayExpr(func) => func.into(),
-            BinaryExpr(func) => func.into(),
-            #[cfg(feature = "dtype-categorical")]
-            Categorical(func) => func.into(),
-            ListExpr(func) => func.into(),
-            #[cfg(feature = "strings")]
-            StringExpr(func) => func.into(),
-            #[cfg(feature = "dtype-struct")]
-            StructExpr(func) => func.into(),
-            #[cfg(feature = "temporal")]
-            TemporalExpr(func) => func.into(),
-            #[cfg(feature = "bitwise")]
-            Bitwise(func) => func.into(),
-
-            // Other expressions
-            Boolean(func) => func.into(),
-            #[cfg(feature = "business")]
-            Business(func) => func.into(),
-            #[cfg(feature = "abs")]
-            Abs => map!(abs::abs),
-            Negate => map!(dispatch::negate),
-            NullCount => {
-                let f = |s: &mut [Column]| {
-                    let s = &s[0];
-                    Ok(Some(Column::new(
-                        s.name().clone(),
-                        [s.null_count() as IdxSize],
-                    )))
-                };
-                wrap!(f)
-            },
-            Pow(func) => match func {
-                IRPowFunction::Generic => wrap!(pow::pow),
-                IRPowFunction::Sqrt => map!(pow::sqrt),
-                IRPowFunction::Cbrt => map!(pow::cbrt),
-            },
-            #[cfg(feature = "row_hash")]
-            Hash(k0, k1, k2, k3) => {
-                map!(row_hash::row_hash, k0, k1, k2, k3)
-            },
-            #[cfg(feature = "arg_where")]
-            ArgWhere => {
-                wrap!(arg_where::arg_where)
-            },
-            #[cfg(feature = "index_of")]
-            IndexOf => {
-                map_as_slice!(index_of::index_of)
-            },
-            #[cfg(feature = "search_sorted")]
-            SearchSorted { side, descending } => {
-                map_as_slice!(search_sorted::search_sorted_impl, side, descending)
-            },
-            #[cfg(feature = "range")]
-            Range(func) => func.into(),
-
-            #[cfg(feature = "trigonometry")]
-            Trigonometry(trig_function) => {
-                map!(trigonometry::apply_trigonometric_function, trig_function)
-            },
-            #[cfg(feature = "trigonometry")]
-            Atan2 => {
-                wrap!(trigonometry::apply_arctan2)
-            },
-
-            #[cfg(feature = "sign")]
-            Sign => {
-                map!(sign::sign)
-            },
-            FillNull => {
-                map_as_slice!(fill_null::fill_null)
-            },
-            #[cfg(feature = "rolling_window")]
-            RollingExpr(f) => {
-                use IRRollingFunction::*;
-                match f {
-                    Min(options) => map!(rolling::rolling_min, options.clone()),
-                    Max(options) => map!(rolling::rolling_max, options.clone()),
-                    Mean(options) => map!(rolling::rolling_mean, options.clone()),
-                    Sum(options) => map!(rolling::rolling_sum, options.clone()),
-                    Quantile(options) => map!(rolling::rolling_quantile, options.clone()),
-                    Var(options) => map!(rolling::rolling_var, options.clone()),
-                    Std(options) => map!(rolling::rolling_std, options.clone()),
-                    #[cfg(feature = "moment")]
-                    Skew(options) => map!(rolling::rolling_skew, options.clone()),
-                    #[cfg(feature = "moment")]
-                    Kurtosis(options) => map!(rolling::rolling_kurtosis, options.clone()),
-                    #[cfg(feature = "cov")]
-                    CorrCov {
-                        rolling_options,
-                        corr_cov_options,
-                        is_corr,
-                    } => {
-                        map_as_slice!(
-                            rolling::rolling_corr_cov,
-                            rolling_options.clone(),
-                            corr_cov_options,
-                            is_corr
-                        )
-                    },
-                }
-            },
-            #[cfg(feature = "rolling_window_by")]
-            RollingExprBy(f) => {
-                use IRRollingFunctionBy::*;
-                match f {
-                    MinBy(options) => map_as_slice!(rolling_by::rolling_min_by, options.clone()),
-                    MaxBy(options) => map_as_slice!(rolling_by::rolling_max_by, options.clone()),
-                    MeanBy(options) => map_as_slice!(rolling_by::rolling_mean_by, options.clone()),
-                    SumBy(options) => map_as_slice!(rolling_by::rolling_sum_by, options.clone()),
-                    QuantileBy(options) => {
-                        map_as_slice!(rolling_by::rolling_quantile_by, options.clone())
-                    },
-                    VarBy(options) => map_as_slice!(rolling_by::rolling_var_by, options.clone()),
-                    StdBy(options) => map_as_slice!(rolling_by::rolling_std_by, options.clone()),
-                }
-            },
-            #[cfg(feature = "hist")]
-            Hist {
-                bin_count,
-                include_category,
-                include_breakpoint,
-            } => {
-                map_as_slice!(
-                    dispatch::hist,
-                    bin_count,
-                    include_category,
-                    include_breakpoint
-                )
-            },
-            ShiftAndFill => {
-                map_as_slice!(shift_and_fill::shift_and_fill)
-            },
-            DropNans => map_owned!(nan::drop_nans),
-            DropNulls => map!(dispatch::drop_nulls),
-            #[cfg(feature = "round_series")]
-            Clip { has_min, has_max } => {
-                map_as_slice!(clip::clip, has_min, has_max)
-            },
-            #[cfg(feature = "mode")]
-            Mode => map!(dispatch::mode),
-            #[cfg(feature = "moment")]
-            Skew(bias) => map!(dispatch::skew, bias),
-            #[cfg(feature = "moment")]
-            Kurtosis(fisher, bias) => map!(dispatch::kurtosis, fisher, bias),
-            ArgUnique => map!(dispatch::arg_unique),
-            Repeat => map_as_slice!(repeat::repeat),
-            #[cfg(feature = "rank")]
-            Rank { options, seed } => map!(dispatch::rank, options, seed),
-            #[cfg(feature = "dtype-struct")]
-            AsStruct => {
-                map_as_slice!(coerce::as_struct)
-            },
-            #[cfg(feature = "top_k")]
-            TopK { descending } => {
-                map_as_slice!(top_k, descending)
-            },
-            #[cfg(feature = "top_k")]
-            TopKBy { descending } => map_as_slice!(top_k_by, descending.clone()),
-            Shift => map_as_slice!(shift_and_fill::shift),
-            #[cfg(feature = "cum_agg")]
-            CumCount { reverse } => map!(cum::cum_count, reverse),
-            #[cfg(feature = "cum_agg")]
-            CumSum { reverse } => map!(cum::cum_sum, reverse),
-            #[cfg(feature = "cum_agg")]
-            CumProd { reverse } => map!(cum::cum_prod, reverse),
-            #[cfg(feature = "cum_agg")]
-            CumMin { reverse } => map!(cum::cum_min, reverse),
-            #[cfg(feature = "cum_agg")]
-            CumMax { reverse } => map!(cum::cum_max, reverse),
-            #[cfg(feature = "dtype-struct")]
-            ValueCounts {
-                sort,
-                parallel,
-                name,
-                normalize,
-            } => map!(
-                dispatch::value_counts,
-                sort,
-                parallel,
-                name.clone(),
-                normalize
-            ),
-            #[cfg(feature = "unique_counts")]
-            UniqueCounts => map!(dispatch::unique_counts),
-            Reverse => map!(dispatch::reverse),
-            #[cfg(feature = "approx_unique")]
-            ApproxNUnique => map!(dispatch::approx_n_unique),
-            Coalesce => map_as_slice!(fill_null::coalesce),
-            ShrinkType => map_owned!(shrink_type::shrink),
-            #[cfg(feature = "diff")]
-            Diff(null_behavior) => map_as_slice!(dispatch::diff, null_behavior),
-            #[cfg(feature = "pct_change")]
-            PctChange => map_as_slice!(dispatch::pct_change),
-            #[cfg(feature = "interpolate")]
-            Interpolate(method) => {
-                map!(dispatch::interpolate, method)
-            },
-            #[cfg(feature = "interpolate_by")]
-            InterpolateBy => {
-                map_as_slice!(dispatch::interpolate_by)
-            },
-            #[cfg(feature = "log")]
-            Entropy { base, normalize } => map!(log::entropy, base, normalize),
-            #[cfg(feature = "log")]
-            Log { base } => map!(log::log, base),
-            #[cfg(feature = "log")]
-            Log1p => map!(log::log1p),
-            #[cfg(feature = "log")]
-            Exp => map!(log::exp),
-            Unique(stable) => map!(unique::unique, stable),
-            #[cfg(feature = "round_series")]
-            Round { decimals, mode } => map!(round::round, decimals, mode),
-            #[cfg(feature = "round_series")]
-            RoundSF { digits } => map!(round::round_sig_figs, digits),
-            #[cfg(feature = "round_series")]
-            Floor => map!(round::floor),
-            #[cfg(feature = "round_series")]
-            Ceil => map!(round::ceil),
-            UpperBound => map!(bounds::upper_bound),
-            LowerBound => map!(bounds::lower_bound),
-            #[cfg(feature = "fused")]
-            Fused(op) => map_as_slice!(fused::fused, op),
-            ConcatExpr(rechunk) => map_as_slice!(concat::concat_expr, rechunk),
-            #[cfg(feature = "cov")]
-            Correlation { method } => map_as_slice!(correlation::corr, method),
-            #[cfg(feature = "peaks")]
-            PeakMin => map!(peaks::peak_min),
-            #[cfg(feature = "peaks")]
-            PeakMax => map!(peaks::peak_max),
-            #[cfg(feature = "repeat_by")]
-            RepeatBy => map_as_slice!(dispatch::repeat_by),
-            #[cfg(feature = "dtype-array")]
-            Reshape(dims) => map!(dispatch::reshape, &dims),
-            #[cfg(feature = "cutqcut")]
-            Cut {
-                breaks,
-                labels,
-                left_closed,
-                include_breaks,
-            } => map!(
-                cut::cut,
-                breaks.clone(),
-                labels.clone(),
-                left_closed,
-                include_breaks
-            ),
-            #[cfg(feature = "cutqcut")]
-            QCut {
-                probs,
-                labels,
-                left_closed,
-                allow_duplicates,
-                include_breaks,
-            } => map!(
-                cut::qcut,
-                probs.clone(),
-                labels.clone(),
-                left_closed,
-                allow_duplicates,
-                include_breaks
-            ),
-            #[cfg(feature = "rle")]
-            RLE => map!(rle),
-            #[cfg(feature = "rle")]
-            RLEID => map!(rle_id),
-            ToPhysical => map!(dispatch::to_physical),
-            #[cfg(feature = "random")]
-            Random { method, seed } => {
-                use IRRandomMethod::*;
-                match method {
-                    Shuffle => map!(random::shuffle, seed),
-                    Sample {
-                        is_fraction,
-                        with_replacement,
-                        shuffle,
-                    } => {
-                        if is_fraction {
-                            map_as_slice!(random::sample_frac, with_replacement, shuffle, seed)
-                        } else {
-                            map_as_slice!(random::sample_n, with_replacement, shuffle, seed)
-                        }
-                    },
-                }
-            },
-            SetSortedFlag(sorted) => map!(dispatch::set_sorted_flag, sorted),
-            #[cfg(feature = "ffi_plugin")]
-            FfiPlugin {
-                flags: _,
-                lib,
-                symbol,
-                kwargs,
-            } => unsafe {
-                map_as_slice!(
-                    plugin::call_plugin,
-                    lib.as_ref(),
-                    symbol.as_ref(),
-                    kwargs.as_ref()
-                )
-            },
-            MaxHorizontal => wrap!(dispatch::max_horizontal),
-            MinHorizontal => wrap!(dispatch::min_horizontal),
-            SumHorizontal { ignore_nulls } => wrap!(dispatch::sum_horizontal, ignore_nulls),
-            MeanHorizontal { ignore_nulls } => wrap!(dispatch::mean_horizontal, ignore_nulls),
-            #[cfg(feature = "ewma")]
-            EwmMean { options } => map!(ewm::ewm_mean, options),
-            #[cfg(feature = "ewma_by")]
-            EwmMeanBy { half_life } => map_as_slice!(ewm_by::ewm_mean_by, half_life),
-            #[cfg(feature = "ewma")]
-            EwmStd { options } => map!(ewm::ewm_std, options),
-            #[cfg(feature = "ewma")]
-            EwmVar { options } => map!(ewm::ewm_var, options),
-            #[cfg(feature = "replace")]
-            Replace => {
-                map_as_slice!(dispatch::replace)
-            },
-            #[cfg(feature = "replace")]
-            ReplaceStrict { return_dtype } => {
-                map_as_slice!(dispatch::replace_strict, return_dtype.clone())
-            },
-
-            FillNullWithStrategy(strategy) => map!(dispatch::fill_null_with_strategy, strategy),
-            GatherEvery { n, offset } => map!(dispatch::gather_every, n, offset),
-            #[cfg(feature = "reinterpret")]
-            Reinterpret(signed) => map!(dispatch::reinterpret, signed),
-            ExtendConstant => map_as_slice!(dispatch::extend_constant),
-        }
-    }
-}
-
 impl IRFunctionExpr {
     pub fn function_options(&self) -> FunctionOptions {
         use IRFunctionExpr as F;
@@ -1255,7 +994,7 @@ impl IRFunctionExpr {
             F::Negate => FunctionOptions::elementwise(),
             #[cfg(feature = "hist")]
             F::Hist { .. } => FunctionOptions::groupwise(),
-            F::NullCount => FunctionOptions::aggregation(),
+            F::NullCount => FunctionOptions::aggregation().flag(FunctionFlags::NON_ORDER_OBSERVING),
             #[cfg(feature = "row_hash")]
             F::Hash(_, _, _, _) => FunctionOptions::elementwise(),
             #[cfg(feature = "arg_where")]
@@ -1280,27 +1019,40 @@ impl IRFunctionExpr {
             },
             F::FillNullWithStrategy(_) => FunctionOptions::groupwise(),
             #[cfg(feature = "rolling_window")]
-            F::RollingExpr(_) => FunctionOptions::length_preserving(),
+            F::RollingExpr { .. } => FunctionOptions::length_preserving(),
             #[cfg(feature = "rolling_window_by")]
-            F::RollingExprBy(_) => FunctionOptions::length_preserving(),
+            F::RollingExprBy { .. } => FunctionOptions::length_preserving(),
+            F::Rechunk => FunctionOptions::length_preserving(),
+            F::Append { .. } => FunctionOptions::groupwise(),
             F::ShiftAndFill => FunctionOptions::length_preserving(),
             F::Shift => FunctionOptions::length_preserving(),
-            F::DropNans => FunctionOptions::row_separable(),
+            F::DropNans => {
+                FunctionOptions::row_separable().flag(FunctionFlags::NON_ORDER_PRODUCING)
+            },
             F::DropNulls => FunctionOptions::row_separable()
-                .with_flags(|f| f | FunctionFlags::ALLOW_EMPTY_INPUTS),
+                .flag(FunctionFlags::ALLOW_EMPTY_INPUTS | FunctionFlags::NON_ORDER_PRODUCING),
             #[cfg(feature = "mode")]
-            F::Mode => FunctionOptions::groupwise(),
+            F::Mode => FunctionOptions::groupwise().flag(
+                FunctionFlags::NON_ORDER_OBSERVING
+                    | FunctionFlags::TERMINATES_INPUT_ORDER
+                    | FunctionFlags::NON_ORDER_PRODUCING,
+            ),
             #[cfg(feature = "moment")]
-            F::Skew(_) => FunctionOptions::aggregation(),
+            F::Skew(_) => FunctionOptions::aggregation().flag(FunctionFlags::NON_ORDER_OBSERVING),
             #[cfg(feature = "moment")]
-            F::Kurtosis(_, _) => FunctionOptions::aggregation(),
+            F::Kurtosis(_, _) => {
+                FunctionOptions::aggregation().flag(FunctionFlags::NON_ORDER_OBSERVING)
+            },
             #[cfg(feature = "dtype-array")]
             F::Reshape(_) => FunctionOptions::groupwise(),
             #[cfg(feature = "repeat_by")]
             F::RepeatBy => FunctionOptions::elementwise(),
             F::ArgUnique => FunctionOptions::groupwise(),
+            F::ArgMin | F::ArgMax => FunctionOptions::aggregation(),
+            F::ArgSort { .. } => FunctionOptions::length_preserving(),
+            F::Product => FunctionOptions::aggregation().flag(FunctionFlags::NON_ORDER_OBSERVING),
             #[cfg(feature = "rank")]
-            F::Rank { .. } => FunctionOptions::groupwise(),
+            F::Rank { .. } => FunctionOptions::length_preserving(),
             F::Repeat => {
                 FunctionOptions::groupwise().with_flags(|f| f | FunctionFlags::ALLOW_RENAME)
             },
@@ -1320,19 +1072,24 @@ impl IRFunctionExpr {
             | F::CumProd { .. }
             | F::CumMin { .. }
             | F::CumMax { .. } => FunctionOptions::length_preserving(),
-            F::Reverse => FunctionOptions::length_preserving(),
+            F::Reverse => FunctionOptions::length_preserving()
+                .with_flags(|f| f | FunctionFlags::NON_ORDER_OBSERVING),
             #[cfg(feature = "dtype-struct")]
-            F::ValueCounts { .. } => {
-                FunctionOptions::groupwise().with_flags(|f| f | FunctionFlags::PASS_NAME_TO_APPLY)
-            },
+            F::ValueCounts { sort, .. } => FunctionOptions::groupwise().with_flags(|mut f| {
+                if !sort {
+                    f |= FunctionFlags::TERMINATES_INPUT_ORDER | FunctionFlags::NON_ORDER_PRODUCING
+                }
+                f | FunctionFlags::PASS_NAME_TO_APPLY | FunctionFlags::NON_ORDER_OBSERVING
+            }),
             #[cfg(feature = "unique_counts")]
             F::UniqueCounts => FunctionOptions::groupwise(),
             #[cfg(feature = "approx_unique")]
-            F::ApproxNUnique => FunctionOptions::aggregation(),
+            F::ApproxNUnique => {
+                FunctionOptions::aggregation().flag(FunctionFlags::NON_ORDER_OBSERVING)
+            },
             F::Coalesce => FunctionOptions::elementwise()
                 .with_flags(|f| f | FunctionFlags::INPUT_WILDCARD_EXPANSION)
                 .with_supertyping(Default::default()),
-            F::ShrinkType => FunctionOptions::length_preserving(),
             #[cfg(feature = "diff")]
             F::Diff(NullBehavior::Drop) => FunctionOptions::groupwise(),
             #[cfg(feature = "diff")]
@@ -1344,15 +1101,24 @@ impl IRFunctionExpr {
             #[cfg(feature = "interpolate_by")]
             F::InterpolateBy => FunctionOptions::length_preserving(),
             #[cfg(feature = "log")]
-            F::Log { .. } | F::Log1p | F::Exp => FunctionOptions::elementwise(),
+            F::Log | F::Log1p | F::Exp => FunctionOptions::elementwise(),
             #[cfg(feature = "log")]
-            F::Entropy { .. } => FunctionOptions::aggregation(),
-            F::Unique(_) => FunctionOptions::groupwise(),
+            F::Entropy { .. } => {
+                FunctionOptions::aggregation().flag(FunctionFlags::NON_ORDER_OBSERVING)
+            },
+            F::Unique(maintain_order) => FunctionOptions::groupwise().with_flags(|f| {
+                let f = f | FunctionFlags::NON_ORDER_PRODUCING;
+
+                if !*maintain_order {
+                    f | FunctionFlags::NON_ORDER_OBSERVING | FunctionFlags::TERMINATES_INPUT_ORDER
+                } else {
+                    f
+                }
+            }),
             #[cfg(feature = "round_series")]
             F::Round { .. } | F::RoundSF { .. } | F::Floor | F::Ceil => {
                 FunctionOptions::elementwise()
             },
-            F::UpperBound | F::LowerBound => FunctionOptions::aggregation(),
             #[cfg(feature = "fused")]
             F::Fused(_) => FunctionOptions::elementwise(),
             F::ConcatExpr(_) => FunctionOptions::groupwise()
@@ -1390,6 +1156,26 @@ impl IRFunctionExpr {
             }),
             F::MeanHorizontal { .. } | F::SumHorizontal { .. } => FunctionOptions::elementwise()
                 .with_flags(|f| f | FunctionFlags::INPUT_WILDCARD_EXPANSION),
+
+            F::FoldHorizontal { returns_scalar, .. }
+            | F::ReduceHorizontal { returns_scalar, .. } => FunctionOptions::groupwise()
+                .with_flags(|mut f| {
+                    f |= FunctionFlags::INPUT_WILDCARD_EXPANSION;
+                    if *returns_scalar {
+                        f |= FunctionFlags::RETURNS_SCALAR;
+                    }
+                    f
+                }),
+            #[cfg(feature = "dtype-struct")]
+            F::CumFoldHorizontal { returns_scalar, .. }
+            | F::CumReduceHorizontal { returns_scalar, .. } => FunctionOptions::groupwise()
+                .with_flags(|mut f| {
+                    f |= FunctionFlags::INPUT_WILDCARD_EXPANSION;
+                    if *returns_scalar {
+                        f |= FunctionFlags::RETURNS_SCALAR;
+                    }
+                    f
+                }),
             #[cfg(feature = "ewma")]
             F::EwmMean { .. } | F::EwmStd { .. } | F::EwmVar { .. } => {
                 FunctionOptions::length_preserving()
@@ -1404,6 +1190,10 @@ impl IRFunctionExpr {
             #[cfg(feature = "reinterpret")]
             F::Reinterpret(_) => FunctionOptions::elementwise(),
             F::ExtendConstant => FunctionOptions::groupwise(),
+
+            F::RowEncode(..) => FunctionOptions::elementwise(),
+            #[cfg(feature = "dtype-struct")]
+            F::RowDecode(..) => FunctionOptions::elementwise(),
         }
     }
 }
