@@ -2,7 +2,8 @@ mod cse_expr;
 mod cse_lp;
 
 pub(super) use cse_expr::CommonSubExprOptimizer;
-pub(super) use cse_lp::{elim_cmn_subplans, prune_unused_caches};
+pub use cse_expr::NaiveExprMerger;
+pub(super) use cse_lp::elim_cmn_subplans;
 
 use super::*;
 
