@@ -3,15 +3,17 @@ mod simplify_functions;
 use polars_utils::floor_divmod::FloorDivMod;
 use polars_utils::total_ord::ToTotalOrd;
 use simplify_functions::optimize_functions;
+mod arity;
 
 use crate::plans::*;
 
 fn new_null_count(input: &[ExprIR]) -> AExpr {
+    let function = IRFunctionExpr::NullCount;
+    let options = function.function_options();
     AExpr::Function {
         input: input.to_vec(),
-        function: IRFunctionExpr::NullCount,
-        options: FunctionOptions::aggregation()
-            .with_flags(|f| f | FunctionFlags::ALLOW_GROUP_AWARE),
+        function,
+        options,
     }
 }
 
@@ -140,124 +142,20 @@ impl OptimizationRule for SimplifyBooleanRule {
         ctx: OptimizeExprContext,
     ) -> PolarsResult<Option<AExpr>> {
         let expr = expr_arena.get(expr_node);
-        let in_filter = ctx.in_filter;
 
         let out = match expr {
             // true AND x => x
-            AExpr::BinaryExpr {
-                left,
-                op: Operator::And,
-                right,
-            } if matches!(
-                expr_arena.get(*left),
-                AExpr::Literal(lv) if lv.bool() == Some(true)
-            ) && in_filter =>
-            {
-                // Only in filter as we might change the name from "literal"
-                // to whatever lhs columns is.
-                return Ok(Some(expr_arena.get(*right).clone()));
+            AExpr::BinaryExpr { left, op, right } => {
+                return Ok(arity::simplify_binary(*left, *op, *right, ctx, expr_arena));
             },
-            // x AND true => x
-            AExpr::BinaryExpr {
-                left,
-                op: Operator::And,
-                right,
-            } if matches!(
-                expr_arena.get(*right),
-                AExpr::Literal(lv) if lv.bool() == Some(true)
-            ) =>
-            {
-                Some(expr_arena.get(*left).clone())
-            },
-
-            // x AND false -> false
-            // FIXME: we need an optimizer redesign to allow x & false to be optimized
-            // in general as we can forget the length of a series otherwise.
-            AExpr::BinaryExpr {
-                left,
-                op: Operator::And,
-                right,
-            } if matches!(expr_arena.get(*left), AExpr::Literal(_))
-                && matches!(
-                    expr_arena.get(*right),
-                    AExpr::Literal(lv) if lv.bool() == Some(false)
-                ) =>
-            {
-                Some(AExpr::Literal(Scalar::from(false).into()))
-            },
-
-            // false AND x -> false
-            // FIXME: we need an optimizer redesign to allow false & x to be optimized
-            // in general as we can forget the length of a series otherwise.
-            AExpr::BinaryExpr {
-                left,
-                op: Operator::And,
-                right,
-            } if matches!(
-                expr_arena.get(*left),
-                AExpr::Literal(lv) if lv.bool() == Some(false)
-            ) && matches!(expr_arena.get(*right), AExpr::Literal(_)) =>
-            {
-                Some(AExpr::Literal(Scalar::from(false).into()))
-            },
-
-            // false or x => x
-            AExpr::BinaryExpr {
-                left,
-                op: Operator::Or,
-                right,
-            } if matches!(
-                expr_arena.get(*left),
-                AExpr::Literal(lv) if lv.bool() == Some(false)
-            ) && in_filter =>
-            {
-                // Only in filter as we might change the name from "literal"
-                // to whatever lhs columns is.
-                return Ok(Some(expr_arena.get(*right).clone()));
-            },
-            // x or false => x
-            AExpr::BinaryExpr {
-                left,
-                op: Operator::Or,
-                right,
-                ..
-            } if matches!(
-                expr_arena.get(*right),
-                AExpr::Literal(lv) if lv.bool() == Some(false)
-            ) =>
-            {
-                Some(expr_arena.get(*left).clone())
-            },
-
-            // true OR x => true
-            // FIXME: we need an optimizer redesign to allow true | x to be optimized
-            // in general as we can forget the length of a series otherwise.
-            AExpr::BinaryExpr {
-                left,
-                op: Operator::Or,
-                right,
-            } if matches!(expr_arena.get(*left), AExpr::Literal(_))
-                && matches!(
-                    expr_arena.get(*right),
-                    AExpr::Literal(lv) if lv.bool() == Some(true)
-                ) =>
-            {
-                Some(AExpr::Literal(Scalar::from(true).into()))
-            },
-
-            // x OR true => true
-            // FIXME: we need an optimizer redesign to allow true | x to be optimized
-            // in general as we can forget the length of a series otherwise.
-            AExpr::BinaryExpr {
-                left,
-                op: Operator::Or,
-                right,
-            } if matches!(
-                expr_arena.get(*left),
-                    AExpr::Literal(lv) if lv.bool() == Some(true)
-            ) && matches!(expr_arena.get(*right), AExpr::Literal(_)) =>
-            {
-                Some(AExpr::Literal(Scalar::from(true).into()))
+            AExpr::Ternary {
+                predicate,
+                truthy,
+                falsy,
+            } => {
+                return Ok(arity::simplify_ternary(
+                    *predicate, *truthy, *falsy, expr_arena,
+                ));
             },
             AExpr::Function {
                 input,
@@ -326,7 +224,10 @@ fn string_addition_to_linear_concat(
         let left_e = ExprIR::from_node(left_node, expr_arena);
         let right_e = ExprIR::from_node(right_node, expr_arena);
 
-        let get_type = |ae: &AExpr| ae.get_type(input_schema, Context::Default, expr_arena).ok();
+        let get_type = |ae: &AExpr| {
+            ae.to_dtype(&ToFieldContext::new(expr_arena, input_schema))
+                .ok()
+        };
         let type_a = get_type(left_aexpr).or_else(|| get_type(right_aexpr))?;
         let type_b = get_type(right_aexpr).or_else(|| get_type(right_aexpr))?;
 
@@ -451,12 +352,16 @@ impl OptimizationRule for SimplifyExprRule {
         schema: &Schema,
         _ctx: OptimizeExprContext,
     ) -> PolarsResult<Option<AExpr>> {
-        let expr = expr_arena.get(expr_node).clone();
+        let expr = expr_arena.get(expr_node);
 
         let out = match &expr {
+            AExpr::SortBy { expr, by, .. } if by.is_empty() => Some(expr_arena.get(*expr).clone()),
             // drop_nulls().len() -> len() - null_count()
             // drop_nulls().count() -> len() - null_count()
-            AExpr::Agg(IRAggExpr::Count(input, _)) => {
+            AExpr::Agg(IRAggExpr::Count {
+                input,
+                include_nulls: _,
+            }) => {
                 let input_expr = expr_arena.get(*input);
                 match input_expr {
                     AExpr::Function {
@@ -472,10 +377,10 @@ impl OptimizationRule for SimplifyExprRule {
                                 AExpr::Column(_) => Some(AExpr::BinaryExpr {
                                     op: Operator::Minus,
                                     right: expr_arena.add(new_null_count(input)),
-                                    left: expr_arena.add(AExpr::Agg(IRAggExpr::Count(
-                                        drop_nulls_input_node,
-                                        true,
-                                    ))),
+                                    left: expr_arena.add(AExpr::Agg(IRAggExpr::Count {
+                                        input: drop_nulls_input_node,
+                                        include_nulls: true,
+                                    })),
                                 }),
                                 _ => None,
                             }
@@ -509,10 +414,10 @@ impl OptimizationRule for SimplifyExprRule {
                                 AExpr::Column(_) => Some(AExpr::BinaryExpr {
                                     op: Operator::Minus,
                                     right: expr_arena.add(new_null_count(input)),
-                                    left: expr_arena.add(AExpr::Agg(IRAggExpr::Count(
-                                        is_not_null_input_node,
-                                        true,
-                                    ))),
+                                    left: expr_arena.add(AExpr::Agg(IRAggExpr::Count {
+                                        input: is_not_null_input_node,
+                                        include_nulls: true,
+                                    })),
                                 }),
                                 _ => None,
                             }
@@ -770,7 +675,9 @@ impl OptimizationRule for SimplifyExprRule {
                 function,
                 options,
                 ..
-            } => return optimize_functions(input, function, options, expr_arena),
+            } => {
+                return optimize_functions(input.clone(), function.clone(), *options, expr_arena);
+            },
             _ => None,
         };
         Ok(out)
