@@ -1,3 +1,4 @@
+#![allow(unsafe_op_in_unsafe_fn)]
 use arrow::bitmap::{Bitmap, BitmapBuilder};
 
 #[macro_export]
@@ -16,6 +17,7 @@ macro_rules! with_match_arrow_primitive_type {(
         UInt16 => __with_ty__! { u16 },
         UInt32 => __with_ty__! { u32 },
         UInt64 => __with_ty__! { u64 },
+        UInt128 => __with_ty__! { u128 },
         Float32 => __with_ty__! { f32 },
         Float64 => __with_ty__! { f64 },
         _ => unreachable!(),
