@@ -18,7 +18,6 @@ use rayon::prelude::*;
 
 use super::compute_node_prelude::*;
 use crate::async_executor;
-use crate::async_primitives::connector::Receiver;
 use crate::expression::StreamExpr;
 use crate::morsel::get_ideal_morsel_size;
 use crate::nodes::in_memory_source::InMemorySourceNode;
@@ -110,13 +109,14 @@ struct GroupBySinkState {
     locals: Vec<LocalGroupBySinkState>,
     random_state: PlRandomState,
     partitioner: HashPartitioner,
+    has_order_sensitive_agg: bool,
 }
 
 impl GroupBySinkState {
     fn spawn<'env, 's>(
         &'env mut self,
         scope: &'s TaskScope<'s, 'env>,
-        receivers: Vec<Receiver<Morsel>>,
+        receivers: Vec<PortReceiver>,
         state: &'s StreamingExecutionState,
         join_handles: &mut Vec<JoinHandle<PolarsResult<()>>>,
     ) {
@@ -126,6 +126,7 @@ impl GroupBySinkState {
             let grouped_reduction_cols = &self.grouped_reduction_cols;
             let random_state = &self.random_state;
             let partitioner = self.partitioner.clone();
+            let has_order_sensitive_agg = self.has_order_sensitive_agg;
             join_handles.push(scope.spawn_task(TaskPriority::High, async move {
                 let mut hot_idxs = Vec::new();
                 let mut hot_group_idxs = Vec::new();
@@ -140,7 +141,7 @@ impl GroupBySinkState {
                         key_columns.push(s.into_column());
                     }
                     let keys = DataFrame::new_with_broadcast_len(key_columns, df.height())?;
-                    let hash_keys = HashKeys::from_df(&keys, *random_state, true, false);
+                    let hash_keys = HashKeys::from_df(&keys, random_state.clone(), true, false);
 
                     hot_idxs.clear();
                     hot_group_idxs.clear();
@@ -150,6 +151,7 @@ impl GroupBySinkState {
                         &mut hot_idxs,
                         &mut hot_group_idxs,
                         &mut cold_idxs,
+                        has_order_sensitive_agg,
                     );
 
                     // Drop columns not used for reductions (key-only columns).
@@ -466,6 +468,7 @@ impl GroupByNode {
         output_schema: Arc<Schema>,
         random_state: PlRandomState,
         num_pipelines: usize,
+        has_order_sensitive_agg: bool,
     ) -> Self {
         let hot_table_size = std::env::var("POLARS_HOT_TABLE_SIZE")
             .map(|sz| sz.parse::<usize>().unwrap())
@@ -499,6 +502,7 @@ impl GroupByNode {
                 grouped_reduction_cols,
                 locals,
                 partitioner,
+                has_order_sensitive_agg,
             }),
             key_schema,
             output_schema,
