@@ -2,21 +2,17 @@ use std::ops::Range;
 use std::sync::Arc;
 
 use async_trait::async_trait;
-#[cfg(feature = "dtype-categorical")]
-use polars_core::StringCacheHolder;
 use polars_core::prelude::{Column, Field};
 use polars_core::schema::{SchemaExt, SchemaRef};
 use polars_error::{PolarsResult, polars_bail, polars_err, polars_warn};
 use polars_io::RowIndex;
 use polars_io::cloud::CloudOptions;
 use polars_io::prelude::_csv_read_internal::{
-    CountLines, NullValuesCompiled, cast_columns, find_starting_point, prepare_csv_schema,
-    read_chunk,
+    CountLines, NullValuesCompiled, cast_columns, find_starting_point, is_comment_line,
+    prepare_csv_schema, read_chunk,
 };
 use polars_io::prelude::buffer::validate_utf8;
-use polars_io::prelude::{
-    CommentPrefix, CsvEncoding, CsvParseOptions, CsvReadOptions, count_rows_from_slice,
-};
+use polars_io::prelude::{CsvEncoding, CsvParseOptions, CsvReadOptions};
 use polars_io::utils::compression::maybe_decompress_bytes;
 use polars_io::utils::slice::SplitSlicePosition;
 use polars_plan::dsl::ScanSource;
@@ -24,14 +20,15 @@ use polars_utils::IdxSize;
 use polars_utils::mmap::MemSlice;
 use polars_utils::slice_enum::Slice;
 
-use super::multi_file_reader::reader_interface::output::FileReaderOutputRecv;
-use super::multi_file_reader::reader_interface::{BeginReadArgs, FileReader, FileReaderCallbacks};
+use super::multi_scan::reader_interface::output::FileReaderOutputRecv;
+use super::multi_scan::reader_interface::{BeginReadArgs, FileReader, FileReaderCallbacks};
 use crate::DEFAULT_DISTRIBUTOR_BUFFER_SIZE;
 use crate::async_executor::{AbortOnDropHandle, spawn};
 use crate::async_primitives::distributor_channel::{self, distributor_channel};
 use crate::morsel::SourceToken;
 use crate::nodes::compute_node_prelude::*;
-use crate::nodes::io_sources::multi_file_reader::reader_interface::output::FileReaderOutputSend;
+use crate::nodes::io_sources::multi_scan::reader_interface::Projection;
+use crate::nodes::io_sources::multi_scan::reader_interface::output::FileReaderOutputSend;
 use crate::nodes::{MorselSeq, TaskPriority};
 
 pub mod builder {
@@ -43,9 +40,9 @@ pub mod builder {
     use polars_plan::dsl::ScanSource;
 
     use super::CsvFileReader;
-    use crate::nodes::io_sources::multi_file_reader::reader_interface::FileReader;
-    use crate::nodes::io_sources::multi_file_reader::reader_interface::builder::FileReaderBuilder;
-    use crate::nodes::io_sources::multi_file_reader::reader_interface::capabilities::ReaderCapabilities;
+    use crate::nodes::io_sources::multi_scan::reader_interface::FileReader;
+    use crate::nodes::io_sources::multi_scan::reader_interface::builder::FileReaderBuilder;
+    use crate::nodes::io_sources::multi_scan::reader_interface::capabilities::ReaderCapabilities;
 
     impl FileReaderBuilder for Arc<CsvReadOptions> {
         fn reader_name(&self) -> &str {
@@ -55,11 +52,12 @@ pub mod builder {
         fn reader_capabilities(&self) -> ReaderCapabilities {
             use ReaderCapabilities as RC;
 
-            if self.parse_options.comment_prefix.is_some() {
-                RC::empty()
-            } else {
-                RC::PRE_SLICE
-            }
+            RC::NEEDS_FILE_CACHE_INIT
+                | if self.parse_options.comment_prefix.is_some() {
+                    RC::empty()
+                } else {
+                    RC::PRE_SLICE
+                }
         }
 
         fn build_file_reader(
@@ -134,7 +132,7 @@ impl FileReader for CsvFileReader {
         let memslice = self.get_bytes_maybe_decompress()?;
 
         let BeginReadArgs {
-            projected_schema,
+            projection: Projection::Plain(projected_schema),
             // Because we currently only support PRE_SLICE we don't need to handle row index here.
             row_index,
             pre_slice,
@@ -251,15 +249,11 @@ impl FileReader for CsvFileReader {
             )
         }
 
-        // Only used on empty projection, or if we need the exact row count.
-        let alt_count_lines: Option<Arc<CountLinesWithComments>> =
-            CountLinesWithComments::opt_new(&self.options.parse_options).map(Arc::new);
         let chunk_reader = Arc::new(ChunkReader::try_new(
             self.options.clone(),
             inferred_schema.clone(),
             projection,
             row_index,
-            alt_count_lines.clone(),
         )?);
 
         let needs_full_row_count = n_rows_in_file_tx.is_some();
@@ -274,6 +268,7 @@ impl FileReader for CsvFileReader {
                 line_counter: CountLines::new(
                     self.options.parse_options.quote_char,
                     self.options.parse_options.eol_char,
+                    self.options.parse_options.comment_prefix.clone(),
                 ),
                 line_batch_tx,
                 options: self.options.clone(),
@@ -303,7 +298,6 @@ impl FileReader for CsvFileReader {
                 let chunk_reader = chunk_reader.clone();
                 // Note: We don't use this (it is handled by the bridge). But morsels require a source token.
                 let source_token = SourceToken::new();
-                let alt_count_lines = alt_count_lines.clone();
 
                 AbortOnDropHandle::new(spawn(TaskPriority::Low, async move {
                     while let Ok(LineBatch {
@@ -351,7 +345,7 @@ impl FileReader for CsvFileReader {
                         }
 
                         while let Ok(LineBatch {
-                            bytes,
+                            bytes: _,
                             n_lines,
                             slice,
                             row_offset: _,
@@ -360,12 +354,6 @@ impl FileReader for CsvFileReader {
                         {
                             assert_eq!(slice, SLICE_ENDED);
 
-                            let n_lines = if let Some(v) = alt_count_lines.as_deref() {
-                                v.count_lines(bytes)?
-                            } else {
-                                n_lines
-                            };
-
                             n_rows_processed = n_rows_processed.saturating_add(n_lines);
                         }
                     }
@@ -472,17 +460,18 @@ impl LineBatchSource {
 
         let global_bytes: &[u8] = memslice.as_ref();
         let global_bytes: &'static [u8] = unsafe { std::mem::transmute(global_bytes) };
+        let comment_prefix = options.parse_options.comment_prefix.as_ref();
+
+        let parse_options = options.parse_options.as_ref();
+        let eol_char = parse_options.eol_char;
 
         let i = {
-            let parse_options = options.parse_options.as_ref();
-
             let quote_char = parse_options.quote_char;
-            let eol_char = parse_options.eol_char;
 
             let skip_lines = options.skip_lines;
             let skip_rows_before_header = options.skip_rows;
             let skip_rows_after_header = options.skip_rows_after_header;
-            let comment_prefix = parse_options.comment_prefix.clone();
+            let comment_prefix = comment_prefix.cloned();
             let has_header = options.has_header;
 
             find_starting_point(
@@ -523,7 +512,16 @@ impl LineBatchSource {
 
             let (count, position) = line_counter.find_next(bytes, &mut chunk_size);
             let (count, position) = if count == 0 {
-                (1, bytes.len())
+                let c = if *bytes.last().unwrap() != eol_char
+                    && !is_comment_line(
+                        bytes.rsplit(|c| *c == eol_char).next().unwrap(),
+                        comment_prefix,
+                    ) {
+                    1
+                } else {
+                    0
+                };
+                (c, bytes.len())
             } else {
                 let pos = (position + 1).min(bytes.len()); // +1 for '\n'
                 (count, pos)
@@ -590,15 +588,11 @@ struct ChunkReader {
     reader_schema: SchemaRef,
     parse_options: Arc<CsvParseOptions>,
     fields_to_cast: Vec<Field>,
-    #[cfg(feature = "dtype-categorical")]
-    _cat_lock: Option<StringCacheHolder>,
     ignore_errors: bool,
     projection: Vec<usize>,
     null_values: Option<NullValuesCompiled>,
     validate_utf8: bool,
     row_index: Option<RowIndex>,
-    // Alternate line counter when there are comments. This is used on empty projection.
-    alt_count_lines: Option<Arc<CountLinesWithComments>>,
 }
 
 impl ChunkReader {
@@ -607,13 +601,9 @@ impl ChunkReader {
         mut reader_schema: SchemaRef,
         projection: Vec<usize>,
         row_index: Option<RowIndex>,
-        alt_count_lines: Option<Arc<CountLinesWithComments>>,
     ) -> PolarsResult<Self> {
         let mut fields_to_cast: Vec<Field> = options.fields_to_cast.clone();
-        let has_categorical = prepare_csv_schema(&mut reader_schema, &mut fields_to_cast)?;
-
-        #[cfg(feature = "dtype-categorical")]
-        let _cat_lock = has_categorical.then(polars_core::StringCacheHolder::hold);
+        prepare_csv_schema(&mut reader_schema, &mut fields_to_cast)?;
 
         let parse_options = options.parse_options.clone();
 
@@ -632,14 +622,11 @@ impl ChunkReader {
             reader_schema,
             parse_options,
             fields_to_cast,
-            #[cfg(feature = "dtype-categorical")]
-            _cat_lock,
             ignore_errors: options.ignore_errors,
             projection,
             null_values,
             validate_utf8,
             row_index,
-            alt_count_lines,
         })
     }
 
@@ -658,13 +645,7 @@ impl ChunkReader {
 
         // If projection is empty create a DataFrame with the correct height by counting the lines.
         let mut df = if self.projection.is_empty() {
-            let h = if let Some(v) = &self.alt_count_lines {
-                v.count_lines(chunk)?
-            } else {
-                n_lines
-            };
-
-            DataFrame::empty_with_height(h)
+            DataFrame::empty_with_height(n_lines)
         } else {
             read_chunk(
                 chunk,
@@ -685,9 +666,7 @@ impl ChunkReader {
         let n_lines_is_correct = df.height() == n_lines;
 
         // Check malformed
-        if df.height() > n_lines
-            || (df.height() < n_lines && self.parse_options.comment_prefix.is_none())
-        {
+        if !n_lines_is_correct {
             // Note: in case data is malformed, df.height() is more likely to be correct than n_lines.
             let msg = format!(
                 "CSV malformed: expected {} rows, actual {} rows, in chunk starting at row_offset {}, length {}",
@@ -728,32 +707,3 @@ impl ChunkReader {
         Ok((df, height))
     }
 }
-
-struct CountLinesWithComments {
-    quote_char: Option<u8>,
-    eol_char: u8,
-    comment_prefix: CommentPrefix,
-}
-
-impl CountLinesWithComments {
-    fn opt_new(parse_options: &CsvParseOptions) -> Option<Self> {
-        parse_options
-            .comment_prefix
-            .clone()
-            .map(|comment_prefix| CountLinesWithComments {
-                quote_char: parse_options.quote_char,
-                eol_char: parse_options.eol_char,
-                comment_prefix,
-            })
-    }
-
-    fn count_lines(&self, bytes: &[u8]) -> PolarsResult<usize> {
-        count_rows_from_slice(
-            bytes,
-            self.quote_char,
-            Some(&self.comment_prefix),
-            self.eol_char,
-            false, // has_header
-        )
-    }
-}
