@@ -25,7 +25,6 @@ use rayon::prelude::*;
 
 use super::{BufferedStream, JOIN_SAMPLE_LIMIT, LOPSIDED_SAMPLE_FACTOR};
 use crate::async_executor;
-use crate::async_primitives::connector::{Receiver, Sender};
 use crate::async_primitives::wait_group::WaitGroup;
 use crate::expression::StreamExpr;
 use crate::morsel::{SourceToken, get_ideal_morsel_size};
@@ -81,7 +80,6 @@ fn compute_payload_selector(
 ) -> PolarsResult<Vec<Option<PlSmallStr>>> {
     let should_coalesce = args.should_coalesce();
 
-    let mut coalesce_idx = 0;
     this.iter_names()
         .map(|c| {
             #[expect(clippy::never_loop)]
@@ -105,8 +103,8 @@ fn compute_payload_selector(
                     } else if args.how == JoinType::Full {
                         // We must keep the right-hand side keycols around for
                         // coalescing.
-                        let name = format_pl_smallstr!("__POLARS_COALESCE_KEYCOL{coalesce_idx}");
-                        coalesce_idx += 1;
+                        let key_idx = this_key_schema.index_of(c).unwrap();
+                        let name = format_pl_smallstr!("__POLARS_COALESCE_KEYCOL_{key_idx}");
                         Some(name)
                     } else {
                         None
@@ -137,17 +135,13 @@ fn compute_payload_selector(
 fn postprocess_join(df: DataFrame, params: &EquiJoinParams) -> DataFrame {
     if params.args.how == JoinType::Full && params.args.should_coalesce() {
         // TODO: don't do string-based column lookups for each dataframe, pre-compute coalesce indices.
-        let mut coalesce_idx = 0;
         df.get_columns()
             .iter()
             .filter_map(|c| {
-                if params.left_key_schema.contains(c.name()) {
+                if let Some(key_idx) = params.left_key_schema.index_of(c.name()) {
                     let other = df
-                        .column(&format_pl_smallstr!(
-                            "__POLARS_COALESCE_KEYCOL{coalesce_idx}"
-                        ))
+                        .column(&format_pl_smallstr!("__POLARS_COALESCE_KEYCOL_{key_idx}"))
                         .unwrap();
-                    coalesce_idx += 1;
                     return Some(coalesce_columns(&[c.clone(), other.clone()]).unwrap());
                 }
 
@@ -184,7 +178,7 @@ async fn select_keys(
     let keys = DataFrame::new_with_broadcast_len(key_columns, df.height())?;
     Ok(HashKeys::from_df(
         &keys,
-        params.random_state,
+        params.random_state.clone(),
         params.args.nulls_equal,
         false,
     ))
@@ -262,7 +256,7 @@ struct SampleState {
 
 impl SampleState {
     async fn sink(
-        mut recv: Receiver<Morsel>,
+        mut recv: PortReceiver,
         morsels: &mut Vec<Morsel>,
         len: &mut usize,
         this_final_len: Arc<RelaxedCell<usize>>,
@@ -449,7 +443,7 @@ impl BuildState {
     }
 
     async fn partition_and_sink(
-        mut recv: Receiver<Morsel>,
+        mut recv: PortReceiver,
         local: &mut LocalBuilder,
         partitioner: HashPartitioner,
         params: &EquiJoinParams,
@@ -744,8 +738,8 @@ struct ProbeState {
 impl ProbeState {
     /// Returns the max morsel sequence sent.
     async fn partition_and_probe(
-        mut recv: Receiver<Morsel>,
-        mut send: Sender<Morsel>,
+        mut recv: PortReceiver,
+        mut send: PortSender,
         partitions: &[ProbeTable],
         unordered_morsel_seq: &AtomicU64,
         partitioner: HashPartitioner,
@@ -1088,7 +1082,7 @@ struct EmitUnmatchedState {
 impl EmitUnmatchedState {
     async fn emit_unmatched(
         &mut self,
-        mut send: Sender<Morsel>,
+        mut send: PortSender,
         params: &EquiJoinParams,
         num_pipelines: usize,
     ) -> PolarsResult<()> {
