@@ -1,6 +1,4 @@
-use std::path::PathBuf;
 use std::sync::Arc;
-use std::sync::atomic::AtomicUsize;
 
 use parking_lot::Mutex;
 use polars_core::prelude::PlRandomState;
@@ -11,15 +9,18 @@ use polars_expr::groups::new_hash_grouper;
 use polars_expr::planner::{ExpressionConversionState, create_physical_expr};
 use polars_expr::reduce::into_reduction;
 use polars_expr::state::ExecutionState;
-use polars_mem_engine::{create_physical_plan, create_scan_predicate};
+use polars_mem_engine::create_physical_plan;
+use polars_mem_engine::scan_predicate::create_scan_predicate;
 use polars_plan::dsl::{JoinOptionsIR, PartitionVariantIR, ScanSources};
 use polars_plan::plans::expr_ir::ExprIR;
-use polars_plan::plans::{AExpr, ArenaExprIter, Context, IR};
+use polars_plan::plans::{AExpr, ArenaExprIter, Context, IR, IRAggExpr};
 use polars_plan::prelude::{FileType, FunctionFlags};
 use polars_utils::arena::{Arena, Node};
 use polars_utils::format_pl_smallstr;
 use polars_utils::itertools::Itertools;
 use polars_utils::pl_str::PlSmallStr;
+use polars_utils::plpath::PlPath;
+use polars_utils::relaxed_cell::RelaxedCell;
 use recursive::recursive;
 use slotmap::{SecondaryMap, SlotMap};
 
@@ -31,9 +32,9 @@ use crate::morsel::{MorselSeq, get_ideal_morsel_size};
 use crate::nodes;
 use crate::nodes::io_sinks::SinkComputeNode;
 use crate::nodes::io_sinks::partition::PerPartitionSortBy;
-use crate::nodes::io_sources::multi_file_reader::MultiFileReaderConfig;
-use crate::nodes::io_sources::multi_file_reader::reader_interface::builder::FileReaderBuilder;
-use crate::nodes::io_sources::multi_file_reader::reader_interface::capabilities::ReaderCapabilities;
+use crate::nodes::io_sources::multi_scan::config::MultiScanConfig;
+use crate::nodes::io_sources::multi_scan::reader_interface::builder::FileReaderBuilder;
+use crate::nodes::io_sources::multi_scan::reader_interface::capabilities::ReaderCapabilities;
 use crate::physical_plan::lower_expr::compute_output_schema;
 use crate::utils::late_materialized_df::LateMaterializedDataFrame;
 
@@ -143,6 +144,53 @@ fn to_graph_rec<'a>(
             )
         },
 
+        DynamicSlice {
+            input,
+            offset,
+            length,
+        } => {
+            let input_key = to_graph_rec(input.node, ctx)?;
+            let offset_key = to_graph_rec(offset.node, ctx)?;
+            let length_key = to_graph_rec(length.node, ctx)?;
+            let offset_schema = ctx.phys_sm[offset.node].output_schema.clone();
+            let length_schema = ctx.phys_sm[length.node].output_schema.clone();
+            ctx.graph.add_node(
+                nodes::dynamic_slice::DynamicSliceNode::new(offset_schema, length_schema),
+                [
+                    (input_key, input.port),
+                    (offset_key, offset.port),
+                    (length_key, length.port),
+                ],
+            )
+        },
+
+        Shift {
+            input,
+            offset,
+            fill,
+        } => {
+            let input_schema = ctx.phys_sm[input.node].output_schema.clone();
+            let offset_schema = ctx.phys_sm[offset.node].output_schema.clone();
+            let input_key = to_graph_rec(input.node, ctx)?;
+            let offset_key = to_graph_rec(offset.node, ctx)?;
+            if let Some(fill) = fill {
+                let fill_key = to_graph_rec(fill.node, ctx)?;
+                ctx.graph.add_node(
+                    nodes::shift::ShiftNode::new(input_schema, offset_schema, true),
+                    [
+                        (input_key, input.port),
+                        (offset_key, offset.port),
+                        (fill_key, fill.port),
+                    ],
+                )
+            } else {
+                ctx.graph.add_node(
+                    nodes::shift::ShiftNode::new(input_schema, offset_schema, false),
+                    [(input_key, input.port), (offset_key, offset.port)],
+                )
+            }
+        },
+
         Filter { predicate, input } => {
             let input_schema = &ctx.phys_sm[input.node].output_schema;
             let phys_predicate_expr = create_stream_expr(predicate, ctx, input_schema)?;
@@ -241,6 +289,23 @@ fn to_graph_rec<'a>(
             )
         },
 
+        CallbackSink {
+            input,
+            function,
+            maintain_order,
+            chunk_size,
+        } => {
+            let input_key = to_graph_rec(input.node, ctx)?;
+            ctx.graph.add_node(
+                nodes::callback_sink::CallbackSinkNode::new(
+                    function.clone(),
+                    *maintain_order,
+                    *chunk_size,
+                ),
+                [(input_key, input.port)],
+            )
+        },
+
         FileSink {
             target,
             sink_options,
@@ -450,7 +515,7 @@ fn to_graph_rec<'a>(
                 sort_node,
                 &mut lp_arena,
                 ctx.expr_arena,
-                None,
+                Some(crate::dispatch::build_streaming_query_executor),
             )?);
 
             let input_key = to_graph_rec(input.node, ctx)?;
@@ -467,6 +532,96 @@ fn to_graph_rec<'a>(
             )
         },
 
+        TopK {
+            input,
+            k,
+            by_column,
+            reverse,
+            nulls_last,
+        } => {
+            let input_key = to_graph_rec(input.node, ctx)?;
+            let k_key = to_graph_rec(k.node, ctx)?;
+
+            let k_schema = ctx.phys_sm[k.node].output_schema.clone();
+            let input_schema = &ctx.phys_sm[input.node].output_schema;
+            let key_schema = compute_output_schema(input_schema, by_column, ctx.expr_arena)?;
+
+            let key_selectors = by_column
+                .iter()
+                .map(|e| create_stream_expr(e, ctx, input_schema))
+                .try_collect_vec()?;
+
+            ctx.graph.add_node(
+                nodes::top_k::TopKNode::new(
+                    k_schema,
+                    reverse.clone(),
+                    nulls_last.clone(),
+                    key_schema,
+                    key_selectors,
+                ),
+                [(input_key, input.port), (k_key, k.port)],
+            )
+        },
+
+        Repeat { value, repeats } => {
+            let value_key = to_graph_rec(value.node, ctx)?;
+            let repeats_key = to_graph_rec(repeats.node, ctx)?;
+            let value_schema = ctx.phys_sm[value.node].output_schema.clone();
+            let repeats_schema = ctx.phys_sm[repeats.node].output_schema.clone();
+            ctx.graph.add_node(
+                nodes::repeat::RepeatNode::new(value_schema, repeats_schema),
+                [(value_key, value.port), (repeats_key, repeats.port)],
+            )
+        },
+
+        #[cfg(feature = "cum_agg")]
+        CumAgg { input, kind } => {
+            let input_key = to_graph_rec(input.node, ctx)?;
+            ctx.graph.add_node(
+                nodes::cum_agg::CumAggNode::new(*kind),
+                [(input_key, input.port)],
+            )
+        },
+
+        GatherEvery { input, n, offset } => {
+            let (n, offset) = (*n, *offset);
+            let input_key = to_graph_rec(input.node, ctx)?;
+            ctx.graph.add_node(
+                nodes::gather_every::GatherEveryNode::new(n, offset)?,
+                [(input_key, input.port)],
+            )
+        },
+
+        Rle(input) => {
+            let input_key = to_graph_rec(input.node, ctx)?;
+            let input_schema = &ctx.phys_sm[input.node].output_schema;
+            assert_eq!(input_schema.len(), 1);
+            let (name, dtype) = input_schema.get_at_index(0).unwrap();
+            ctx.graph.add_node(
+                nodes::rle::RleNode::new(name.clone(), dtype.clone()),
+                [(input_key, input.port)],
+            )
+        },
+
+        RleId(input) => {
+            let input_key = to_graph_rec(input.node, ctx)?;
+            let input_schema = &ctx.phys_sm[input.node].output_schema;
+            assert_eq!(input_schema.len(), 1);
+            let (_, dtype) = input_schema.get_at_index(0).unwrap();
+            ctx.graph.add_node(
+                nodes::rle_id::RleIdNode::new(dtype.clone()),
+                [(input_key, input.port)],
+            )
+        },
+
+        PeakMinMax { input, is_peak_max } => {
+            let input_key = to_graph_rec(input.node, ctx)?;
+            ctx.graph.add_node(
+                nodes::peak_minmax::PeakMinMaxNode::new(*is_peak_max),
+                [(input_key, input.port)],
+            )
+        },
+
         OrderedUnion { inputs } => {
             let input_keys = inputs
                 .iter()
@@ -506,17 +661,19 @@ fn to_graph_rec<'a>(
             scan_sources,
             file_reader_builder,
             cloud_options,
-            projected_file_schema,
+            file_projection_builder,
             output_schema,
             row_index,
             pre_slice,
             predicate,
+            predicate_file_skip_applied,
             hive_parts,
             missing_columns_policy,
-            extra_columns_policy,
             cast_columns_policy,
             include_file_paths,
+            forbid_extra_columns,
             deletion_files,
+            table_statistics,
             file_schema,
         } => {
             let hive_parts = hive_parts.clone();
@@ -538,51 +695,51 @@ fn to_graph_rec<'a>(
                 })
                 .transpose()?
                 .map(|p| p.to_io(None, file_schema.clone()));
+            let predicate_file_skip_applied = *predicate_file_skip_applied;
 
             let sources = scan_sources.clone();
             let file_reader_builder = file_reader_builder.clone();
             let cloud_options = cloud_options.clone();
 
             let final_output_schema = output_schema.clone();
-            let projected_file_schema = projected_file_schema.clone();
-            let full_file_schema = file_schema.clone();
+            let file_projection_builder = file_projection_builder.clone();
 
             let row_index = row_index.clone();
             let pre_slice = pre_slice.clone();
             let hive_parts = hive_parts.map(Arc::new);
             let include_file_paths = include_file_paths.clone();
             let missing_columns_policy = *missing_columns_policy;
-            let extra_columns_policy = *extra_columns_policy;
+            let forbid_extra_columns = forbid_extra_columns.clone();
             let cast_columns_policy = cast_columns_policy.clone();
             let deletion_files = deletion_files.clone();
+            let table_statistics = table_statistics.clone();
 
             let verbose = config::verbose();
 
             ctx.graph.add_node(
-                nodes::io_sources::multi_file_reader::MultiFileReader::new(Arc::new(
-                    MultiFileReaderConfig {
-                        sources,
-                        file_reader_builder,
-                        cloud_options,
-                        final_output_schema,
-                        projected_file_schema,
-                        full_file_schema,
-                        row_index,
-                        pre_slice,
-                        predicate,
-                        hive_parts,
-                        include_file_paths,
-                        missing_columns_policy,
-                        extra_columns_policy,
-                        cast_columns_policy,
-                        deletion_files,
-                        // Initialized later
-                        num_pipelines: AtomicUsize::new(0),
-                        n_readers_pre_init: AtomicUsize::new(0),
-                        max_concurrent_scans: AtomicUsize::new(0),
-                        verbose,
-                    },
-                )),
+                nodes::io_sources::multi_scan::MultiScan::new(Arc::new(MultiScanConfig {
+                    sources,
+                    file_reader_builder,
+                    cloud_options,
+                    final_output_schema,
+                    file_projection_builder,
+                    row_index,
+                    pre_slice,
+                    predicate,
+                    predicate_file_skip_applied,
+                    hive_parts,
+                    include_file_paths,
+                    missing_columns_policy,
+                    forbid_extra_columns,
+                    cast_columns_policy,
+                    deletion_files,
+                    table_statistics,
+                    // Initialized later
+                    num_pipelines: RelaxedCell::new_usize(0),
+                    n_readers_pre_init: RelaxedCell::new_usize(0),
+                    max_concurrent_scans: RelaxedCell::new_usize(0),
+                    verbose,
+                })),
                 [],
             )
         },
@@ -601,7 +758,12 @@ fn to_graph_rec<'a>(
 
             let mut grouped_reductions = Vec::new();
             let mut grouped_reduction_cols = Vec::new();
+            let mut has_order_sensitive_agg = false;
             for agg in aggs {
+                has_order_sensitive_agg |= matches!(
+                    ctx.expr_arena.get(agg.node()),
+                    AExpr::Agg(IRAggExpr::First(..) | IRAggExpr::Last(..))
+                );
                 let (reduction, input_node) =
                     into_reduction(agg.node(), ctx.expr_arena, input_schema)?;
                 let AExpr::Column(col) = ctx.expr_arena.get(input_node) else {
@@ -621,11 +783,45 @@ fn to_graph_rec<'a>(
                     node.output_schema.clone(),
                     PlRandomState::default(),
                     ctx.num_pipelines,
+                    has_order_sensitive_agg,
                 ),
                 [(input_key, input.port)],
             )
         },
 
+        #[cfg(feature = "dynamic_group_by")]
+        RollingGroupBy {
+            input,
+            index_column,
+            period,
+            offset,
+            closed,
+            aggs,
+        } => {
+            let input_schema = &ctx.phys_sm[input.node].output_schema;
+            let input_key = to_graph_rec(input.node, ctx)?;
+            let aggs = aggs
+                .iter()
+                .map(|e| {
+                    Ok((
+                        e.output_name().clone(),
+                        create_stream_expr(e, ctx, input_schema)?,
+                    ))
+                })
+                .collect::<PolarsResult<Arc<[_]>>>()?;
+            ctx.graph.add_node(
+                nodes::rolling_group_by::RollingGroupBy::new(
+                    input_schema.clone(),
+                    index_column.clone(),
+                    *period,
+                    *offset,
+                    *closed,
+                    aggs,
+                )?,
+                [(input_key, input.port)],
+            )
+        },
+
         InMemoryJoin {
             input_left,
             input_right,
@@ -657,8 +853,6 @@ fn to_graph_rec<'a>(
                     force_parallel: false,
                     args: args.clone(),
                     options: options.clone(),
-                    rows_left: (None, 0),
-                    rows_right: (None, 0),
                 }),
             });
 
@@ -666,7 +860,7 @@ fn to_graph_rec<'a>(
                 join_node,
                 &mut lp_arena,
                 ctx.expr_arena,
-                None,
+                Some(crate::dispatch::build_streaming_query_executor),
             )?);
 
             ctx.graph.add_node(
@@ -814,15 +1008,11 @@ fn to_graph_rec<'a>(
         MergeSorted {
             input_left,
             input_right,
-            key,
         } => {
             let left_input_key = to_graph_rec(input_left.node, ctx)?;
             let right_input_key = to_graph_rec(input_right.node, ctx)?;
-
-            let input_schema = ctx.phys_sm[input_left.node].output_schema.clone();
-
             ctx.graph.add_node(
-                nodes::merge_sorted::MergeSortedNode::new(input_schema, key.clone()),
+                nodes::merge_sorted::MergeSortedNode::new(),
                 [
                     (left_input_key, input_left.port),
                     (right_input_key, input_right.port),
@@ -832,8 +1022,10 @@ fn to_graph_rec<'a>(
 
         #[cfg(feature = "python")]
         PythonScan { options } => {
+            use arrow::buffer::Buffer;
             use polars_plan::dsl::python_dsl::PythonScanSource as S;
             use polars_plan::plans::PythonPredicate;
+            use polars_utils::relaxed_cell::RelaxedCell;
             use pyo3::exceptions::PyStopIteration;
             use pyo3::prelude::*;
             use pyo3::types::{PyBytes, PyNone};
@@ -879,7 +1071,7 @@ fn to_graph_rec<'a>(
 
                     // Setup the IO plugin generator.
                     let (generator, can_parse_predicate) = {
-                        Python::with_gil(|py| {
+                        Python::attach(|py| {
                             let pl = PyModule::import(py, intern!(py, "polars")).unwrap();
                             let utils = pl.getattr(intern!(py, "_utils")).unwrap();
                             let callable =
@@ -929,7 +1121,7 @@ fn to_graph_rec<'a>(
                     }?;
 
                     let get_batch_fn = Box::new(move |state: &StreamingExecutionState| {
-                        let df = Python::with_gil(|py| {
+                        let df = Python::attach(|py| {
                             match generator.bind(py).call_method0(intern!(py, "__next__")) {
                                 Ok(out) => polars_plan::plans::python_df_to_rust(py, out).map(Some),
                                 Err(err)
@@ -971,74 +1163,132 @@ fn to_graph_rec<'a>(
                 },
             };
 
-            use polars_plan::dsl::{CastColumnsPolicy, ExtraColumnsPolicy, MissingColumnsPolicy};
+            use polars_plan::dsl::{CastColumnsPolicy, MissingColumnsPolicy};
 
             use crate::nodes::io_sources::batch::builder::BatchFnReaderBuilder;
             use crate::nodes::io_sources::batch::{BatchFnReader, GetBatchState};
+            use crate::nodes::io_sources::multi_scan::components::projection::builder::ProjectionBuilder;
 
-            let mut reader = BatchFnReader {
+            let reader = BatchFnReader {
                 name: name.clone(),
                 // If validate_schema is false, the schema of the morsels may not match the
                 // configured schema. In this case we set this to `None` and the reader will
                 // retrieve the schema from the first morsel.
                 output_schema: validate_schema.then(|| output_schema.clone()),
                 get_batch_state: Some(GetBatchState::from(get_batch_fn)),
+                execution_state: None,
                 verbose: config::verbose(),
             };
 
-            // Note: This will potentially override the output schema if `validate_schema` is `false`.
-            let output_schema = reader._file_schema()?;
-
             let file_reader_builder = Arc::new(BatchFnReaderBuilder {
                 name,
                 reader: std::sync::Mutex::new(Some(reader)),
+                execution_state: Default::default(),
             }) as Arc<dyn FileReaderBuilder>;
 
             // Give multiscan a single scan source. (It doesn't actually read from this).
-            let sources = ScanSources::Paths(Arc::from([PathBuf::from("python-scan-0")]));
+            let sources =
+                ScanSources::Paths(Buffer::from_iter([PlPath::from_str("python-scan-0")]));
             let cloud_options = None;
             let final_output_schema = output_schema.clone();
-            let projected_file_schema = output_schema.clone();
-            let full_file_schema = output_schema.clone();
+            let file_projection_builder = ProjectionBuilder::new(output_schema, None, None);
             let row_index = None;
             let pre_slice = None;
             let predicate = None;
+            let predicate_file_skip_applied = None;
             let hive_parts = None;
             let include_file_paths = None;
             let missing_columns_policy = MissingColumnsPolicy::Raise;
-            let extra_columns_policy = ExtraColumnsPolicy::Ignore;
+            let forbid_extra_columns = None;
             let cast_columns_policy = CastColumnsPolicy::ERROR_ON_MISMATCH;
             let deletion_files = None;
+            let table_statistics = None;
             let verbose = config::verbose();
 
             ctx.graph.add_node(
-                nodes::io_sources::multi_file_reader::MultiFileReader::new(Arc::new(
-                    MultiFileReaderConfig {
-                        sources,
-                        file_reader_builder,
-                        cloud_options,
-                        final_output_schema,
-                        projected_file_schema,
-                        full_file_schema,
-                        row_index,
-                        pre_slice,
-                        predicate,
-                        hive_parts,
-                        include_file_paths,
-                        missing_columns_policy,
-                        extra_columns_policy,
-                        cast_columns_policy,
-                        deletion_files,
-                        // Initialized later
-                        num_pipelines: AtomicUsize::new(0),
-                        n_readers_pre_init: AtomicUsize::new(0),
-                        max_concurrent_scans: AtomicUsize::new(0),
-                        verbose,
-                    },
-                )),
+                nodes::io_sources::multi_scan::MultiScan::new(Arc::new(MultiScanConfig {
+                    sources,
+                    file_reader_builder,
+                    cloud_options,
+                    final_output_schema,
+                    file_projection_builder,
+                    row_index,
+                    pre_slice,
+                    predicate,
+                    predicate_file_skip_applied,
+                    hive_parts,
+                    include_file_paths,
+                    missing_columns_policy,
+                    forbid_extra_columns,
+                    cast_columns_policy,
+                    deletion_files,
+                    table_statistics,
+                    // Initialized later
+                    num_pipelines: RelaxedCell::new_usize(0),
+                    n_readers_pre_init: RelaxedCell::new_usize(0),
+                    max_concurrent_scans: RelaxedCell::new_usize(0),
+                    verbose,
+                })),
                 [],
             )
         },
+
+        #[cfg(feature = "ewma")]
+        ewm_variant @ EwmMean { input, options }
+        | ewm_variant @ EwmVar { input, options }
+        | ewm_variant @ EwmStd { input, options } => {
+            use arrow::legacy::kernels::ewm::{
+                EwmCovState, EwmStateUpdate, EwmStdState, EwmVarState,
+            };
+            use nodes::ewm::EwmNode;
+            use polars_compute::ewm::mean::EwmMeanState;
+            use polars_core::with_match_physical_float_type;
+
+            let input_key = to_graph_rec(input.node, ctx)?;
+            let input_schema = &ctx.phys_sm[input.node].output_schema;
+            let (_, dtype) = input_schema.get_at_index(0).unwrap();
+
+            let state: Box<dyn EwmStateUpdate + Send> = match ewm_variant {
+                EwmMean { .. } => {
+                    with_match_physical_float_type!(dtype, |$T| {
+                        let state: EwmMeanState<$T> = EwmMeanState::new(
+                            options.alpha as $T,
+                            options.adjust,
+                            options.min_periods,
+                            options.ignore_nulls,
+                        );
+
+                        Box::new(state)
+                    })
+                },
+                _ => with_match_physical_float_type!(dtype, |$T| {
+                    let state: EwmCovState<$T> = EwmCovState::new(
+                        options.alpha as $T,
+                        options.adjust,
+                        options.bias,
+                        options.min_periods,
+                        options.ignore_nulls,
+                    );
+
+                    match ewm_variant {
+                        EwmVar { .. } => Box::new(EwmVarState::new(state)),
+                        EwmStd { .. } => Box::new(EwmStdState::new(state)),
+                        _ => unreachable!(),
+                    }
+                }),
+            };
+
+            let name = match ewm_variant {
+                EwmMean { .. } => "ewm-mean",
+                EwmVar { .. } => "ewm-var",
+                EwmStd { .. } => "ewm-std",
+                _ => unreachable!(),
+            };
+
+            let node = EwmNode::new(name, state);
+
+            ctx.graph.add_node(node, [(input_key, input.port)])
+        },
     };
 
     ctx.phys_to_graph.insert(phys_node_key, graph_key);
