@@ -1,3 +1,5 @@
+use std::collections::VecDeque;
+
 use arrow::legacy::time_zone::Tz;
 use arrow::trusted_len::TrustedLen;
 use polars_core::POOL;
@@ -36,7 +38,9 @@ pub enum Label {
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "dsl-schema", derive(schemars::JsonSchema))]
 #[strum(serialize_all = "snake_case")]
+#[derive(Default)]
 pub enum StartBy {
+    #[default]
     WindowBound,
     DataPoint,
     /// only useful if periods are weekly
@@ -49,12 +53,6 @@ pub enum StartBy {
     Sunday,
 }
 
-impl Default for StartBy {
-    fn default() -> Self {
-        Self::WindowBound
-    }
-}
-
 impl StartBy {
     pub fn weekday(&self) -> Option<u32> {
         match self {
@@ -82,19 +80,34 @@ fn update_groups_and_bounds(
     upper_bound: &mut Vec<i64>,
     groups: &mut Vec<[IdxSize; 2]>,
 ) {
-    'bounds: for bi in bounds_iter {
+    let mut iter = bounds_iter.into_iter();
+    let mut stride = 0;
+
+    'bounds: while let Some(bi) = iter.nth(stride) {
+        let mut has_member = false;
         // find starting point of window
         for &t in &time[start..time.len().saturating_sub(1)] {
             // the window is behind the time values.
             if bi.is_future(t, closed_window) {
+                stride = iter.get_stride(t);
                 continue 'bounds;
             }
             if bi.is_member_entry(t, closed_window) {
+                has_member = true;
                 break;
             }
+            // element drops out of the window
             start += 1;
         }
 
+        // update stride so we can fast-forward in case of sparse data
+        stride = if has_member {
+            0
+        } else {
+            debug_assert!(start < time.len());
+            iter.get_stride(time[start])
+        };
+
         // find members of this window
         let mut end = start;
 
@@ -523,7 +536,7 @@ fn prune_splits_on_duplicates(time: &[i64], thread_offsets: &mut Vec<(usize, usi
         }
     }
     // Check last block
-    if thread_offsets.len() % 2 == 0 {
+    if thread_offsets.len().is_multiple_of(2) {
         let window = &thread_offsets[thread_offsets.len() - 2..];
         if is_valid(window) {
             new.push(thread_offsets[thread_offsets.len() - 1])
@@ -781,6 +794,168 @@ pub fn group_by_values(
     }
 }
 
+pub struct RollingWindower {
+    period: Duration,
+    offset: Duration,
+    closed: ClosedWindow,
+
+    add: fn(&Duration, i64, Option<&Tz>) -> PolarsResult<i64>,
+    tz: Option<Tz>,
+
+    start: IdxSize,
+    end: IdxSize,
+    length: IdxSize,
+
+    active: VecDeque<ActiveWindow>,
+}
+
+struct ActiveWindow {
+    start: i64,
+    end: i64,
+}
+
+impl ActiveWindow {
+    #[inline(always)]
+    fn above_lower_bound(&self, t: i64, closed: ClosedWindow) -> bool {
+        (t > self.start)
+            | (matches!(closed, ClosedWindow::Left | ClosedWindow::Both) & (t == self.start))
+    }
+
+    #[inline(always)]
+    fn below_upper_bound(&self, t: i64, closed: ClosedWindow) -> bool {
+        (t < self.end)
+            | (matches!(closed, ClosedWindow::Right | ClosedWindow::Both) & (t == self.end))
+    }
+}
+
+fn skip_in_2d_list(l: &[&[i64]], mut n: usize) -> (usize, usize) {
+    let mut y = 0;
+    while y < l.len() && (n >= l[y].len() || l[y].is_empty()) {
+        n -= l[y].len();
+        y += 1;
+    }
+    assert!(n == 0 || y < l.len());
+    (n, y)
+}
+fn increment_2d(x: &mut usize, y: &mut usize, l: &[&[i64]]) {
+    *x += 1;
+    while *y < l.len() && *x == l[*y].len() {
+        *y += 1;
+        *x = 0;
+    }
+}
+
+impl RollingWindower {
+    pub fn new(
+        period: Duration,
+        offset: Duration,
+        closed: ClosedWindow,
+        tu: TimeUnit,
+        tz: Option<Tz>,
+    ) -> Self {
+        Self {
+            period,
+            offset,
+            closed,
+
+            add: match tu {
+                TimeUnit::Nanoseconds => Duration::add_ns,
+                TimeUnit::Microseconds => Duration::add_us,
+                TimeUnit::Milliseconds => Duration::add_ms,
+            },
+            tz,
+
+            start: 0,
+            end: 0,
+            length: 0,
+
+            active: Default::default(),
+        }
+    }
+
+    /// Insert new values into the windower.
+    ///
+    /// This should be given all the old values that were not processed yet.
+    pub fn insert(
+        &mut self,
+        time: &[&[i64]],
+        windows: &mut Vec<[IdxSize; 2]>,
+    ) -> PolarsResult<IdxSize> {
+        let (mut i_x, mut i_y) = skip_in_2d_list(time, (self.length - self.start) as usize);
+        let (mut s_x, mut s_y) = skip_in_2d_list(time, 0); // skip over empty lists
+        let (mut e_x, mut e_y) = skip_in_2d_list(time, (self.end - self.start) as usize);
+
+        let time_start = self.start;
+        let mut i = self.length;
+        while i_y < time.len() {
+            let t = time[i_y][i_x];
+            let window_start = (self.add)(&self.offset, t, self.tz.as_ref())?;
+            // For datetime arithmetic, it does *NOT* hold 0 + a - a == 0. Therefore, we make sure
+            // that if `offset` and `period` are inverses we keep the `t`.
+            let window_end = if self.offset == -self.period {
+                t
+            } else {
+                (self.add)(&self.period, window_start, self.tz.as_ref())?
+            };
+
+            self.active.push_back(ActiveWindow {
+                start: window_start,
+                end: window_end,
+            });
+
+            while let Some(w) = self.active.front() {
+                if w.below_upper_bound(t, self.closed) {
+                    break;
+                }
+
+                let w = self.active.pop_front().unwrap();
+                while self.start < i && !w.above_lower_bound(time[s_y][s_x], self.closed) {
+                    increment_2d(&mut s_x, &mut s_y, time);
+                    self.start += 1;
+                }
+                while self.end < i && w.below_upper_bound(time[e_y][e_x], self.closed) {
+                    increment_2d(&mut e_x, &mut e_y, time);
+                    self.end += 1;
+                }
+                windows.push([self.start, self.end - self.start]);
+            }
+
+            increment_2d(&mut i_x, &mut i_y, time);
+            i += 1;
+        }
+
+        self.length = i;
+        Ok(self.start - time_start)
+    }
+
+    /// Process all remaining items and signal that no more items are coming.
+    pub fn finalize(&mut self, time: &[&[i64]], windows: &mut Vec<[IdxSize; 2]>) {
+        assert_eq!(
+            time.iter().map(|t| t.len()).sum::<usize>() as IdxSize,
+            self.length - self.start
+        );
+
+        let (mut s_x, mut s_y) = skip_in_2d_list(time, 0);
+        let (mut e_x, mut e_y) = skip_in_2d_list(time, (self.end - self.start) as usize);
+
+        windows.extend(self.active.drain(..).map(|w| {
+            while self.start < self.length && !w.above_lower_bound(time[s_y][s_x], self.closed) {
+                increment_2d(&mut s_x, &mut s_y, time);
+                self.start += 1;
+            }
+            while self.end < self.length && w.below_upper_bound(time[e_y][e_x], self.closed) {
+                increment_2d(&mut e_x, &mut e_y, time);
+                self.end += 1;
+            }
+            [self.start, self.end - self.start]
+        }));
+
+        self.start = 0;
+        self.end = 0;
+        self.length = 0;
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
