@@ -1,4 +1,14 @@
 //! Collection of common functions that can be used to create menus
+use std::borrow::Cow;
+use unicase::UniCase;
+
+use itertools::{
+    FoldWhile::{Continue, Done},
+    Itertools,
+};
+use nu_ansi_term::{ansi::RESET, Style};
+use unicode_segmentation::UnicodeSegmentation;
+
 use crate::{Editor, Suggestion, UndoBehavior};
 
 /// Index result obtained from parsing a string with an index marker
@@ -70,7 +80,6 @@ pub fn parse_selection_char(buffer: &str, marker: char) -> ParseResult<'_> {
     let mut input = buffer.chars().peekable();
 
     let mut index = 0;
-    let mut action = ParseAction::ForwardSearch;
     while let Some(char) = input.next() {
         if char == marker {
             match input.peek() {
@@ -97,12 +106,15 @@ pub fn parse_selection_char(buffer: &str, marker: char) -> ParseResult<'_> {
                 Some(&x) if x.is_ascii_digit() || x == '-' => {
                     let mut count: usize = 0;
                     let mut size: usize = marker.len_utf8();
+                    let action = if x == '-' {
+                        size += 1;
+                        let _ = input.next();
+                        ParseAction::BackwardSearch
+                    } else {
+                        ParseAction::ForwardSearch
+                    };
                     while let Some(&c) = input.peek() {
-                        if c == '-' {
-                            let _ = input.next();
-                            size += 1;
-                            action = ParseAction::BackwardSearch;
-                        } else if c.is_ascii_digit() {
+                        if c.is_ascii_digit() {
                             let c = c.to_digit(10).expect("already checked if is a digit");
                             let _ = input.next();
                             count *= 10;
@@ -141,7 +153,7 @@ pub fn parse_selection_char(buffer: &str, marker: char) -> ParseResult<'_> {
                         remainder: &buffer[0..index],
                         index: Some(0),
                         marker: Some(&buffer[index..buffer.len()]),
-                        action,
+                        action: ParseAction::ForwardSearch,
                         prefix: Some(&buffer[index..buffer.len()]),
                     }
                 }
@@ -155,45 +167,34 @@ pub fn parse_selection_char(buffer: &str, marker: char) -> ParseResult<'_> {
         remainder: buffer,
         index: None,
         marker: None,
-        action,
+        action: ParseAction::ForwardSearch,
         prefix: None,
     }
 }
 
 /// Finds index for the common string in a list of suggestions
-pub fn find_common_string(values: &[Suggestion]) -> (Option<&Suggestion>, Option<usize>) {
-    let first = values.iter().next();
+pub fn find_common_string(values: &[Suggestion]) -> Option<(&Suggestion, usize)> {
+    let first_suggestion = values.first()?;
+    let max_len = first_suggestion.value.len();
 
-    let index = first.and_then(|first| {
-        values.iter().skip(1).fold(None, |index, suggestion| {
-            if suggestion.value.starts_with(&first.value) {
-                Some(first.value.len())
+    let index = values
+        .iter()
+        .skip(1)
+        .fold_while(max_len, |cumulated_min, current_suggestion| {
+            let new_common_prefix_len = first_suggestion
+                .value
+                .char_indices()
+                .zip(current_suggestion.value.chars())
+                .find_map(|((idx, lhs), rhs)| (rhs != lhs).then_some(idx))
+                .unwrap_or(max_len);
+            if new_common_prefix_len == 0 {
+                Done(0)
             } else {
-                first
-                    .value
-                    .char_indices()
-                    .zip(suggestion.value.char_indices())
-                    .find(|((_, mut lhs), (_, mut rhs))| {
-                        lhs.make_ascii_lowercase();
-                        rhs.make_ascii_lowercase();
-
-                        lhs != rhs
-                    })
-                    .map(|((new_index, _), _)| match index {
-                        Some(index) => {
-                            if index <= new_index {
-                                index
-                            } else {
-                                new_index
-                            }
-                        }
-                        None => new_index,
-                    })
+                Continue(cumulated_min.min(new_common_prefix_len))
             }
-        })
-    });
+        });
 
-    (first, index)
+    Some((first_suggestion, index.into_inner()))
 }
 
 /// Finds different string between two strings
@@ -338,15 +339,17 @@ pub fn replace_in_buffer(value: Option<Suggestion>, editor: &mut Editor) {
 
 /// Helper for `Menu::can_partially_complete`
 pub fn can_partially_complete(values: &[Suggestion], editor: &mut Editor) -> bool {
-    if let (Some(Suggestion { value, span, .. }), Some(index)) = find_common_string(values) {
-        let index = index.min(value.len());
+    if let Some((Suggestion { value, span, .. }, index)) = find_common_string(values) {
         let matching = &value[0..index];
         let end = floor_char_boundary(editor.get_buffer(), span.end);
         let start = floor_char_boundary(editor.get_buffer(), span.start).min(end);
 
         // make sure that the partial completion does not overwrite user entered input
-        let extends_input = matching.starts_with(&editor.get_buffer()[start..end])
-            && matching != &editor.get_buffer()[start..end];
+        let entered_input = &editor.get_buffer()[start..end];
+        let extends_input = UniCase::new(matching)
+            .to_folded_case()
+            .contains(&UniCase::new(entered_input).to_folded_case())
+            && matching != entered_input;
 
         if !matching.is_empty() && extends_input {
             let mut line_buffer = editor.line_buffer().clone();
@@ -372,10 +375,104 @@ pub fn can_partially_complete(values: &[Suggestion], editor: &mut Editor) -> boo
     }
 }
 
+/// Parse ANSI sequences for setting display attributes in the given string.
+/// Each returned item is a tuple (escape start, escape end, text end), for
+/// finding each sequence and the text affected by it.
+///
+/// Essentially just looks for 'ESC [' followed by /[0-9;]*m/, ignoring other ANSI sequences.
+fn parse_ansi(s: &str) -> Vec<(usize, usize, usize)> {
+    let mut segments = Vec::new();
+
+    let mut last_escape_start = 0;
+    let mut last_escape_end = 0;
+    let mut offset = 0;
+    while offset < s.len() {
+        let Some(start) = &s[offset..].find("\x1b[") else {
+            break;
+        };
+        let escape_start = offset + start;
+
+        let after_params = &s[escape_start + 2..]
+            .trim_start_matches(['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', ';']);
+        if !after_params.starts_with('m') {
+            // Not a valid Select Graphic Rendition sequence
+            offset = s.len() - after_params.len();
+            continue;
+        }
+
+        if escape_start != 0 {
+            segments.push((last_escape_start, last_escape_end, escape_start));
+        }
+        last_escape_start = escape_start;
+        last_escape_end = s.len() - after_params.len() + 1;
+        offset = last_escape_end;
+    }
+
+    segments.push((last_escape_start, last_escape_end, s.len()));
+    segments
+}
+
+/// Style a suggestion to be shown in a completer menu
+///
+/// * `match_indices` - Indices of graphemes (NOT bytes or chars) that matched the typed text
+/// * `match_style` - Style to use for matched characters
+pub fn style_suggestion(suggestion: &str, match_indices: &[usize], match_style: &Style) -> String {
+    let mut res = String::new();
+    let mut offset = 0;
+    for (escape_start, text_start, text_end) in parse_ansi(suggestion) {
+        let escape = &suggestion[escape_start..text_start];
+        let text = &suggestion[text_start..text_end];
+        let graphemes = text.graphemes(true).collect::<Vec<_>>();
+        let mut prev_matched = false;
+
+        res.push_str(escape);
+        for (i, grapheme) in graphemes.iter().enumerate() {
+            let is_match = match_indices.contains(&(i + offset));
+
+            if is_match && !prev_matched {
+                res.push_str(&match_style.prefix().to_string());
+            } else if !is_match && prev_matched && i != 0 {
+                res.push_str(RESET);
+                res.push_str(escape);
+            }
+            res.push_str(grapheme);
+            prev_matched = is_match;
+        }
+
+        if prev_matched {
+            res.push_str(RESET);
+        }
+
+        offset += graphemes.len();
+    }
+
+    res
+}
+
+/// If `match_indices` is given, then returns that. Otherwise, tries to find `typed_text`
+/// inside `value`, then returns the indices for that substring.
+pub fn get_match_indices<'a>(
+    value: &str,
+    match_indices: &'a Option<Vec<usize>>,
+    typed_text: &str,
+) -> Cow<'a, Vec<usize>> {
+    if let Some(inds) = match_indices {
+        Cow::Borrowed(inds)
+    } else {
+        let Some(match_pos) = value.to_lowercase().find(&typed_text.to_lowercase()) else {
+            // Don't highlight anything if no match
+            return Cow::Owned(vec![]);
+        };
+        let match_len = typed_text.graphemes(true).count();
+        Cow::Owned((match_pos..match_pos + match_len).collect())
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
     use crate::{EditCommand, LineBuffer, Span};
+    use nu_ansi_term::Color;
     use rstest::rstest;
 
     #[test]
@@ -617,44 +714,23 @@ mod tests {
         assert_eq!(res, (1, "e"));
     }
 
-    #[test]
-    fn find_common_string_with_ansi() {
-        use crate::Span;
-
-        let input: Vec<_> = ["nushell", "null"]
+    #[rstest]
+    #[case::ascii(vec!["nushell", "null"], 2)]
+    #[case::non_ascii(vec!["ｎｕｓｈｅｌｌ", "ｎｕｌｌ"], 6)]
+    // https://github.com/nushell/nushell/pull/16765#issuecomment-3384411809
+    #[case::unsorted(vec!["a", "b", "ab"], 0)]
+    #[case::should_be_case_sensitive(vec!["a", "A"], 0)]
+    fn test_find_common_string(#[case] input: Vec<&str>, #[case] expected: usize) {
+        let input: Vec<_> = input
             .into_iter()
             .map(|s| Suggestion {
                 value: s.into(),
-                description: None,
-                style: None,
-                extra: None,
-                span: Span::new(0, s.len()),
-                append_whitespace: false,
+                ..Default::default()
             })
             .collect();
-        let res = find_common_string(&input);
+        let (_, len) = find_common_string(&input).unwrap();
 
-        assert!(matches!(res, (Some(elem), Some(2)) if elem == &input[0]));
-    }
-
-    #[test]
-    fn find_common_string_with_non_ansi() {
-        use crate::Span;
-
-        let input: Vec<_> = ["ｎｕｓｈｅｌｌ", "ｎｕｌｌ"]
-            .into_iter()
-            .map(|s| Suggestion {
-                value: s.into(),
-                description: None,
-                style: None,
-                extra: None,
-                span: Span::new(0, s.len()),
-                append_whitespace: false,
-            })
-            .collect();
-        let res = find_common_string(&input);
-
-        assert!(matches!(res, (Some(elem), Some(6)) if elem == &input[0]));
+        assert!(len == expected);
     }
 
     #[rstest]
@@ -700,11 +776,8 @@ mod tests {
         replace_in_buffer(
             Some(Suggestion {
                 value,
-                description: None,
-                style: None,
-                extra: None,
                 span: Span::new(start, end),
-                append_whitespace: false,
+                ..Default::default()
             }),
             &mut editor,
         );
@@ -715,4 +788,72 @@ mod tests {
         assert_eq!(orig_buffer, editor.get_buffer());
         assert_eq!(orig_insertion_point, editor.insertion_point());
     }
+
+    #[rstest]
+    #[case("plain", vec![(0, 0, 5)])]
+    #[case("\x1b[mempty", vec![(0, 3, 8)])]
+    #[case("\x1b[\x1b[minvalid", vec![(0, 0, 2), (2, 5, 12)])]
+    #[case("a\x1b[1;mb\x1b[;mc", vec![(0, 0, 1), (1, 6, 7), (7, 11, 12)])]
+    fn test_parse_ansi(#[case] s: &str, #[case] expected: Vec<(usize, usize, usize)>) {
+        assert_eq!(parse_ansi(s), expected);
+    }
+
+    #[test]
+    fn style_fuzzy_suggestion() {
+        let match_style = Style::new().underline();
+        let style1 = Style::new().on(Color::Blue);
+        let style2 = Style::new().on(Color::Green);
+
+        let expected = format!(
+            "{}{}{}{}{}{}{}{}{}{}{}{}{}",
+            style1.prefix(),
+            "ab",
+            match_style.paint("汉"),
+            style1.prefix(),
+            "d",
+            RESET,
+            style2.prefix(),
+            match_style.paint("y̆👩🏾"),
+            style2.prefix(),
+            "e",
+            RESET,
+            "b@",
+            match_style.paint("r"),
+        );
+        let match_indices = &[
+            2, // 汉
+            4, 5, // y̆👩🏾
+            9, // r
+        ];
+        assert_eq!(
+            expected,
+            style_suggestion(
+                &format!("{}{}{}", style1.paint("ab汉d"), style2.paint("y̆👩🏾e"), "b@r"),
+                match_indices,
+                &match_style
+            )
+        );
+    }
+
+    #[test]
+    fn style_fuzzy_suggestion_out_of_bounds() {
+        let text_style = Style::new().on(Color::Blue).bold();
+        let match_style = Style::new().underline();
+
+        let expected = format!(
+            "{}{}{}{}",
+            text_style.prefix(),
+            "go",
+            match_style.paint("o"),
+            RESET
+        );
+        assert_eq!(
+            expected,
+            style_suggestion(
+                &text_style.paint("goo").to_string(),
+                &[2, 3, 4, 6],
+                &match_style
+            )
+        );
+    }
 }
