@@ -91,6 +91,8 @@ pub struct Painter {
     // Stdout
     stdout: W,
     prompt_start_row: u16,
+    // The number of lines that the prompt takes up
+    prompt_height: u16,
     terminal_size: (u16, u16),
     last_required_lines: u16,
     large_buffer: bool,
@@ -103,6 +105,7 @@ impl Painter {
         Painter {
             stdout,
             prompt_start_row: 0,
+            prompt_height: 0,
             terminal_size: (0, 0),
             last_required_lines: 0,
             large_buffer: false,
@@ -121,7 +124,18 @@ impl Painter {
         self.terminal_size.0
     }
 
-    /// Returns the available lines from the prompt down
+    /// Returns the empty lines from the prompt down.
+    pub fn remaining_lines_real(&self) -> u16 {
+        self.screen_height()
+            .saturating_sub(self.prompt_start_row)
+            .saturating_sub(self.prompt_height)
+    }
+
+    /// Returns the number of lines that are available or can be made available by
+    /// stripping the prompt.
+    ///
+    /// If you want the number of empty lines below the prompt,
+    /// use [`Painter::remaining_lines_real`] instead.
     pub fn remaining_lines(&self) -> u16 {
         self.screen_height().saturating_sub(self.prompt_start_row)
     }
@@ -199,18 +213,21 @@ impl Painter {
         let screen_width = self.screen_width();
         let screen_height = self.screen_height();
 
-        // Handle resize for multi line prompt
+        // We add one here as [`PromptLines::prompt_lines_with_wrap`] intentionally subtracts 1 from the real value.
+        self.prompt_height = lines.prompt_lines_with_wrap(screen_width) + 1;
+        let lines_before_cursor = lines.required_lines(screen_width, true, None);
+
+        // Calibrate prompt start position for multi-line prompt/content before cursor. Check issue #841/#848/#930
         if self.just_resized {
-            self.prompt_start_row = self.prompt_start_row.saturating_sub(
-                (lines.prompt_str_left.matches('\n').count()
-                    + lines.prompt_indicator.matches('\n').count()) as u16,
-            );
+            self.prompt_start_row = self
+                .prompt_start_row
+                .saturating_sub(lines_before_cursor - 1);
             self.just_resized = false;
         }
 
         // Lines and distance parameters
         let remaining_lines = self.remaining_lines();
-        let required_lines = lines.required_lines(screen_width, menu);
+        let required_lines = lines.required_lines(screen_width, false, menu);
 
         // Marking the painter state as larger buffer to avoid animations
         self.large_buffer = required_lines >= screen_height;
@@ -226,6 +243,9 @@ impl Painter {
 
         // Moving the start position of the cursor based on the size of the required lines
         if self.large_buffer || is_reset() {
+            for _ in 0..screen_height.saturating_sub(lines_before_cursor) {
+                self.stdout.queue(Print(&coerce_crlf("\n")))?;
+            }
             self.prompt_start_row = 0;
         } else if required_lines >= remaining_lines {
             let extra = required_lines.saturating_sub(remaining_lines);
