@@ -1,4 +1,4 @@
-use crate::{stry, Error, ErrorType};
+use crate::{Error, ErrorType, macros::stry};
 use serde_ext::ser;
 use std::io::Write;
 use std::str;
@@ -7,7 +7,7 @@ use value_trait::generator::BaseGenerator;
 use super::key_must_be_a_string;
 
 macro_rules! iomap {
-    ($e:expr) => {
+    ($e:expr_2021) => {
         ($e).map_err(|err| Error::generic(ErrorType::Io(err)))
     };
 }
@@ -106,7 +106,7 @@ struct SerializeSeq<'serializer, W: Write + 'serializer> {
     s: &'serializer mut PrettySerializer<W>,
     first: bool,
 }
-impl<'serializer, W> ser::SerializeSeq for SerializeSeq<'serializer, W>
+impl<W> ser::SerializeSeq for SerializeSeq<'_, W>
 where
     W: Write,
 {
@@ -141,7 +141,7 @@ where
     }
 }
 
-impl<'serializer, W> ser::SerializeTuple for SerializeSeq<'serializer, W>
+impl<W> ser::SerializeTuple for SerializeSeq<'_, W>
 where
     W: Write,
 {
@@ -175,7 +175,7 @@ where
     }
 }
 
-impl<'serializer, W> ser::SerializeTupleStruct for SerializeSeq<'serializer, W>
+impl<W> ser::SerializeTupleStruct for SerializeSeq<'_, W>
 where
     W: Write,
 {
@@ -209,7 +209,7 @@ where
     }
 }
 
-impl<'serializer, W> ser::SerializeTupleVariant for SerializeSeq<'serializer, W>
+impl<W> ser::SerializeTupleVariant for SerializeSeq<'_, W>
 where
     W: Write,
 {
@@ -247,7 +247,7 @@ struct MapKeySerializer<'serializer, W: Write + 'serializer> {
     s: &'serializer mut PrettySerializer<W>,
 }
 
-impl<'serializer, W> ser::Serializer for MapKeySerializer<'serializer, W>
+impl<W> ser::Serializer for MapKeySerializer<'_, W>
 where
     W: Write,
 {
@@ -290,83 +290,93 @@ where
     }
 
     fn serialize_i8(self, v: i8) -> Result<Self::Ok, Self::Error> {
-        iomap!(self
-            .s
-            .write_char(b'"')
-            .and_then(|()| self.s.write_int(v))
-            .and_then(|()| self.s.write_char(b'"')))
+        iomap!(
+            self.s
+                .write_char(b'"')
+                .and_then(|()| self.s.write_int(v))
+                .and_then(|()| self.s.write_char(b'"'))
+        )
     }
 
     fn serialize_i16(self, v: i16) -> Result<Self::Ok, Self::Error> {
-        iomap!(self
-            .s
-            .write_char(b'"')
-            .and_then(|()| self.s.write_int(v))
-            .and_then(|()| self.s.write_char(b'"')))
+        iomap!(
+            self.s
+                .write_char(b'"')
+                .and_then(|()| self.s.write_int(v))
+                .and_then(|()| self.s.write_char(b'"'))
+        )
     }
 
     fn serialize_i32(self, v: i32) -> Result<Self::Ok, Self::Error> {
-        iomap!(self
-            .s
-            .write_char(b'"')
-            .and_then(|()| self.s.write_int(v))
-            .and_then(|()| self.s.write_char(b'"')))
+        iomap!(
+            self.s
+                .write_char(b'"')
+                .and_then(|()| self.s.write_int(v))
+                .and_then(|()| self.s.write_char(b'"'))
+        )
     }
 
     fn serialize_i64(self, v: i64) -> Result<Self::Ok, Self::Error> {
-        iomap!(self
-            .s
-            .write_char(b'"')
-            .and_then(|()| self.s.write_int(v))
-            .and_then(|()| self.s.write_char(b'"')))
+        iomap!(
+            self.s
+                .write_char(b'"')
+                .and_then(|()| self.s.write_int(v))
+                .and_then(|()| self.s.write_char(b'"'))
+        )
     }
 
     fn serialize_i128(self, v: i128) -> Result<Self::Ok, Self::Error> {
-        iomap!(self
-            .s
-            .write_char(b'"')
-            .and_then(|()| self.s.write_int(v))
-            .and_then(|()| self.s.write_char(b'"')))
+        iomap!(
+            self.s
+                .write_char(b'"')
+                .and_then(|()| self.s.write_int(v))
+                .and_then(|()| self.s.write_char(b'"'))
+        )
     }
 
     fn serialize_u8(self, v: u8) -> Result<Self::Ok, Self::Error> {
-        iomap!(self
-            .s
-            .write_char(b'"')
-            .and_then(|()| self.s.write_int(v))
-            .and_then(|()| self.s.write_char(b'"')))
+        iomap!(
+            self.s
+                .write_char(b'"')
+                .and_then(|()| self.s.write_int(v))
+                .and_then(|()| self.s.write_char(b'"'))
+        )
     }
 
     fn serialize_u16(self, v: u16) -> Result<Self::Ok, Self::Error> {
-        iomap!(self
-            .s
-            .write_char(b'"')
-            .and_then(|()| self.s.write_int(v))
-            .and_then(|()| self.s.write_char(b'"')))
+        iomap!(
+            self.s
+                .write_char(b'"')
+                .and_then(|()| self.s.write_int(v))
+                .and_then(|()| self.s.write_char(b'"'))
+        )
     }
 
     fn serialize_u32(self, v: u32) -> Result<Self::Ok, Self::Error> {
-        iomap!(self
-            .s
-            .write_char(b'"')
-            .and_then(|()| self.s.write_int(v))
-            .and_then(|()| self.s.write_char(b'"')))
+        iomap!(
+            self.s
+                .write_char(b'"')
+                .and_then(|()| self.s.write_int(v))
+                .and_then(|()| self.s.write_char(b'"'))
+        )
     }
 
     fn serialize_u64(self, v: u64) -> Result<Self::Ok, Self::Error> {
-        iomap!(self
-            .s
-            .write_char(b'"')
-            .and_then(|()| self.s.write_int(v))
-            .and_then(|()| self.s.write_char(b'"')))
+        iomap!(
+            self.s
+                .write_char(b'"')
+                .and_then(|()| self.s.write_int(v))
+                .and_then(|()| self.s.write_char(b'"'))
+        )
     }
 
     fn serialize_u128(self, v: u128) -> Result<Self::Ok, Self::Error> {
-        iomap!(self
-            .s
-            .write_char(b'"')
-            .and_then(|()| self.s.write_int(v))
-            .and_then(|()| self.s.write_char(b'"')))
+        iomap!(
+            self.s
+                .write_char(b'"')
+                .and_then(|()| self.s.write_int(v))
+                .and_then(|()| self.s.write_char(b'"'))
+        )
     }
 
     fn serialize_f32(self, _v: f32) -> Result<Self::Ok, Self::Error> {
@@ -471,7 +481,7 @@ struct SerializeMap<'serializer, W: Write + 'serializer> {
     first: bool,
 }
 
-impl<'serializer, W> ser::SerializeMap for SerializeMap<'serializer, W>
+impl<W> ser::SerializeMap for SerializeMap<'_, W>
 where
     W: Write,
 {
@@ -518,7 +528,7 @@ where
     }
 }
 
-impl<'serializer, W> ser::SerializeStruct for SerializeMap<'serializer, W>
+impl<W> ser::SerializeStruct for SerializeMap<'_, W>
 where
     W: Write,
 {
@@ -536,16 +546,18 @@ where
         } = *self;
         if *first {
             *first = false;
-            iomap!(s
-                .new_line()
-                .and_then(|()| s.write_simple_string(key))
-                .and_then(|()| s.write(b": ")))
+            iomap!(
+                s.new_line()
+                    .and_then(|()| s.write_simple_string(key))
+                    .and_then(|()| s.write(b": "))
+            )
             .and_then(|()| value.serialize(&mut **s))
         } else {
-            iomap!(s
-                .write(b",")
-                .and_then(|()| s.write_simple_string(key))
-                .and_then(|()| s.write(b": ")))
+            iomap!(
+                s.write(b",")
+                    .and_then(|()| s.write_simple_string(key))
+                    .and_then(|()| s.write(b": "))
+            )
             .and_then(|()| value.serialize(&mut **s))
         }
     }
@@ -565,7 +577,7 @@ struct SerializeStructVariant<'serializer, W: Write + 'serializer> {
     first: bool,
 }
 
-impl<'serializer, W> ser::SerializeStructVariant for SerializeStructVariant<'serializer, W>
+impl<W> ser::SerializeStructVariant for SerializeStructVariant<'_, W>
 where
     W: Write,
 {
@@ -584,16 +596,18 @@ where
         if *first {
             *first = false;
             s.indent();
-            iomap!(s
-                .new_line()
-                .and_then(|()| s.write_simple_string(key))
-                .and_then(|()| s.write(b": ")))
+            iomap!(
+                s.new_line()
+                    .and_then(|()| s.write_simple_string(key))
+                    .and_then(|()| s.write(b": "))
+            )
             .and_then(|()| value.serialize(&mut **s))
         } else {
-            iomap!(s
-                .write(b",")
-                .and_then(|()| s.write_simple_string(key))
-                .and_then(|()| s.write(b": ")))
+            iomap!(
+                s.write(b",")
+                    .and_then(|()| s.write_simple_string(key))
+                    .and_then(|()| s.write(b": "))
+            )
             .and_then(|()| value.serialize(&mut **s))
         }
     }
@@ -764,10 +778,11 @@ where
     where
         T: ?Sized + serde_ext::Serialize,
     {
-        iomap!(self
-            .write(b"{")
-            .and_then(|()| self.write_simple_string(variant))
-            .and_then(|()| self.write(b": ")))
+        iomap!(
+            self.write(b"{")
+                .and_then(|()| self.write_simple_string(variant))
+                .and_then(|()| self.write(b": "))
+        )
         .and_then(|()| value.serialize(&mut *self))
         .and_then(|()| iomap!(self.write(b"}")))
     }
@@ -808,11 +823,12 @@ where
         len: usize,
     ) -> Result<Self::SerializeTupleVariant, Self::Error> {
         self.indent();
-        iomap!(self
-            .write(b"{")
-            .and_then(|()| self.new_line())
-            .and_then(|()| self.write_simple_string(variant))
-            .and_then(|()| self.write(b": ")))
+        iomap!(
+            self.write(b"{")
+                .and_then(|()| self.new_line())
+                .and_then(|()| self.write_simple_string(variant))
+                .and_then(|()| self.write(b": "))
+        )
         .and_then(move |()| self.serialize_seq(Some(len)))
     }
 
@@ -848,11 +864,12 @@ where
         len: usize,
     ) -> Result<Self::SerializeStructVariant, Self::Error> {
         self.indent();
-        iomap!(self
-            .write(b"{")
-            .and_then(|()| self.new_line())
-            .and_then(|()| self.write_simple_string(variant))
-            .and_then(|()| self.write(b": ")))
+        iomap!(
+            self.write(b"{")
+                .and_then(|()| self.new_line())
+                .and_then(|()| self.write_simple_string(variant))
+                .and_then(|()| self.write(b": "))
+        )
         .and_then(move |()| {
             if len == 0 {
                 iomap!(self.write(b"{}"))
@@ -869,11 +886,11 @@ where
 
 #[cfg(test)]
 mod test {
-    #![allow(clippy::ignored_unit_patterns)]
-    use crate::from_slice;
+    #![allow(clippy::ignored_unit_patterns, unused_imports)]
     use crate::OwnedValue as Value;
     #[cfg(not(target_arch = "wasm32"))]
     use crate::StaticNode;
+    use crate::from_slice;
     #[cfg(not(target_arch = "wasm32"))]
     use proptest::prelude::*;
     #[test]
@@ -1008,7 +1025,16 @@ mod test {
     #[test]
     fn prettyfy() {
         let v = crate::json!({"key1":{}, "key2":[], "key3":[1,{"key4":null}]});
-        let mut res = crate::to_vec_pretty(&v).expect("encoding failed");
+        let mut res = match crate::to_vec_pretty(&v) {
+            Ok(res) => res,
+            Err(e) => {
+                println!("prettify: {e}");
+                assert_eq!(v, "snot");
+                vec![]
+            }
+        };
+        let s = unsafe { String::from_utf8_unchecked(res.clone()) };
+        println!("{s}");
         let v2: Value = from_slice(&mut res).expect("generated bad json");
         assert_eq!(v, v2);
     }
