@@ -1,11 +1,3 @@
-//! This example demonstrates a minimalist implementation of [`Tabling`](Table) records
-//! with struct fields.
-//!
-//! * This second nested [`Table`] example showcases the [`derive`] approach.
-//!
-//! * Note how the [`display_with`] attribute macro applies the custom `display_distribution`
-//!   filter function, which, in this case, applies styles to the final display.
-
 use tabled::{settings::Style, Table, Tabled};
 
 fn main() {
@@ -25,9 +17,9 @@ fn main() {
 #[derive(Tabled)]
 struct Vendor {
     name: String,
-    #[tabled(display_with = "display_distribution")]
+    #[tabled(display = "display_distribution")]
     main_os: Dist,
-    #[tabled(display_with = "display_distribution")]
+    #[tabled(display = "display_distribution")]
     switch_os: Dist,
 }
 
