@@ -147,13 +147,13 @@
 //! * **platform-verifier** enables verifying the server certificates using a method native to the
 //!   platform ureq is executing on. See [rustls-platform-verifier] crate
 //! * **socks-proxy** enables proxy config using the `socks4://`, `socks4a://`, `socks5://`
-//!    and `socks://` (equal to `socks5://`) prefix
+//!   and `socks://` (equal to `socks5://`) prefix
 //! * **cookies** enables cookies
 //! * **gzip** enables requests of gzip-compressed responses and decompresses them
 //! * **brotli** enables requests brotli-compressed responses and decompresses them
 //! * **charset** enables interpreting the charset part of the Content-Type header
-//!    (e.g.  `Content-Type: text/plain; charset=iso-8859-1`). Without this, the
-//!    library defaults to Rust's built in `utf-8`
+//!   (e.g.  `Content-Type: text/plain; charset=iso-8859-1`). Without this, the
+//!   library defaults to Rust's built in `utf-8`
 //! * **json** enables JSON sending and receiving via serde_json
 //!
 //! ### Unstable
@@ -522,15 +522,17 @@ pub use ureq_proto::http;
 pub use body::{Body, BodyBuilder, BodyReader, BodyWithConfig};
 use http::Method;
 use http::{Request, Response, Uri};
-pub use proxy::Proxy;
+pub use proxy::{Proxy, ProxyBuilder, ProxyProtocol};
 pub use request::RequestBuilder;
 use request::{WithBody, WithoutBody};
+pub use request_ext::RequestExt;
 pub use response::ResponseExt;
 pub use send_body::AsSendBody;
 
 mod agent;
 mod body;
 pub mod config;
+mod connect;
 mod error;
 mod pool;
 mod proxy;
@@ -553,6 +555,8 @@ pub mod tls;
 
 #[cfg(feature = "cookies")]
 mod cookies;
+mod request_ext;
+
 #[cfg(feature = "cookies")]
 pub use cookies::{Cookie, CookieJar};
 
@@ -584,29 +588,113 @@ pub fn agent() -> Agent {
     Agent::new_with_defaults()
 }
 
-macro_rules! mk_method {
-    ($f:tt, $m:tt, $b:ty) => {
-        #[doc = concat!("Make a ", stringify!($m), " request.\n\nRun on a use-once [`Agent`].")]
-        #[must_use]
-        pub fn $f<T>(uri: T) -> RequestBuilder<$b>
-        where
-            Uri: TryFrom<T>,
-            <Uri as TryFrom<T>>::Error: Into<http::Error>,
-        {
-            RequestBuilder::<$b>::new(Agent::new_with_defaults(), Method::$m, uri)
-        }
-    };
+/// Make a GET request.
+///
+/// Run on a use-once [`Agent`].
+#[must_use]
+pub fn get<T>(uri: T) -> RequestBuilder<WithoutBody>
+where
+    Uri: TryFrom<T>,
+    <Uri as TryFrom<T>>::Error: Into<http::Error>,
+{
+    RequestBuilder::<WithoutBody>::new(Agent::new_with_defaults(), Method::GET, uri)
 }
 
-mk_method!(get, GET, WithoutBody);
-mk_method!(post, POST, WithBody);
-mk_method!(put, PUT, WithBody);
-mk_method!(delete, DELETE, WithoutBody);
-mk_method!(head, HEAD, WithoutBody);
-mk_method!(options, OPTIONS, WithoutBody);
-mk_method!(connect, CONNECT, WithoutBody);
-mk_method!(patch, PATCH, WithBody);
-mk_method!(trace, TRACE, WithoutBody);
+/// Make a POST request.
+///
+/// Run on a use-once [`Agent`].
+#[must_use]
+pub fn post<T>(uri: T) -> RequestBuilder<WithBody>
+where
+    Uri: TryFrom<T>,
+    <Uri as TryFrom<T>>::Error: Into<http::Error>,
+{
+    RequestBuilder::<WithBody>::new(Agent::new_with_defaults(), Method::POST, uri)
+}
+
+/// Make a PUT request.
+///
+/// Run on a use-once [`Agent`].
+#[must_use]
+pub fn put<T>(uri: T) -> RequestBuilder<WithBody>
+where
+    Uri: TryFrom<T>,
+    <Uri as TryFrom<T>>::Error: Into<http::Error>,
+{
+    RequestBuilder::<WithBody>::new(Agent::new_with_defaults(), Method::PUT, uri)
+}
+
+/// Make a DELETE request.
+///
+/// Run on a use-once [`Agent`].
+#[must_use]
+pub fn delete<T>(uri: T) -> RequestBuilder<WithoutBody>
+where
+    Uri: TryFrom<T>,
+    <Uri as TryFrom<T>>::Error: Into<http::Error>,
+{
+    RequestBuilder::<WithoutBody>::new(Agent::new_with_defaults(), Method::DELETE, uri)
+}
+
+/// Make a HEAD request.
+///
+/// Run on a use-once [`Agent`].
+#[must_use]
+pub fn head<T>(uri: T) -> RequestBuilder<WithoutBody>
+where
+    Uri: TryFrom<T>,
+    <Uri as TryFrom<T>>::Error: Into<http::Error>,
+{
+    RequestBuilder::<WithoutBody>::new(Agent::new_with_defaults(), Method::HEAD, uri)
+}
+
+/// Make an OPTIONS request.
+///
+/// Run on a use-once [`Agent`].
+#[must_use]
+pub fn options<T>(uri: T) -> RequestBuilder<WithoutBody>
+where
+    Uri: TryFrom<T>,
+    <Uri as TryFrom<T>>::Error: Into<http::Error>,
+{
+    RequestBuilder::<WithoutBody>::new(Agent::new_with_defaults(), Method::OPTIONS, uri)
+}
+
+/// Make a CONNECT request.
+///
+/// Run on a use-once [`Agent`].
+#[must_use]
+pub fn connect<T>(uri: T) -> RequestBuilder<WithoutBody>
+where
+    Uri: TryFrom<T>,
+    <Uri as TryFrom<T>>::Error: Into<http::Error>,
+{
+    RequestBuilder::<WithoutBody>::new(Agent::new_with_defaults(), Method::CONNECT, uri)
+}
+
+/// Make a PATCH request.
+///
+/// Run on a use-once [`Agent`].
+#[must_use]
+pub fn patch<T>(uri: T) -> RequestBuilder<WithBody>
+where
+    Uri: TryFrom<T>,
+    <Uri as TryFrom<T>>::Error: Into<http::Error>,
+{
+    RequestBuilder::<WithBody>::new(Agent::new_with_defaults(), Method::PATCH, uri)
+}
+
+/// Make a TRACE request.
+///
+/// Run on a use-once [`Agent`].
+#[must_use]
+pub fn trace<T>(uri: T) -> RequestBuilder<WithoutBody>
+where
+    Uri: TryFrom<T>,
+    <Uri as TryFrom<T>>::Error: Into<http::Error>,
+{
+    RequestBuilder::<WithoutBody>::new(Agent::new_with_defaults(), Method::TRACE, uri)
+}
 
 #[cfg(test)]
 pub(crate) mod test {
@@ -624,7 +712,7 @@ pub(crate) mod test {
 
     pub fn init_test_log() {
         static INIT_LOG: OnceLock<()> = OnceLock::new();
-        INIT_LOG.get_or_init(|| env_logger::init());
+        INIT_LOG.get_or_init(env_logger::init);
     }
 
     #[test]
@@ -930,6 +1018,7 @@ pub(crate) mod test {
 
     #[test]
     fn post_big_body_chunked() {
+        init_test_log();
         // https://github.com/algesten/ureq/issues/879
         let mut data = io::Cursor::new(vec![42; 153_600]);
         post("http://httpbin.org/post")
@@ -1040,12 +1129,85 @@ pub(crate) mod test {
             };
         }
 
-        ensure!(RequestBuilder<WithoutBody>, 400); // 288
+        ensure!(RequestBuilder<WithoutBody>, 400); // 304
         ensure!(Agent, 100); // 32
         ensure!(Config, 400); // 320
         ensure!(ConfigBuilder<AgentScope>, 400); // 320
-        ensure!(Response<Body>, 800); // 760
-        ensure!(Body, 700); // 648
+        ensure!(Response<Body>, 250); // 136
+        ensure!(Body, 50); // 24
+    }
+
+    #[test]
+    #[cfg(feature = "_test")]
+    fn limit_max_response_header_size() {
+        init_test_log();
+        let err = get("http://httpbin.org/get")
+            .config()
+            .max_response_header_size(5)
+            .build()
+            .call()
+            .unwrap_err();
+        assert!(matches!(err, Error::LargeResponseHeader(65, 5)));
+    }
+
+    #[test]
+    #[cfg(feature = "_test")]
+    fn propfind_with_body() {
+        init_test_log();
+
+        // https://github.com/algesten/ureq/issues/1034
+        let request = http::Request::builder()
+            .method("PROPFIND")
+            .uri("https://www.google.com/")
+            .body("Some really cool body")
+            .unwrap();
+
+        let _ = Agent::config_builder()
+            .allow_non_standard_methods(true)
+            .build()
+            .new_agent()
+            .run(request)
+            .unwrap();
+    }
+
+    #[test]
+    #[cfg(feature = "_test")]
+    fn non_standard_method() {
+        init_test_log();
+        let method = Method::from_bytes(b"FNORD").unwrap();
+
+        let req = Request::builder()
+            .method(method)
+            .uri("http://httpbin.org/fnord")
+            .body(())
+            .unwrap();
+
+        let agent = Agent::new_with_defaults();
+
+        let req = agent
+            .configure_request(req)
+            .allow_non_standard_methods(true)
+            .build();
+
+        agent.run(req).unwrap();
+    }
+
+    #[test]
+    #[cfg(feature = "_test")]
+    fn chunk_abort() {
+        init_test_log();
+        let mut res = get("http://my-fine-server/1chunk-abort").call().unwrap();
+        let body = res.body_mut().read_to_string().unwrap();
+        assert_eq!(body, "OK");
+        let mut res = get("http://my-fine-server/2chunk-abort").call().unwrap();
+        let body = res.body_mut().read_to_string().unwrap();
+        assert_eq!(body, "OK");
+        let mut res = get("http://my-fine-server/3chunk-abort").call().unwrap();
+        let body = res.body_mut().read_to_string().unwrap();
+        assert_eq!(body, "OK");
+        let mut res = get("http://my-fine-server/4chunk-abort").call().unwrap();
+        let body = res.body_mut().read_to_string().unwrap();
+        assert_eq!(body, "OK");
     }
 
     // This doesn't need to run, just compile.
@@ -1089,5 +1251,10 @@ pub(crate) mod test {
         let response = post("https://yaz").send(&data).unwrap();
         let owned_reader = response.into_parts().1.into_reader();
         is_sync(owned_reader);
+
+        let err = Error::HostNotFound;
+        is_send(err);
+        let err = Error::HostNotFound;
+        is_sync(err);
     }
 }
